@@ -28,8 +28,9 @@
 
 #if BX_SUPPORT_EVEX
 
-extern float_status_t mxcsr_to_softfloat_status_word(bx_mxcsr_t mxcsr);
+extern softfloat_status_t mxcsr_to_softfloat_status_word(bx_mxcsr_t mxcsr);
 
+#include "softfloat3e/include/softfloat.h"
 #include "simd_int.h"
 #include "simd_pfp.h"
 
@@ -45,10 +46,10 @@ extern float_status_t mxcsr_to_softfloat_status_word(bx_mxcsr_t mxcsr);
       float32 op2 = BX_READ_XMM_REG_LO_DWORD(i->src2());                      \
       float32 op3 = BX_READ_XMM_REG_LO_DWORD(i->src3());                      \
                                                                               \
-      float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);          \
+      softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);      \
       softfloat_status_word_rc_override(status, i);                           \
-      op1 = (func)(op1, op2, op3, status);                                    \
-      check_exceptionsSSE(get_exception_flags(status));                       \
+      op1 = (func)(op1, op2, op3, &status);                                   \
+      check_exceptionsSSE(softfloat_getExceptionFlags(&status));              \
                                                                               \
       BX_WRITE_XMM_REG_LO_DWORD(i->dst(), op1);                               \
     }                                                                         \
@@ -61,10 +62,10 @@ extern float_status_t mxcsr_to_softfloat_status_word(bx_mxcsr_t mxcsr);
     BX_NEXT_INSTR(i);                                                         \
   }
 
-EVEX_FMA_SCALAR_SINGLE(VFMADDSS_MASK_VpsHssWssR, float32_fmadd)
-EVEX_FMA_SCALAR_SINGLE(VFMSUBSS_MASK_VpsHssWssR, float32_fmsub)
-EVEX_FMA_SCALAR_SINGLE(VFNMADDSS_MASK_VpsHssWssR, float32_fnmadd)
-EVEX_FMA_SCALAR_SINGLE(VFNMSUBSS_MASK_VpsHssWssR, float32_fnmsub)
+EVEX_FMA_SCALAR_SINGLE(VFMADDSS_MASK_VpsHssWssR, f32_fmadd)
+EVEX_FMA_SCALAR_SINGLE(VFMSUBSS_MASK_VpsHssWssR, f32_fmsub)
+EVEX_FMA_SCALAR_SINGLE(VFNMADDSS_MASK_VpsHssWssR, f32_fnmadd)
+EVEX_FMA_SCALAR_SINGLE(VFNMSUBSS_MASK_VpsHssWssR, f32_fnmsub)
 
 #define EVEX_FMA_SCALAR_DOUBLE(HANDLER, func)                                 \
   void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i)         \
@@ -74,10 +75,10 @@ EVEX_FMA_SCALAR_SINGLE(VFNMSUBSS_MASK_VpsHssWssR, float32_fnmsub)
       float64 op2 = BX_READ_XMM_REG_LO_QWORD(i->src2());                      \
       float64 op3 = BX_READ_XMM_REG_LO_QWORD(i->src3());                      \
                                                                               \
-      float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);          \
+      softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);      \
       softfloat_status_word_rc_override(status, i);                           \
-      op1 = (func)(op1, op2, op3, status);                                    \
-      check_exceptionsSSE(get_exception_flags(status));                       \
+      op1 = (func)(op1, op2, op3, &status);                                   \
+      check_exceptionsSSE(softfloat_getExceptionFlags(&status));              \
                                                                               \
       BX_WRITE_XMM_REG_LO_QWORD(i->dst(), op1);                               \
     }                                                                         \
@@ -90,9 +91,42 @@ EVEX_FMA_SCALAR_SINGLE(VFNMSUBSS_MASK_VpsHssWssR, float32_fnmsub)
     BX_NEXT_INSTR(i);                                                         \
   }
 
-EVEX_FMA_SCALAR_DOUBLE(VFMADDSD_MASK_VpdHsdWsdR, float64_fmadd)
-EVEX_FMA_SCALAR_DOUBLE(VFMSUBSD_MASK_VpdHsdWsdR, float64_fmsub)
-EVEX_FMA_SCALAR_DOUBLE(VFNMADDSD_MASK_VpdHsdWsdR, float64_fnmadd)
-EVEX_FMA_SCALAR_DOUBLE(VFNMSUBSD_MASK_VpdHsdWsdR, float64_fnmsub)
+EVEX_FMA_SCALAR_DOUBLE(VFMADDSD_MASK_VpdHsdWsdR, f64_fmadd)
+EVEX_FMA_SCALAR_DOUBLE(VFMSUBSD_MASK_VpdHsdWsdR, f64_fmsub)
+EVEX_FMA_SCALAR_DOUBLE(VFNMADDSD_MASK_VpdHsdWsdR, f64_fnmadd)
+EVEX_FMA_SCALAR_DOUBLE(VFNMSUBSD_MASK_VpdHsdWsdR, f64_fnmsub)
+
+///////////////////////////////////
+// AVX-512 FP16 FMA Instructions //
+///////////////////////////////////
+
+#define EVEX_FMA_SCALAR_HALF_PRECISION(HANDLER, func)                         \
+  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i)         \
+  {                                                                           \
+    if (BX_SCALAR_ELEMENT_MASK(i->opmask())) {                                \
+      float16 op1 = BX_READ_XMM_REG_LO_WORD(i->src1());                       \
+      float16 op2 = BX_READ_XMM_REG_LO_WORD(i->src2());                       \
+      float16 op3 = BX_READ_XMM_REG_LO_WORD(i->src3());                       \
+                                                                              \
+      softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);      \
+      softfloat_status_word_rc_override(status, i);                           \
+      op1 = (func)(op1, op2, op3, &status);                                   \
+      check_exceptionsSSE(softfloat_getExceptionFlags(&status));              \
+                                                                              \
+      BX_WRITE_XMM_REG_LO_WORD(i->dst(), op1);                                \
+    }                                                                         \
+    else {                                                                    \
+      if (i->isZeroMasking())                                                 \
+        BX_WRITE_XMM_REG_LO_WORD(i->dst(), 0);                                \
+    }                                                                         \
+                                                                              \
+    BX_CLEAR_AVX_HIGH128(i->dst());                                           \
+    BX_NEXT_INSTR(i);                                                         \
+  }
+
+EVEX_FMA_SCALAR_HALF_PRECISION(VFMADDSH_MASK_VphHshWshR, f16_fmadd)
+EVEX_FMA_SCALAR_HALF_PRECISION(VFMSUBSH_MASK_VphHshWshR, f16_fmsub)
+EVEX_FMA_SCALAR_HALF_PRECISION(VFNMADDSH_MASK_VphHshWshR, f16_fnmadd)
+EVEX_FMA_SCALAR_HALF_PRECISION(VFNMSUBSH_MASK_VphHshWshR, f16_fnmsub)
 
 #endif
