@@ -85,9 +85,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVAPS_MASK_VpsWpsM(bxInstruction_c *i)
     BX_WRITE_AVX_REGZ(i->dst(), reg, len);
   }
   else {
-    for (unsigned n=0; n < len; n++, mask >>= 4)
-      xmm_blendps(&BX_READ_AVX_REG_LANE(i->dst(), n), &reg.vmm128(n), mask);
-
+    simd_blendps(&BX_READ_AVX_REG(i->dst()), &reg, mask, DWORD_ELEMENTS(len));
     BX_CLEAR_AVX_REGZ(i->dst(), len);
   }
 
@@ -113,9 +111,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVAPD_MASK_VpdWpdM(bxInstruction_c *i)
     BX_WRITE_AVX_REGZ(i->dst(), reg, len);
   }
   else {
-    for (unsigned n=0; n < len; n++, mask >>= 2)
-      xmm_blendpd(&BX_READ_AVX_REG_LANE(i->dst(), n), &reg.vmm128(n), mask);
-
+    simd_blendpd(&BX_READ_AVX_REG(i->dst()), &reg, mask, QWORD_ELEMENTS(len));
     BX_CLEAR_AVX_REGZ(i->dst(), len);
   }
 
@@ -171,9 +167,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVDQU8_MASK_VdqWdqM(bxInstruction_c *i)
     BX_WRITE_AVX_REGZ(i->dst(), reg, len);
   }
   else {
-    for (unsigned n=0; n < len; n++, mask >>= 16)
-      xmm_pblendb(&BX_READ_AVX_REG_LANE(i->dst(), n), &reg.vmm128(n), mask);
-
+    simd_pblendb(&BX_READ_AVX_REG(i->dst()), &reg, mask, BYTE_ELEMENTS(len));
     BX_CLEAR_AVX_REGZ(i->dst(), len);
   }
 
@@ -193,9 +187,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVDQU16_MASK_VdqWdqM(bxInstruction_c *i)
     BX_WRITE_AVX_REGZ(i->dst(), reg, len);
   }
   else {
-    for (unsigned n=0; n < len; n++, mask >>= 8)
-      xmm_pblendw(&BX_READ_AVX_REG_LANE(i->dst(), n), &reg.vmm128(n), mask);
-
+    simd_pblendw(&BX_READ_AVX_REG(i->dst()), &reg, mask, WORD_ELEMENTS(len));
     BX_CLEAR_AVX_REGZ(i->dst(), len);
   }
 
@@ -215,9 +207,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVUPS_MASK_VpsWpsM(bxInstruction_c *i)
     BX_WRITE_AVX_REGZ(i->dst(), reg, len);
   }
   else {
-    for (unsigned n=0; n < len; n++, mask >>= 4)
-      xmm_blendps(&BX_READ_AVX_REG_LANE(i->dst(), n), &reg.vmm128(n), mask);
-
+    simd_blendps(&BX_READ_AVX_REG(i->dst()), &reg, mask, DWORD_ELEMENTS(len));
     BX_CLEAR_AVX_REGZ(i->dst(), len);
   }
 
@@ -237,9 +227,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVUPD_MASK_VpdWpdM(bxInstruction_c *i)
     BX_WRITE_AVX_REGZ(i->dst(), reg, len);
   }
   else {
-    for (unsigned n=0; n < len; n++, mask >>= 2)
-      xmm_blendpd(&BX_READ_AVX_REG_LANE(i->dst(), n), &reg.vmm128(n), mask);
-
+    simd_blendpd(&BX_READ_AVX_REG(i->dst()), &reg, mask, QWORD_ELEMENTS(len));
     BX_CLEAR_AVX_REGZ(i->dst(), len);
   }
 
@@ -387,6 +375,109 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVSS_MASK_VssHpsWssR(bxInstruction_c *i)
   BX_NEXT_INSTR(i);
 }
 
+////////////////////
+// Half precision //
+////////////////////
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVW_VshEwR(bxInstruction_c *i)
+{
+  BxPackedXmmRegister op;
+
+  op.xmm64u(0) = (Bit64u) BX_READ_16BIT_REG(i->src());
+  op.xmm64u(1) = 0;
+
+  BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op);
+  BX_NEXT_INSTR(i);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVW_EdVshR(bxInstruction_c *i)
+{
+  BX_WRITE_32BIT_REGZ(i->dst(), BX_READ_XMM_REG_LO_WORD(i->src()));
+  BX_NEXT_INSTR(i);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVSH_VshWshM(bxInstruction_c *i)
+{
+  BxPackedXmmRegister op;
+
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+  op.xmm64u(0) = (Bit64u) read_virtual_word(i->seg(), eaddr);
+  op.xmm64u(1) = 0;
+
+  BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op);
+  BX_NEXT_INSTR(i);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVSH_MASK_VshWshM(bxInstruction_c *i)
+{
+  BxPackedXmmRegister op;
+
+  op.xmm64u(1) = 0;
+
+  if (BX_SCALAR_ELEMENT_MASK(i->opmask())) {
+    bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+    op.xmm64u(0) = (Bit64u) read_virtual_word(i->seg(), eaddr);
+  }
+  else {
+    if (! i->isZeroMasking()) {
+      op.xmm64u(0) = (Bit64u) BX_READ_XMM_REG_LO_WORD(i->dst());
+    }
+    else {
+      op.xmm64u(0) = 0;
+    }
+  }
+
+  BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op);
+  BX_NEXT_INSTR(i);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVSH_VshHphWshR(bxInstruction_c *i)
+{
+  BxPackedXmmRegister op = BX_READ_XMM_REG(i->src1());
+  op.xmm16u(0) = BX_READ_XMM_REG_LO_WORD(i->src2());
+  BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op);
+
+  BX_NEXT_INSTR(i);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVSH_MASK_VshHphWshR(bxInstruction_c *i)
+{
+  BxPackedXmmRegister op = BX_READ_XMM_REG(i->src1());
+
+  if (BX_SCALAR_ELEMENT_MASK(i->opmask())) {
+    op.xmm16u(0) = BX_READ_XMM_REG_LO_WORD(i->src2());
+  }
+  else {
+    if (! i->isZeroMasking()) {
+      op.xmm16u(0) = BX_READ_XMM_REG_LO_WORD(i->dst());
+    }
+    else {
+      op.xmm16u(0) = 0;
+    }
+  }
+
+  BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op);
+
+  BX_NEXT_INSTR(i);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVSH_WshVshM(bxInstruction_c *i)
+{
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+  write_virtual_word(i->seg(), eaddr, BX_READ_XMM_REG_LO_WORD(i->src()));
+  BX_NEXT_INSTR(i);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMOVSH_MASK_WshVshM(bxInstruction_c *i)
+{
+  if (BX_SCALAR_ELEMENT_MASK(i->opmask())) {
+    bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+    write_virtual_word(i->seg(), eaddr, BX_READ_XMM_REG_LO_WORD(i->src()));
+  }
+
+  BX_NEXT_INSTR(i);
+}
+
 ////////////////////////////////////
 // masked store with down convert //
 ////////////////////////////////////
