@@ -28,13 +28,13 @@
 
 #if BX_SUPPORT_AVX
 
-extern float_status_t mxcsr_to_softfloat_status_word(bx_mxcsr_t mxcsr);
-extern void mxcsr_to_softfloat_status_word_imm_override(float_status_t &status, Bit8u immb);
+extern softfloat_status_t mxcsr_to_softfloat_status_word(bx_mxcsr_t mxcsr);
+extern void mxcsr_to_softfloat_status_word_imm_override(softfloat_status_t &status, Bit8u immb);
 
 extern float32 approximate_rsqrt(float32 op);
 extern float32 approximate_rcp(float32 op);
 
-#include "fpu/softfloat-compare.h"
+#include "softfloat3e/include/softfloat-compare.h"
 #include "simd_pfp.h"
 #include "simd_int.h"
 
@@ -60,86 +60,124 @@ void BX_CPU_C::print_state_AVX(void)
 
 /* Comparison predicate for VCMPSS/VCMPPS instructions */
 float32_compare_method avx_compare32[32] = {
-  float32_eq_ordered_quiet,
-  float32_lt_ordered_signalling,
-  float32_le_ordered_signalling,
-  float32_unordered_quiet,
-  float32_neq_unordered_quiet,
-  float32_nlt_unordered_signalling,
-  float32_nle_unordered_signalling,
-  float32_ordered_quiet,
-  float32_eq_unordered_quiet,
-  float32_nge_unordered_signalling,
-  float32_ngt_unordered_signalling,
-  float32_false_quiet,
-  float32_neq_ordered_quiet,
-  float32_ge_ordered_signalling,
-  float32_gt_ordered_signalling,
-  float32_true_quiet,
-  float32_eq_ordered_signalling,
-  float32_lt_ordered_quiet,
-  float32_le_ordered_quiet,
-  float32_unordered_signalling,
-  float32_neq_unordered_signalling,
-  float32_nlt_unordered_quiet,
-  float32_nle_unordered_quiet,
-  float32_ordered_signalling,
-  float32_eq_unordered_signalling,
-  float32_nge_unordered_quiet,
-  float32_ngt_unordered_quiet,
-  float32_false_signalling,
-  float32_neq_ordered_signalling,
-  float32_ge_ordered_quiet,
-  float32_gt_ordered_quiet,
-  float32_true_signalling
+  f32_eq_ordered_quiet,
+  f32_lt_ordered_signalling,
+  f32_le_ordered_signalling,
+  f32_unordered_quiet,
+  f32_neq_unordered_quiet,
+  f32_nlt_unordered_signalling,
+  f32_nle_unordered_signalling,
+  f32_ordered_quiet,
+  f32_eq_unordered_quiet,
+  f32_nge_unordered_signalling,
+  f32_ngt_unordered_signalling,
+  f32_false_quiet,
+  f32_neq_ordered_quiet,
+  f32_ge_ordered_signalling,
+  f32_gt_ordered_signalling,
+  f32_true_quiet,
+  f32_eq_ordered_signalling,
+  f32_lt_ordered_quiet,
+  f32_le_ordered_quiet,
+  f32_unordered_signalling,
+  f32_neq_unordered_signalling,
+  f32_nlt_unordered_quiet,
+  f32_nle_unordered_quiet,
+  f32_ordered_signalling,
+  f32_eq_unordered_signalling,
+  f32_nge_unordered_quiet,
+  f32_ngt_unordered_quiet,
+  f32_false_signalling,
+  f32_neq_ordered_signalling,
+  f32_ge_ordered_quiet,
+  f32_gt_ordered_quiet,
+  f32_true_signalling
 };
 
 /* Comparison predicate for VCMPSD/VCMPPD instructions */
 float64_compare_method avx_compare64[32] = {
-  float64_eq_ordered_quiet,
-  float64_lt_ordered_signalling,
-  float64_le_ordered_signalling,
-  float64_unordered_quiet,
-  float64_neq_unordered_quiet,
-  float64_nlt_unordered_signalling,
-  float64_nle_unordered_signalling,
-  float64_ordered_quiet,
-  float64_eq_unordered_quiet,
-  float64_nge_unordered_signalling,
-  float64_ngt_unordered_signalling,
-  float64_false_quiet,
-  float64_neq_ordered_quiet,
-  float64_ge_ordered_signalling,
-  float64_gt_ordered_signalling,
-  float64_true_quiet,
-  float64_eq_ordered_signalling,
-  float64_lt_ordered_quiet,
-  float64_le_ordered_quiet,
-  float64_unordered_signalling,
-  float64_neq_unordered_signalling,
-  float64_nlt_unordered_quiet,
-  float64_nle_unordered_quiet,
-  float64_ordered_signalling,
-  float64_eq_unordered_signalling,
-  float64_nge_unordered_quiet,
-  float64_ngt_unordered_quiet,
-  float64_false_signalling,
-  float64_neq_ordered_signalling,
-  float64_ge_ordered_quiet,
-  float64_gt_ordered_quiet,
-  float64_true_signalling
+  f64_eq_ordered_quiet,
+  f64_lt_ordered_signalling,
+  f64_le_ordered_signalling,
+  f64_unordered_quiet,
+  f64_neq_unordered_quiet,
+  f64_nlt_unordered_signalling,
+  f64_nle_unordered_signalling,
+  f64_ordered_quiet,
+  f64_eq_unordered_quiet,
+  f64_nge_unordered_signalling,
+  f64_ngt_unordered_signalling,
+  f64_false_quiet,
+  f64_neq_ordered_quiet,
+  f64_ge_ordered_signalling,
+  f64_gt_ordered_signalling,
+  f64_true_quiet,
+  f64_eq_ordered_signalling,
+  f64_lt_ordered_quiet,
+  f64_le_ordered_quiet,
+  f64_unordered_signalling,
+  f64_neq_unordered_signalling,
+  f64_nlt_unordered_quiet,
+  f64_nle_unordered_quiet,
+  f64_ordered_signalling,
+  f64_eq_unordered_signalling,
+  f64_nge_unordered_quiet,
+  f64_ngt_unordered_quiet,
+  f64_false_signalling,
+  f64_neq_ordered_signalling,
+  f64_ge_ordered_quiet,
+  f64_gt_ordered_quiet,
+  f64_true_signalling
 };
 
+#if BX_SUPPORT_EVEX
+/* Comparison predicate for VCMPSH/VCMPPH instructions */
+float16_compare_method avx_compare16[32] = {
+  f16_eq_ordered_quiet,
+  f16_lt_ordered_signalling,
+  f16_le_ordered_signalling,
+  f16_unordered_quiet,
+  f16_neq_unordered_quiet,
+  f16_nlt_unordered_signalling,
+  f16_nle_unordered_signalling,
+  f16_ordered_quiet,
+  f16_eq_unordered_quiet,
+  f16_nge_unordered_signalling,
+  f16_ngt_unordered_signalling,
+  f16_false_quiet,
+  f16_neq_ordered_quiet,
+  f16_ge_ordered_signalling,
+  f16_gt_ordered_signalling,
+  f16_true_quiet,
+  f16_eq_ordered_signalling,
+  f16_lt_ordered_quiet,
+  f16_le_ordered_quiet,
+  f16_unordered_signalling,
+  f16_neq_unordered_signalling,
+  f16_nlt_unordered_quiet,
+  f16_nle_unordered_quiet,
+  f16_ordered_signalling,
+  f16_eq_unordered_signalling,
+  f16_nge_unordered_quiet,
+  f16_ngt_unordered_quiet,
+  f16_false_signalling,
+  f16_neq_ordered_signalling,
+  f16_ge_ordered_quiet,
+  f16_gt_ordered_quiet,
+  f16_true_signalling
+};
+#endif
+
 /* Opcode: VEX.NDS.F3.0F 51 (VEX.W ignore, VEX.L ignore) */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSQRTSS_VssHpsWssR(bxInstruction_c *i)
 {
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
   float32 op2 = BX_READ_XMM_REG_LO_DWORD(i->src2());
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   softfloat_status_word_rc_override(status, i);
-  op1.xmm32u(0) = float32_sqrt(op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
+  op1.xmm32u(0) = f32_sqrt(op2, &status);
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
 
   BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
   BX_NEXT_INSTR(i);
@@ -151,10 +189,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSQRTSD_VsdHpdWsdR(bxInstruction_c *i)
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
   float64 op2 = BX_READ_XMM_REG_LO_QWORD(i->src2());
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   softfloat_status_word_rc_override(status, i);
-  op1.xmm64u(0) = float64_sqrt(op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
+  op1.xmm64u(0) = f64_sqrt(op2, &status);
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
 
   BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
   BX_NEXT_INSTR(i);
@@ -218,25 +256,25 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VRCPSS_VssHpsWssR(bxInstruction_c *i)
     BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());                                   \
     float32 op2 = BX_READ_XMM_REG_LO_DWORD(i->src2());                                      \
                                                                                             \
-    float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);                          \
+    softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);                      \
     softfloat_status_word_rc_override(status, i);                                           \
                                                                                             \
-    op1.xmm32u(0) = (func)(op1.xmm32u(0), op2, status);                                     \
+    op1.xmm32u(0) = (func)(op1.xmm32u(0), op2, &status);                                    \
                                                                                             \
-    check_exceptionsSSE(get_exception_flags(status));                                       \
+    check_exceptionsSSE(softfloat_getExceptionFlags(&status));                              \
     BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);                                             \
                                                                                             \
     BX_NEXT_INSTR(i);                                                                       \
   }
 
-AVX_SCALAR_SINGLE_FP(VADDSS_VssHpsWssR, float32_add);
-AVX_SCALAR_SINGLE_FP(VSUBSS_VssHpsWssR, float32_sub);
-AVX_SCALAR_SINGLE_FP(VMULSS_VssHpsWssR, float32_mul);
-AVX_SCALAR_SINGLE_FP(VDIVSS_VssHpsWssR, float32_div);
-AVX_SCALAR_SINGLE_FP(VMINSS_VssHpsWssR, float32_min);
-AVX_SCALAR_SINGLE_FP(VMAXSS_VssHpsWssR, float32_max);
+AVX_SCALAR_SINGLE_FP(VADDSS_VssHpsWssR, f32_add);
+AVX_SCALAR_SINGLE_FP(VSUBSS_VssHpsWssR, f32_sub);
+AVX_SCALAR_SINGLE_FP(VMULSS_VssHpsWssR, f32_mul);
+AVX_SCALAR_SINGLE_FP(VDIVSS_VssHpsWssR, f32_div);
+AVX_SCALAR_SINGLE_FP(VMINSS_VssHpsWssR, f32_min);
+AVX_SCALAR_SINGLE_FP(VMAXSS_VssHpsWssR, f32_max);
 #if BX_SUPPORT_EVEX
-AVX_SCALAR_SINGLE_FP(VSCALEFSS_VssHpsWssR, float32_scalef);
+AVX_SCALAR_SINGLE_FP(VSCALEFSS_VssHpsWssR, f32_scalef);
 #endif
 
 #define AVX_SCALAR_DOUBLE_FP(HANDLER, func)                                                 \
@@ -245,25 +283,25 @@ AVX_SCALAR_SINGLE_FP(VSCALEFSS_VssHpsWssR, float32_scalef);
     BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());                                   \
     float64 op2 = BX_READ_XMM_REG_LO_QWORD(i->src2());                                      \
                                                                                             \
-    float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);                          \
+    softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);                      \
     softfloat_status_word_rc_override(status, i);                                           \
                                                                                             \
-    op1.xmm64u(0) = (func)(op1.xmm64u(0), op2, status);                                     \
+    op1.xmm64u(0) = (func)(op1.xmm64u(0), op2, &status);                                    \
                                                                                             \
-    check_exceptionsSSE(get_exception_flags(status));                                       \
+    check_exceptionsSSE(softfloat_getExceptionFlags(&status));                              \
     BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);                                             \
                                                                                             \
     BX_NEXT_INSTR(i);                                                                       \
   }
 
-AVX_SCALAR_DOUBLE_FP(VADDSD_VsdHpdWsdR, float64_add);
-AVX_SCALAR_DOUBLE_FP(VSUBSD_VsdHpdWsdR, float64_sub);
-AVX_SCALAR_DOUBLE_FP(VMULSD_VsdHpdWsdR, float64_mul);
-AVX_SCALAR_DOUBLE_FP(VDIVSD_VsdHpdWsdR, float64_div);
-AVX_SCALAR_DOUBLE_FP(VMINSD_VsdHpdWsdR, float64_min);
-AVX_SCALAR_DOUBLE_FP(VMAXSD_VsdHpdWsdR, float64_max);
+AVX_SCALAR_DOUBLE_FP(VADDSD_VsdHpdWsdR, f64_add);
+AVX_SCALAR_DOUBLE_FP(VSUBSD_VsdHpdWsdR, f64_sub);
+AVX_SCALAR_DOUBLE_FP(VMULSD_VsdHpdWsdR, f64_mul);
+AVX_SCALAR_DOUBLE_FP(VDIVSD_VsdHpdWsdR, f64_div);
+AVX_SCALAR_DOUBLE_FP(VMINSD_VsdHpdWsdR, f64_min);
+AVX_SCALAR_DOUBLE_FP(VMAXSD_VsdHpdWsdR, f64_max);
 #if BX_SUPPORT_EVEX
-AVX_SCALAR_DOUBLE_FP(VSCALEFSD_VsdHpdWsdR, float64_scalef);
+AVX_SCALAR_DOUBLE_FP(VSCALEFSD_VsdHpdWsdR, f64_scalef);
 #endif
 
 /* Opcode: VEX.NDS.0F C2 (VEX.W ignore) */
@@ -272,14 +310,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VCMPPS_VpsHpsWpsIbR(bxInstruction_c *i)
   BxPackedYmmRegister op1 = BX_READ_YMM_REG(i->src1()), op2 = BX_READ_YMM_REG(i->src2());
   unsigned len = i->getVL();
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   int ib = i->Ib() & 0x1F;
 
   for (unsigned n=0; n < DWORD_ELEMENTS(len); n++) {
-    op1.ymm32u(n) = avx_compare32[ib](op1.ymm32u(n), op2.ymm32u(n), status) ? 0xFFFFFFFF : 0;
+    op1.ymm32u(n) = avx_compare32[ib](op1.ymm32u(n), op2.ymm32u(n), &status) ? 0xFFFFFFFF : 0;
   }
 
-  check_exceptionsSSE(get_exception_flags(status));
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
   BX_WRITE_YMM_REGZ_VLEN(i->dst(), op1, len);
 
   BX_NEXT_INSTR(i);
@@ -291,15 +329,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VCMPPD_VpdHpdWpdIbR(bxInstruction_c *i)
   BxPackedYmmRegister op1 = BX_READ_YMM_REG(i->src1()), op2 = BX_READ_YMM_REG(i->src2());
   unsigned len = i->getVL();
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   int ib = i->Ib() & 0x1F;
 
   for (unsigned n=0; n < QWORD_ELEMENTS(len); n++) {
-    op1.ymm64u(n) = avx_compare64[ib](op1.ymm64u(n), op2.ymm64u(n), status) ?
+    op1.ymm64u(n) = avx_compare64[ib](op1.ymm64u(n), op2.ymm64u(n), &status) ?
        BX_CONST64(0xFFFFFFFFFFFFFFFF) : 0;
   }
 
-  check_exceptionsSSE(get_exception_flags(status));
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
   BX_WRITE_YMM_REGZ_VLEN(i->dst(), op1, len);
 
   BX_NEXT_INSTR(i);
@@ -311,16 +349,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VCMPSD_VsdHpdWsdIbR(bxInstruction_c *i)
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
   float64 op2 = BX_READ_XMM_REG_LO_QWORD(i->src2());
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   int ib = i->Ib() & 0x1F;
 
-  if(avx_compare64[ib](op1.xmm64u(0), op2, status)) {
+  if(avx_compare64[ib](op1.xmm64u(0), op2, &status)) {
     op1.xmm64u(0) = BX_CONST64(0xFFFFFFFFFFFFFFFF);
   } else {
     op1.xmm64u(0) = 0;
   }
 
-  check_exceptionsSSE(get_exception_flags(status));
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
   BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
 
   BX_NEXT_INSTR(i);
@@ -332,16 +370,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VCMPSS_VssHpsWssIbR(bxInstruction_c *i)
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
   float32 op2 = BX_READ_XMM_REG_LO_DWORD(i->src2());
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   int ib = i->Ib() & 0x1F;
 
-  if(avx_compare32[ib](op1.xmm32u(0), op2, status)) {
+  if(avx_compare32[ib](op1.xmm32u(0), op2, &status)) {
     op1.xmm32u(0) = 0xFFFFFFFF;
   } else {
     op1.xmm32u(0) = 0;
   }
 
-  check_exceptionsSSE(get_exception_flags(status));
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
   BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
 
   BX_NEXT_INSTR(i);
@@ -395,14 +433,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VROUNDPS_VpsWpsIbR(bxInstruction_c *i)
   BxPackedYmmRegister op = BX_READ_YMM_REG(i->src());
   unsigned len = i->getVL();
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   mxcsr_to_softfloat_status_word_imm_override(status, i->Ib());
 
   for(unsigned n=0; n < DWORD_ELEMENTS(len); n++) {
-    op.ymm32u(n) = float32_round_to_int(op.ymm32u(n), status);
+    op.ymm32u(n) = f32_roundToInt(op.ymm32u(n), &status);
   }
 
-  check_exceptionsSSE(get_exception_flags(status));
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
 
   BX_WRITE_YMM_REGZ_VLEN(i->dst(), op, len);
 
@@ -415,14 +453,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VROUNDPD_VpdWpdIbR(bxInstruction_c *i)
   BxPackedYmmRegister op = BX_READ_YMM_REG(i->src());
   unsigned len = i->getVL();
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   mxcsr_to_softfloat_status_word_imm_override(status, i->Ib());
 
   for(unsigned n=0; n < QWORD_ELEMENTS(len); n++) {
-    op.ymm64u(n) = float64_round_to_int(op.ymm64u(n), status);
+    op.ymm64u(n) = f64_roundToInt(op.ymm64u(n), &status);
   }
 
-  check_exceptionsSSE(get_exception_flags(status));
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
 
   BX_WRITE_YMM_REGZ_VLEN(i->dst(), op, len);
 
@@ -435,12 +473,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VROUNDSS_VssHpsWssIbR(bxInstruction_c *i)
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
   float32 op2 = BX_READ_XMM_REG_LO_DWORD(i->src2());
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   mxcsr_to_softfloat_status_word_imm_override(status, i->Ib());
 
-  op1.xmm32u(0) = float32_round_to_int(op2, status);
+  op1.xmm32u(0) = f32_roundToInt(op2, &status);
 
-  check_exceptionsSSE(get_exception_flags(status));
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
 
   BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
 
@@ -453,12 +491,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VROUNDSD_VsdHpdWsdIbR(bxInstruction_c *i)
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
   float64 op2 = BX_READ_XMM_REG_LO_QWORD(i->src2());
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   mxcsr_to_softfloat_status_word_imm_override(status, i->Ib());
 
-  op1.xmm64u(0) = float64_round_to_int(op2, status);
+  op1.xmm64u(0) = f64_roundToInt(op2, &status);
 
-  check_exceptionsSSE(get_exception_flags(status));
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
 
   BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
 
@@ -472,7 +510,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VDPPS_VpsHpsWpsIbR(bxInstruction_c *i)
   unsigned len = i->getVL();
   Bit8u mask = i->Ib();
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
 
   for (unsigned n=0; n < len; n++) {
 
@@ -495,7 +533,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VDPPS_VpsHpsWpsIbR(bxInstruction_c *i)
     xmm_addps_mask(&op2.ymm128(n), &op1.ymm128(n), status, mask);
   }
 
-  check_exceptionsSSE(get_exception_flags(status));
+  check_exceptionsSSE(softfloat_getExceptionFlags(&status));
 
   BX_WRITE_YMM_REGZ_VLEN(i->dst(), op2, len);
 
