@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bmi64.cc 14086 2021-01-30 08:35:35Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2011-2018 Stanislav Shwartsman
@@ -25,7 +25,7 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-#if BX_SUPPORT_X86_64 
+#if BX_SUPPORT_X86_64
 
 #if BX_SUPPORT_AVX
 
@@ -116,7 +116,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RORX_GqEqIbR(bxInstruction_c *i)
 
   unsigned count = i->Ib() & 0x3f;
   if (count) {
-    op1_64 = (op1_64 >> count) | (op1_64 << (64 - count));
+    op1_64 = ror64(op1_64, count);
   }
 
   BX_WRITE_64BIT_REG(i->dst(), op1_64);
@@ -183,7 +183,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BZHI_GqEqBqR(bxInstruction_c *i)
   unsigned control = BX_READ_8BIT_REGL(i->src2());
   bool tmpCF = 0;
   Bit64u op1_64 = BX_READ_64BIT_REG(i->src1());
-  
+
   if (control < 64) {
     Bit64u mask = (BX_CONST64(1) << control) - 1;
     op1_64 &= mask;
@@ -236,7 +236,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::PDEP_GqBqEqR(bxInstruction_c *i)
     }
     wr_mask <<= 1;
   }
-  
+
   BX_WRITE_64BIT_REG(i->dst(), result_64);
 
   BX_NEXT_INSTR(i);
@@ -266,8 +266,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADOX_GqEqR(bxInstruction_c *i)
 
   BX_WRITE_64BIT_REG(i->dst(), sum_64);
 
-  Bit64u overflow = GET_ADD_OVERFLOW(op1_64, op2_64, sum_64, BX_CONST64(0x8000000000000000));
-  set_OF(!!overflow);
+  Bit64u carry_out = ADD_COUT_VEC(op1_64, op2_64, sum_64);
+  set_OF(carry_out >> 63);
 
   BX_NEXT_INSTR(i);
 }
