@@ -47,9 +47,6 @@ public:
 private:
   void get_std_cpuid_leaf_0(cpuid_function_t *leaf) const;
   void get_std_cpuid_leaf_1(cpuid_function_t *leaf) const;
-#if BX_SUPPORT_MONITOR_MWAIT
-  void get_std_cpuid_leaf_5(cpuid_function_t *leaf) const;
-#endif
 
   void get_ext_cpuid_leaf_0(cpuid_function_t *leaf) const;
   void get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const;
