@@ -58,9 +58,6 @@ sapphire_rapids_t::sapphire_rapids_t(BX_CPU_C *cpu):
   enable_cpu_extension(BX_ISA_PSE);
   enable_cpu_extension(BX_ISA_PAE);
   enable_cpu_extension(BX_ISA_PGE);
-#if BX_PHY_ADDRESS_LONG
-  enable_cpu_extension(BX_ISA_PSE36);
-#endif
   enable_cpu_extension(BX_ISA_MTRR);
   enable_cpu_extension(BX_ISA_PAT);
   enable_cpu_extension(BX_ISA_XAPIC);
@@ -123,6 +120,7 @@ sapphire_rapids_t::sapphire_rapids_t(BX_CPU_C *cpu):
   enable_cpu_extension(BX_ISA_AVX512_BITALG);
   enable_cpu_extension(BX_ISA_AVX512_VPOPCNTDQ);
   enable_cpu_extension(BX_ISA_AVX512_BF16);
+  enable_cpu_extension(BX_ISA_AVX512_FP16);
 #endif
 #if BX_SUPPORT_AMX
   enable_cpu_extension(BX_ISA_AMX);
@@ -131,7 +129,7 @@ sapphire_rapids_t::sapphire_rapids_t(BX_CPU_C *cpu):
 #endif
   enable_cpu_extension(BX_ISA_CLFLUSHOPT);
   enable_cpu_extension(BX_ISA_CLWB);
-  enable_cpu_extension(BX_ISA_CLDEMOTE);
+//enable_cpu_extension(BX_ISA_CLDEMOTE);
   enable_cpu_extension(BX_ISA_XSAVEC);
   enable_cpu_extension(BX_ISA_XSAVES);
 #if BX_SUPPORT_PKEYS
@@ -206,7 +204,7 @@ void sapphire_rapids_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpui
     get_std_cpuid_leaf_4(subfunction, leaf);
     return;
   case 0x00000005: // CPUID leaf 0x00000005 - MONITOR/MWAIT leaf
-    get_std_cpuid_leaf_5(leaf);
+    get_std_cpuid_monitor_mwait_leaf(leaf, 0x00001020);
     return;
   case 0x00000006: // CPUID leaf 0x00000006 - Thermal and Power Management Leaf
     get_leaf(leaf, 0x0045cef7, 0x00000002, 0x00000009, 0x00000000);
@@ -255,6 +253,8 @@ void sapphire_rapids_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpui
   case 0x0000001A: // CPUID leaf 0x0000001A - native Model ID Enumeration leaf (for Hybrid)
   case 0x0000001B: // PCONFIG Information
   case 0x0000001C: // CPUID leaf 0x0000001C - Last Branch Record (Architectural LBR) leaf
+    get_reserved_leaf(leaf);
+    return;
 #if BX_SUPPORT_AMX
   case 0x0000001D: // AMX
     get_std_cpuid_amx_palette_info_leaf(subfunction, leaf);
@@ -462,36 +462,8 @@ void sapphire_rapids_t::get_std_cpuid_leaf_4(Bit32u subfunction, cpuid_function_
   }
 }
 
-// leaf 0x00000005 - MONITOR/MWAIT Leaf //
-void sapphire_rapids_t::get_std_cpuid_leaf_5(cpuid_function_t *leaf) const
-{
-  // CPUID function 0x00000005 - MONITOR/MWAIT Leaf
-
-#if BX_SUPPORT_MONITOR_MWAIT
-  // EAX - Smallest monitor-line size in bytes
-  // EBX - Largest  monitor-line size in bytes
-  // ECX -
-  //   [31:2] - reserved
-  //    [1:1] - exit MWAIT even with EFLAGS.IF = 0
-  //    [0:0] - MONITOR/MWAIT extensions are supported
-  // EDX -
-  //  [03-00] - number of C0 sub C-states supported using MWAIT
-  //  [07-04] - number of C1 sub C-states supported using MWAIT
-  //  [11-08] - number of C2 sub C-states supported using MWAIT
-  //  [15-12] - number of C3 sub C-states supported using MWAIT
-  //  [19-16] - number of C4 sub C-states supported using MWAIT
-  //  [31-20] - reserved (MBZ)
-  leaf->eax = CACHE_LINE_SIZE;
-  leaf->ebx = CACHE_LINE_SIZE;
-  leaf->ecx = 3;
-  leaf->edx = 0x00001020;
-#else
-  leaf->eax = 0;
-  leaf->ebx = 0;
-  leaf->ecx = 0;
-  leaf->edx = 0;
-#endif
-}
+// leaf 0x00000005 MONITOR/MWAIT Leaf                //
+// leaf 0x00000006 Thermal and Power Management Leaf //
 
 // leaf 0x00000007 //
 void sapphire_rapids_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_function_t *leaf) const
@@ -547,7 +519,7 @@ void sapphire_rapids_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_function_
     // * [10:10] VPCLMULQDQ instruction support
     // * [11:11] AVX512 VNNI instructions support
     // * [12:12] AVX512 BITALG instructions support
-    // * [13:13] TME_EN: indicates support for MSRs: IA32_TME_CAPABILITY, IA32_TME_ACTIVATE, IA32_TME_EXCLUDE_MASK, and IA32_TME_EXCLUDE_BASE
+    // ! [13:13] TME_EN: indicates support for MSRs: IA32_TME_CAPABILITY, IA32_TME_ACTIVATE, IA32_TME_EXCLUDE_MASK, and IA32_TME_EXCLUDE_BASE
     // * [14:14] AVX512 VPOPCNTDQ: AVX512 VPOPCNTD/VPOPCNTQ instructions
     //   [15:15] reserved
     // * [16:16] LA57: LA57 and 5-level paging
@@ -641,6 +613,16 @@ void sapphire_rapids_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_function_
     leaf->ebx = 0;
     leaf->ecx = 0;
     leaf->edx = 0;
+    // * [00:00] PSFD: IA32_SPEC_CTRL MSR is supported
+    // * [01:01] IPRED_CTRL: bits 3 (IPRED_DIS_USER) and 4 (IPRED_DIS_SUP) of the IA32_SPEC_CTRL MSR are supported
+    // * [02:02] RRSBA_CTRL: bits 5 (RRSBA_USER) and 6 (RRSBA_SUP) of the IA32_SPEC_CTRL MSR are supported
+    //   [03:03] DDPD_U: bit 8 (Data Dep Prefetcher) of the IA32_SPEC_CTRL MSR are supported
+    // * [04:04] BHI_CTRL: bit 10 (BHI_DIS_S) of the IA32_SPEC_CTRL MSR are supported
+    //   [05:05] MCDT_NO: do not exhibit MXCSR Configuration Dependent Timing (MCDT) behavior and do not need to be 
+    //                    mitigated to avoid data-dep behavior for certain instructions
+    //   [06:06] support for UC-lock disable feature (UC lock causes #AC)
+    //   [07:07] MONITOR_MITG_NO: MONITOR/UMONITOR instructions are not affected by performance or power issues due to 
+    //                            MONITOR/UMONITOR instructions exceeding the capacity of an internal monitor tracking table
     if (is_cpu_extension_supported(BX_ISA_SCA_MITIGATIONS))
       leaf->edx = 0x17;
     break;
@@ -775,7 +757,7 @@ void sapphire_rapids_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [12:12] SKINIT support
   //   [13:13] WDT: Watchdog timer support
   //   [31:14] reserved
-  leaf->ecx = get_ext_cpuid_leaf_1_ecx_intel(BX_CPUID_EXT1_ECX_PREFETCHW);
+  leaf->ecx = get_ext_cpuid_leaf_1_ecx(BX_CPUID_EXT1_ECX_PREFETCHW);
 
   // EDX:
   //    [10:0] Reserved for Intel
