@@ -28,59 +28,60 @@
 
 #if BX_SUPPORT_FPU
 
-float_status_t i387cw_to_softfloat_status_word(Bit16u control_word)
+#include "softfloat3e/include/softfloat.h"
+
+softfloat_status_t i387cw_to_softfloat_status_word(Bit16u control_word)
 {
-  float_status_t status;
+  softfloat_status_t status;
 
   int precision = control_word & FPU_CW_PC;
 
   switch(precision)
   {
      case FPU_PR_32_BITS:
-       status.float_rounding_precision = 32;
+       status.extF80_roundingPrecision = 32;
        break;
      case FPU_PR_64_BITS:
-       status.float_rounding_precision = 64;
+       status.extF80_roundingPrecision = 64;
        break;
      case FPU_PR_80_BITS:
-       status.float_rounding_precision = 80;
+       status.extF80_roundingPrecision = 80;
        break;
      default:
     /* With the precision control bits set to 01 "(reserved)", a
        real CPU behaves as if the precision control bits were
        set to 11 "80 bits" */
-       status.float_rounding_precision = 80;
+       status.extF80_roundingPrecision = 80;
   }
 
-  status.float_exception_flags = 0; // clear exceptions before execution
-  status.float_nan_handling_mode = float_first_operand_nan;
-  status.float_rounding_mode = (control_word & FPU_CW_RC) >> 10;
-  status.flush_underflow_to_zero = 0;
-  status.float_suppress_exception = 0;
-  status.float_exception_masks = control_word & FPU_CW_Exceptions_Mask;
-  status.denormals_are_zeros = 0;
+  status.softfloat_exceptionFlags = 0; // clear exceptions before execution
+  status.softfloat_roundingMode = (control_word & FPU_CW_RC) >> 10;
+  status.softfloat_flush_underflow_to_zero = 0;
+  status.softfloat_suppressException = 0;
+  status.softfloat_exceptionMasks = control_word & FPU_CW_Exceptions_Mask;
+  status.softfloat_denormals_are_zeros = 0;
 
   return status;
 }
 
-#include "softfloatx80.h"
+#include "fpu_trans.h"
 
-floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float32 b32, int bIsNaN, float_status_t &status)
+floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float32 b32, int bIsNaN, softfloat_status_t &status)
 {
-    int aIsSignalingNaN = floatx80_is_signaling_nan(a);
-    int bIsSignalingNaN = float32_is_signaling_nan(b32);
+    int aIsSignalingNaN = extF80_isSignalingNaN(a);
+    int bIsSignalingNaN = f32_isSignalingNaN(b32);
 
     if (aIsSignalingNaN | bIsSignalingNaN)
-        float_raise(status, float_flag_invalid);
+        softfloat_raiseFlags(&status, softfloat_flag_invalid);
 
     // propagate QNaN to SNaN
-    a = propagateFloatx80NaN(a, status);
+    a = softfloat_propagateNaNExtF80UI(a.signExp, a.signif, 0, 0, &status);
 
     if (aIsNaN & !bIsNaN) return a;
 
     // float32 is NaN so conversion will propagate SNaN to QNaN and raise
     // appropriate exception flags
-    floatx80 b = float32_to_floatx80(b32, status);
+    floatx80 b = f32_to_extF80(b32, &status);
 
     if (aIsSignalingNaN) {
         if (bIsSignalingNaN) goto returnLargerSignificand;
@@ -89,24 +90,24 @@ floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float32 b32, int bIsNaN, float_s
     else if (aIsNaN) {
         if (bIsSignalingNaN) return a;
  returnLargerSignificand:
-        if (a.fraction < b.fraction) return b;
-        if (b.fraction < a.fraction) return a;
-        return (a.exp < b.exp) ? a : b;
+        if (a.signif < b.signif) return b;
+        if (b.signif < a.signif) return a;
+        return (a.signExp < b.signExp) ? a : b;
     }
     else {
         return b;
     }
 }
 
-bool FPU_handle_NaN(floatx80 a, float32 b, floatx80 &r, float_status_t &status)
+bool FPU_handle_NaN(floatx80 a, float32 b, floatx80 &r, softfloat_status_t &status)
 {
-  if (floatx80_is_unsupported(a)) {
-     float_raise(status, float_flag_invalid);
+  if (extF80_isUnsupported(a)) {
+     softfloat_raiseFlags(&status, softfloat_flag_invalid);
      r = floatx80_default_nan;
      return true;
   }
 
-  int aIsNaN = floatx80_is_nan(a), bIsNaN = float32_is_nan(b);
+  int aIsNaN = extF80_isNaN(a), bIsNaN = f32_isNaN(b);
   if (aIsNaN | bIsNaN) {
      r = FPU_handle_NaN(a, aIsNaN, b, bIsNaN, status);
      return true;
@@ -114,22 +115,22 @@ bool FPU_handle_NaN(floatx80 a, float32 b, floatx80 &r, float_status_t &status)
   return false;
 }
 
-floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float64 b64, int bIsNaN, float_status_t &status)
+floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float64 b64, int bIsNaN, softfloat_status_t &status)
 {
-    int aIsSignalingNaN = floatx80_is_signaling_nan(a);
-    int bIsSignalingNaN = float64_is_signaling_nan(b64);
+    int aIsSignalingNaN = extF80_isSignalingNaN(a);
+    int bIsSignalingNaN = f64_isSignalingNaN(b64);
 
     if (aIsSignalingNaN | bIsSignalingNaN)
-        float_raise(status, float_flag_invalid);
+        softfloat_raiseFlags(&status, softfloat_flag_invalid);
 
     // propagate QNaN to SNaN
-    a = propagateFloatx80NaN(a, status);
+    a = softfloat_propagateNaNExtF80UI(a.signExp, a.signif, 0, 0, &status);
 
     if (aIsNaN & !bIsNaN) return a;
 
     // float64 is NaN so conversion will propagate SNaN to QNaN and raise
     // appropriate exception flags
-    floatx80 b = float64_to_floatx80(b64, status);
+    floatx80 b = f64_to_extF80(b64, &status);
 
     if (aIsSignalingNaN) {
         if (bIsSignalingNaN) goto returnLargerSignificand;
@@ -138,24 +139,24 @@ floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float64 b64, int bIsNaN, float_s
     else if (aIsNaN) {
         if (bIsSignalingNaN) return a;
  returnLargerSignificand:
-        if (a.fraction < b.fraction) return b;
-        if (b.fraction < a.fraction) return a;
-        return (a.exp < b.exp) ? a : b;
+        if (a.signif < b.signif) return b;
+        if (b.signif < a.signif) return a;
+        return (a.signExp < b.signExp) ? a : b;
     }
     else {
         return b;
     }
 }
 
-bool FPU_handle_NaN(floatx80 a, float64 b, floatx80 &r, float_status_t &status)
+bool FPU_handle_NaN(floatx80 a, float64 b, floatx80 &r, softfloat_status_t &status)
 {
-  if (floatx80_is_unsupported(a)) {
-     float_raise(status, float_flag_invalid);
+  if (extF80_isUnsupported(a)) {
+     softfloat_raiseFlags(&status, softfloat_flag_invalid);
      r = floatx80_default_nan;
      return true;
   }
 
-  int aIsNaN = floatx80_is_nan(a), bIsNaN = float64_is_nan(b);
+  int aIsNaN = extF80_isNaN(a), bIsNaN = f64_isNaN(b);
   if (aIsNaN | bIsNaN) {
      r = FPU_handle_NaN(a, aIsNaN, b, bIsNaN, status);
      return true;
@@ -179,12 +180,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->src());
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_add(a, b, status);
+  floatx80 result = extF80_add(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -208,12 +209,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->dst());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_add(a, b, status);
+  floatx80 result = extF80_add(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags)) {
+  if (! FPU_exception(i, status.softfloat_exceptionFlags)) {
      BX_WRITE_FPU_REG(result, i->dst());
      if (pop_stack)
         BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -238,14 +239,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_SINGLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
-     result = floatx80_add(a, float32_to_floatx80(load_reg, status), status);
+     result = extF80_add(a, f32_to_extF80(load_reg, &status), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -267,14 +268,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_DOUBLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
-     result = floatx80_add(a, float64_to_floatx80(load_reg, status), status);
+     result = extF80_add(a, f64_to_extF80(load_reg, &status), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -297,14 +298,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_WORD_INTEGER(bxInstruction_c *i)
   }
 
   floatx80 a = BX_READ_FPU_REG(0);
-  floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
+  floatx80 b = i32_to_extF80((Bit32s)(load_reg));
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_add(a, b, status);
+  floatx80 result = extF80_add(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -327,14 +328,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_DWORD_INTEGER(bxInstruction_c *i)
   }
 
   floatx80 a = BX_READ_FPU_REG(0);
-  floatx80 b = int32_to_floatx80(load_reg);
+  floatx80 b = i32_to_extF80(load_reg);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_add(a, b, status);
+  floatx80 result = extF80_add(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -356,12 +357,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->src());
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_mul(a, b, status);
+  floatx80 result = extF80_mul(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -385,12 +386,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->dst());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_mul(a, b, status);
+  floatx80 result = extF80_mul(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags)) {
+  if (! FPU_exception(i, status.softfloat_exceptionFlags)) {
      BX_WRITE_FPU_REG(result, i->dst());
      if (pop_stack)
         BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -415,14 +416,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_SINGLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
-     result = floatx80_mul(a, float32_to_floatx80(load_reg, status), status);
+     result = extF80_mul(a, f32_to_extF80(load_reg, &status), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -444,14 +445,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_DOUBLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
-     result = floatx80_mul(a, float64_to_floatx80(load_reg, status), status);
+     result = extF80_mul(a, f64_to_extF80(load_reg, &status), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -474,14 +475,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_WORD_INTEGER(bxInstruction_c *i)
   }
 
   floatx80 a = BX_READ_FPU_REG(0);
-  floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
+  floatx80 b = i32_to_extF80((Bit32s)(load_reg));
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_mul(a, b, status);
+  floatx80 result = extF80_mul(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -504,14 +505,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_DWORD_INTEGER(bxInstruction_c *i)
   }
 
   floatx80 a = BX_READ_FPU_REG(0);
-  floatx80 b = int32_to_floatx80(load_reg);
+  floatx80 b = i32_to_extF80(load_reg);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_mul(a, b, status);
+  floatx80 result = extF80_mul(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -533,12 +534,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->src());
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(a, b, status);
+  floatx80 result = extF80_sub(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -560,12 +561,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->src());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(a, b, status);
+  floatx80 result = extF80_sub(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -589,12 +590,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->dst());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(a, b, status);
+  floatx80 result = extF80_sub(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags)) {
+  if (! FPU_exception(i, status.softfloat_exceptionFlags)) {
      BX_WRITE_FPU_REG(result, i->dst());
      if (pop_stack)
         BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -621,12 +622,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->dst());
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(a, b, status);
+  floatx80 result = extF80_sub(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags)) {
+  if (! FPU_exception(i, status.softfloat_exceptionFlags)) {
      BX_WRITE_FPU_REG(result, i->dst());
      if (pop_stack)
         BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -651,14 +652,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_SINGLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
-     result = floatx80_sub(a, float32_to_floatx80(load_reg, status), status);
+     result = extF80_sub(a, f32_to_extF80(load_reg, &status), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -680,14 +681,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_SINGLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 b = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(b, load_reg, result, status))
-     result = floatx80_sub(float32_to_floatx80(load_reg, status), b, status);
+     result = extF80_sub(f32_to_extF80(load_reg, &status), b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -709,14 +710,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_DOUBLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
-     result = floatx80_sub(a, float64_to_floatx80(load_reg, status), status);
+     result = extF80_sub(a, f64_to_extF80(load_reg, &status), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -738,15 +739,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_DOUBLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
 
   floatx80 b = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(b, load_reg, result, status))
-     result = floatx80_sub(float64_to_floatx80(load_reg, status), b, status);
+     result = extF80_sub(f64_to_extF80(load_reg, &status), b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -769,14 +770,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_WORD_INTEGER(bxInstruction_c *i)
   }
 
   floatx80 a = BX_READ_FPU_REG(0);
-  floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
+  floatx80 b = i32_to_extF80((Bit32s)(load_reg));
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(a, b, status);
+  floatx80 result = extF80_sub(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -798,15 +799,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_WORD_INTEGER(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  floatx80 a = int32_to_floatx80((Bit32s)(load_reg));
+  floatx80 a = i32_to_extF80((Bit32s)(load_reg));
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(a, b, status);
+  floatx80 result = extF80_sub(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -828,12 +829,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_DWORD_INTEGER(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(BX_READ_FPU_REG(0), int32_to_floatx80(load_reg), status);
+  floatx80 result = extF80_sub(BX_READ_FPU_REG(0), i32_to_extF80(load_reg), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -855,15 +856,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_DWORD_INTEGER(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  floatx80 a = int32_to_floatx80(load_reg);
+  floatx80 a = i32_to_extF80(load_reg);
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(a, b, status);
+  floatx80 result = extF80_sub(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -885,12 +886,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->src());
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(a, b, status);
+  floatx80 result = extF80_div(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -912,12 +913,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->src());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(a, b, status);
+  floatx80 result = extF80_div(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -941,12 +942,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->dst());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(a, b, status);
+  floatx80 result = extF80_div(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags)) {
+  if (! FPU_exception(i, status.softfloat_exceptionFlags)) {
      BX_WRITE_FPU_REG(result, i->dst());
      if (pop_stack)
         BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -973,12 +974,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->dst());
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(a, b, status);
+  floatx80 result = extF80_div(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags)) {
+  if (! FPU_exception(i, status.softfloat_exceptionFlags)) {
      BX_WRITE_FPU_REG(result, i->dst());
      if (pop_stack)
         BX_CPU_THIS_PTR the_i387.FPU_pop();
@@ -1003,14 +1004,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_SINGLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
-     result = floatx80_div(a, float32_to_floatx80(load_reg, status), status);
+     result = extF80_div(a, f32_to_extF80(load_reg, &status), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -1032,14 +1033,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_SINGLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 b = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(b, load_reg, result, status))
-     result = floatx80_div(float32_to_floatx80(load_reg, status), b, status);
+     result = extF80_div(f32_to_extF80(load_reg, &status), b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -1061,14 +1062,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_DOUBLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(a, load_reg, result, status))
-     result = floatx80_div(a, float64_to_floatx80(load_reg, status), status);
+     result = extF80_div(a, f64_to_extF80(load_reg, &status), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -1090,14 +1091,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_DOUBLE_REAL(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 b = BX_READ_FPU_REG(0), result;
   if (! FPU_handle_NaN(b, load_reg, result, status))
-     result = floatx80_div(float64_to_floatx80(load_reg, status), b, status);
+     result = extF80_div(f64_to_extF80(load_reg, &status), b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -1120,14 +1121,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_WORD_INTEGER(bxInstruction_c *i)
   }
 
   floatx80 a = BX_READ_FPU_REG(0);
-  floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
+  floatx80 b = i32_to_extF80((Bit32s)(load_reg));
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(a, b, status);
+  floatx80 result = extF80_div(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -1149,15 +1150,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_WORD_INTEGER(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  floatx80 a = int32_to_floatx80((Bit32s)(load_reg));
+  floatx80 a = i32_to_extF80((Bit32s)(load_reg));
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(a, b, status);
+  floatx80 result = extF80_div(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -1180,14 +1181,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_DWORD_INTEGER(bxInstruction_c *i)
   }
 
   floatx80 a = BX_READ_FPU_REG(0);
-  floatx80 b = int32_to_floatx80(load_reg);
+  floatx80 b = i32_to_extF80(load_reg);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(a, b, status);
+  floatx80 result = extF80_div(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -1209,15 +1210,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_DWORD_INTEGER(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  floatx80 a = int32_to_floatx80(load_reg);
+  floatx80 a = i32_to_extF80(load_reg);
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(a, b, status);
+  floatx80 result = extF80_div(a, b, &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -1235,12 +1236,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSQRT(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sqrt(BX_READ_FPU_REG(0), status);
+  floatx80 result = extF80_sqrt(BX_READ_FPU_REG(0), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
@@ -1259,12 +1260,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FRNDINT(bxInstruction_c *i)
      BX_NEXT_INSTR(i);
   }
 
-  float_status_t status =
+  softfloat_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_round_to_int(BX_READ_FPU_REG(0), status);
+  floatx80 result = extF80_roundToInt(BX_READ_FPU_REG(0), &status);
 
-  if (! FPU_exception(i, status.float_exception_flags))
+  if (! FPU_exception(i, status.softfloat_exceptionFlags))
      BX_WRITE_FPU_REG(result, 0);
 
   BX_NEXT_INSTR(i);
