@@ -109,6 +109,20 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_Wsd(bxInstruction_c *i)
 }
 
 #if BX_SUPPORT_EVEX
+// load 16-bit value with mask support and place into low 16-bit of XMM register
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_MASK_Ww(bxInstruction_c *i)
+{
+  Bit16u val_16 = 0;
+
+  if (BX_SCALAR_ELEMENT_MASK(i->opmask())) {
+    bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+    val_16 = read_virtual_word(i->seg(), eaddr);
+  }
+
+  BX_WRITE_XMM_REG_LO_WORD(BX_VECTOR_TMP_REGISTER, val_16);
+  BX_CPU_CALL_METHOD(i->execute2(), (i));
+}
+
 // load 32-bit value with mask support and place into low 32-bit of XMM register
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_MASK_Wss(bxInstruction_c *i)
 {
@@ -451,6 +465,28 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_MASK_Eighth_VectorB(bxInstruction_c *
 
 #include "simd_int.h"
 
+// load vector of words, support broadcast, no fault suppression
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_VectorW(bxInstruction_c *i)
+{
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+  unsigned len = i->getVL();
+
+  if (i->getEvexb()) {
+    Bit16u val_16 = read_virtual_word(i->seg(), eaddr);
+    simd_pbroadcastw(&BX_AVX_REG(BX_VECTOR_TMP_REGISTER), val_16, len * 8);
+  }
+  else {
+    if (len == BX_VL512)
+      read_virtual_zmmword(i->seg(), eaddr, &BX_READ_AVX_REG(BX_VECTOR_TMP_REGISTER));
+    else if (len == BX_VL256)
+      read_virtual_ymmword(i->seg(), eaddr, &BX_READ_YMM_REG(BX_VECTOR_TMP_REGISTER));
+    else
+      read_virtual_xmmword(i->seg(), eaddr, &BX_READ_XMM_REG(BX_VECTOR_TMP_REGISTER));
+  }
+
+  BX_CPU_CALL_METHOD(i->execute2(), (i));
+}
+
 // load vector of dwords, support broadcast, no fault suppression
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_VectorD(bxInstruction_c *i)
 {
@@ -495,6 +531,31 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_VectorQ(bxInstruction_c *i)
   BX_CPU_CALL_METHOD(i->execute2(), (i));
 }
 
+// load vector of words, support broadcast and masked fault suppression
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_MASK_VectorW(bxInstruction_c *i)
+{
+  unsigned len = i->getVL();
+  Bit32u opmask = (i->opmask() != 0) ? BX_READ_32BIT_OPMASK(i->opmask()) : 0xffffffff;
+  opmask &= CUT_OPMASK_TO(WORD_ELEMENTS(len));
+
+  if (opmask == 0) {
+    BX_CPU_CALL_METHOD(i->execute2(), (i)); // for now let execute method to deal with zero/merge masking semantics
+    return;
+  }
+
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+
+  if (i->getEvexb()) {
+    Bit16u val_16 = read_virtual_word(i->seg(), eaddr);
+    simd_pbroadcastw(&BX_AVX_REG(BX_VECTOR_TMP_REGISTER), val_16, len * 8);
+  }
+  else {
+    avx_masked_load16(i, eaddr, &BX_READ_AVX_REG(BX_VECTOR_TMP_REGISTER), opmask);
+  }
+
+  BX_CPU_CALL_METHOD(i->execute2(), (i));
+}
+
 // load vector of dwords, support broadcast and masked fault suppression
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_MASK_VectorD(bxInstruction_c *i)
 {
@@ -545,6 +606,57 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_MASK_VectorQ(bxInstruction_
   BX_CPU_CALL_METHOD(i->execute2(), (i));
 }
 
+// load half vector of words, support broadcast, no fault suppression
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_Half_VectorW(bxInstruction_c *i)
+{
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+  unsigned len = i->getVL();
+
+  if (i->getEvexb()) {
+    Bit16u val_16 = read_virtual_word(i->seg(), eaddr);
+    simd_pbroadcastw(&BX_AVX_REG(BX_VECTOR_TMP_REGISTER), val_16, len * 4);
+  }
+  else {
+    if (len == BX_VL512) {
+      read_virtual_ymmword(i->seg(), eaddr, &BX_READ_YMM_REG(BX_VECTOR_TMP_REGISTER));
+    }
+    else if (len == BX_VL256) {
+      read_virtual_xmmword(i->seg(), eaddr, &BX_READ_XMM_REG(BX_VECTOR_TMP_REGISTER));
+    }
+    else {
+      Bit64u val_64 = read_virtual_qword(i->seg(), eaddr);
+      BX_WRITE_XMM_REG_LO_QWORD(BX_VECTOR_TMP_REGISTER, val_64);
+    }
+  }
+
+  BX_CPU_CALL_METHOD(i->execute2(), (i));
+}
+
+// load half vector of words, support broadcast and masked fault suppression
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_MASK_Half_VectorW(bxInstruction_c *i)
+{
+  Bit32u opmask = (i->opmask() != 0) ? BX_READ_16BIT_OPMASK(i->opmask()) : 0xffff;
+  unsigned len = i->getVL();
+  opmask &= CUT_OPMASK_TO(DWORD_ELEMENTS(len));
+
+  if (opmask == 0) {
+    BX_CPU_CALL_METHOD(i->execute2(), (i)); // for now let execute method to deal with zero/merge masking semantics
+    return;
+  }
+
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+
+  if (i->getEvexb()) {
+    Bit16u val_16 = read_virtual_word(i->seg(), eaddr);
+    simd_pbroadcastw(&BX_AVX_REG(BX_VECTOR_TMP_REGISTER), val_16, len * 4);
+  }
+  else {
+    avx_masked_load16(i, eaddr, &BX_READ_AVX_REG(BX_VECTOR_TMP_REGISTER), opmask);
+  }
+
+  BX_CPU_CALL_METHOD(i->execute2(), (i));
+}
+
 // load half vector of dwords, support broadcast, no fault suppression
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_Half_VectorD(bxInstruction_c *i)
 {
@@ -596,6 +708,58 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_MASK_Half_VectorD(bxInstruc
   BX_CPU_CALL_METHOD(i->execute2(), (i));
 }
 
+// load quarter of vector of words, support broadcast, no fault suppression
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_Quarter_VectorW(bxInstruction_c *i)
+{
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+  unsigned len = i->getVL();
+
+  if (i->getEvexb()) {
+    Bit16u val_16 = read_virtual_word(i->seg(), eaddr);
+    simd_pbroadcastw(&BX_AVX_REG(BX_VECTOR_TMP_REGISTER), val_16, len * 2);
+  }
+  else {
+    if (len == BX_VL512) {
+      read_virtual_xmmword(i->seg(), eaddr, &BX_READ_XMM_REG(BX_VECTOR_TMP_REGISTER));
+    }
+    else if (len == BX_VL256) {
+      Bit64u val_64 = read_virtual_qword(i->seg(), eaddr);
+      BX_WRITE_XMM_REG_LO_QWORD(BX_VECTOR_TMP_REGISTER, val_64);
+    }
+    else {
+      Bit32u val_32 = read_virtual_dword(i->seg(), eaddr);
+      BX_WRITE_XMM_REG_LO_DWORD(BX_VECTOR_TMP_REGISTER, val_32);
+    }
+  }
+
+  BX_CPU_CALL_METHOD(i->execute2(), (i));
+}
+
+// load quarter of vector of words, support broadcast and masked fault suppression
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOAD_BROADCAST_MASK_Quarter_VectorW(bxInstruction_c *i)
+{
+  Bit32u opmask = (i->opmask() != 0) ? BX_READ_8BIT_OPMASK(i->opmask()) : 0xff;
+  unsigned len = i->getVL();
+  opmask &= CUT_OPMASK_TO(QWORD_ELEMENTS(len));
+
+  if (opmask == 0) {
+    BX_CPU_CALL_METHOD(i->execute2(), (i)); // for now let execute method to deal with zero/merge masking semantics
+    return;
+  }
+
+  bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
+
+  if (i->getEvexb()) {
+    Bit16u val_16 = read_virtual_word(i->seg(), eaddr);
+    simd_pbroadcastw(&BX_AVX_REG(BX_VECTOR_TMP_REGISTER), val_16, len * 2);
+  }
+  else {
+    avx_masked_load16(i, eaddr, &BX_READ_AVX_REG(BX_VECTOR_TMP_REGISTER), opmask);
+  }
+
+  BX_CPU_CALL_METHOD(i->execute2(), (i));
+}
+
 #endif // BX_SUPPORT_EVEX
 
 #endif // BX_SUPPORT_AVX
