@@ -37,6 +37,8 @@
 #include "memory/memory-bochs.h"
 #include "pc_system.h"
 
+#include "bx_debug/debug.h"
+
 // X86 Registers Which Affect Paging:
 // ==================================
 //
@@ -896,7 +898,7 @@ bx_phy_address BX_CPU_C::translate_linear_long_mode(bx_address laddr, Bit32u &lp
 
   if (BX_CPU_THIS_PTR cr4.get_PGE())
     combined_access |= (entry[leaf] & BX_COMBINED_ACCESS_GLOBAL_PAGE); // G
-  combined_access |= nx_page;
+  combined_access |= (Bit32u) nx_page;
 
 #if BX_SUPPORT_MEMTYPE
   combined_access = combine_memtype(combined_access, memtype_by_pat(calculate_pat((Bit32u) entry[leaf], lpf_mask)));
@@ -1105,7 +1107,7 @@ bx_phy_address BX_CPU_C::translate_linear_PAE(bx_address laddr, Bit32u &lpf_mask
 
   if (BX_CPU_THIS_PTR cr4.get_PGE())
     combined_access |= (entry[leaf] & BX_COMBINED_ACCESS_GLOBAL_PAGE); // G
-  combined_access |= nx_page;
+  combined_access |= (Bit32u) nx_page;
 
 #if BX_SUPPORT_MEMTYPE
   combined_access = combine_memtype(combined_access, memtype_by_pat(calculate_pat((Bit32u) entry[leaf], lpf_mask)));
@@ -1457,10 +1459,14 @@ bx_phy_address BX_CPU_C::translate_linear(bx_TLB_entry *tlbEntry, bx_address lad
 
 #if BX_SUPPORT_X86_64
     if (long64_mode() && BX_CPU_THIS_PTR cr4.get_LASS()) {
-      if (lpf >> 63) // supervisor, cannot access user pages
-        tlbEntry->accessBits &= ~(TLB_UserReadOK | TLB_UserWriteOK | TLB_UserReadShadowStackOK | TLB_UserWriteShadowStackOK);
-      else           // user, cannot access supervisor pages
-        tlbEntry->accessBits &= ~(TLB_SysReadOK | TLB_SysWriteOK | TLB_SysReadShadowStackOK | TLB_SysWriteShadowStackOK);
+      if (lpf >> 63) { // supervisor, cannot be accessed by user
+        tlbEntry->accessBits &= ~(TLB_UserReadOK | TLB_UserWriteOK | TLB_UserReadShadowStackOK | TLB_UserWriteShadowStackOK | TLB_UserExecuteOK);
+      }
+      else {           // user, cannot be executed by supervisor, cannot be accessed by supervisor if CR4.SMAP=1
+        tlbEntry->accessBits &= ~(TLB_SysExecuteOK);
+        if (BX_CPU_THIS_PTR cr4.get_SMAP())
+          tlbEntry->accessBits &= ~(TLB_SysReadOK | TLB_SysWriteOK | TLB_SysReadShadowStackOK | TLB_SysWriteShadowStackOK);
+      }
     }
 #endif
   }
