@@ -33,7 +33,8 @@
 #include "pc_system.h"
 #include "gui/gui.h"
 
-#include "wide_int.h"
+#include "bx_debug/debug.h"
+
 #include "decoder/ia_opcodes.h"
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxError(bxInstruction_c *i)
@@ -43,10 +44,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxError(bxInstruction_c *i)
   if (ia_opcode == BX_IA_ERROR) {
     BX_DEBUG(("BxError: Encountered an unknown instruction (signalling #UD)"));
 
-#if BX_DEBUGGER == 0 // with debugger it easy to see the #UD
     if (LOG_THIS getonoff(LOGLEV_DEBUG))
       debug_disasm_instruction(BX_CPU_THIS_PTR prev_rip);
-#endif
   }
   else {
     BX_DEBUG(("%s: instruction not supported - signalling #UD", get_bx_opcode_name(ia_opcode)));
@@ -182,7 +181,8 @@ void BX_CPU_C::enter_sleep_state(unsigned state)
   BX_INSTR_HLT(BX_CPU_ID);
 
 #if BX_DEBUGGER
-  bx_dbg_halt(BX_CPU_ID);
+  if (bx_dbg.debugger_active)
+    bx_dbg_halt(BX_CPU_ID);
 #endif
 
 #if BX_USE_IDLE_HACK
@@ -405,12 +405,21 @@ void BX_CPU_C::handleCpuModeChange(void)
   set_PKeys(BX_CPU_THIS_PTR pkru, BX_CPU_THIS_PTR pkrs);
 #endif
 
+#if BX_DEBUGGER
+  if (bx_dbg.debugger_active) {
+    // assert magic async_event to stop trace execution
+    BX_CPU_THIS_PTR async_event |= BX_ASYNC_EVENT_STOP_TRACE;
+  }
+#endif
+
   if (mode != BX_CPU_THIS_PTR cpu_mode) {
     BX_DEBUG(("%s activated", cpu_mode_string(BX_CPU_THIS_PTR cpu_mode)));
 #if BX_DEBUGGER
-    if (BX_CPU_THIS_PTR mode_break) {
-      BX_CPU_THIS_PTR stop_reason = STOP_MODE_BREAK_POINT;
-      bx_debug_break(); // trap into debugger
+    if (bx_dbg.debugger_active) {
+      if (BX_CPU_THIS_PTR mode_break) {
+        BX_CPU_THIS_PTR stop_reason = STOP_MODE_BREAK_POINT;
+        bx_debug_break(); // trap into debugger
+      }
     }
 #endif
   }
@@ -686,6 +695,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDPMC(bxInstruction_c *i)
 
 #if BX_CPU_LEVEL >= 5
 
+#include "wide_int.h"
+
 Bit64u BX_CPU_C::get_TSC(void)
 {
   Bit64u tsc = bx_pc_system.time_ticks() + BX_CPU_THIS_PTR tsc_adjust;
