@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: scalar_arith.h 14141 2021-02-11 15:05:06Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2014 Stanislav Shwartsman
@@ -87,6 +87,11 @@ BX_CPP_INLINE unsigned lzcntw(Bit16u val_16)
   return count;
 }
 
+BX_CPP_INLINE unsigned most_significant_bitw(Bit16u val_16)
+{
+  return 15 - lzcntw(val_16);
+}
+
 BX_CPP_INLINE unsigned lzcntd(Bit32u val_32)
 {
   Bit32u mask = 0x80000000;
@@ -100,6 +105,11 @@ BX_CPP_INLINE unsigned lzcntd(Bit32u val_32)
   return count;
 }
 
+BX_CPP_INLINE unsigned most_significant_bitd(Bit32u val_32)
+{
+  return 31 - lzcntd(val_32);
+}
+
 BX_CPP_INLINE unsigned lzcntq(Bit64u val_64)
 {
   Bit64u mask = BX_CONST64(0x8000000000000000);
@@ -113,6 +123,11 @@ BX_CPP_INLINE unsigned lzcntq(Bit64u val_64)
   return count;
 }
 
+BX_CPP_INLINE unsigned most_significant_bitq(Bit64u val_64)
+{
+  return 63 - lzcntq(val_64);
+}
+
 // popcnt
 
 BX_CPP_INLINE unsigned popcntb(Bit8u val_8)
@@ -191,4 +206,46 @@ BX_CPP_INLINE Bit64u bextrq(Bit64u val_64, unsigned start, unsigned len)
   return result;
 }
 
+// rol, ror
+
+BX_CPP_INLINE Bit8u rol8(Bit8u v8, unsigned count)
+{
+  return (v8 << count) | (v8 >> (8 - count));
+}
+
+BX_CPP_INLINE Bit8u ror8(Bit8u v8, unsigned count)
+{
+  return (v8 >> count) | (v8 << (8 - count));
+}
+
+BX_CPP_INLINE Bit16u rol16(Bit16u v16, unsigned count)
+{
+  return (v16 << count) | (v16 >> (16 - count));
+}
+
+BX_CPP_INLINE Bit16u ror16(Bit16u v16, unsigned count)
+{
+  return (v16 >> count) | (v16 << (16 - count));
+}
+
+BX_CPP_INLINE Bit32u rol32(Bit32u v32, unsigned count)
+{
+  return (v32 << count) | (v32 >> (32 - count));
+}
+
+BX_CPP_INLINE Bit32u ror32(Bit32u v32, unsigned count)
+{
+  return (v32 >> count) | (v32 << (32 - count));
+}
+
+BX_CPP_INLINE Bit64u rol64(Bit64u v64, unsigned count)
+{
+  return (v64 << count) | (v64 >> (64 - count));
+}
+
+BX_CPP_INLINE Bit64u ror64(Bit64u v64, unsigned count)
+{
+  return (v64 >> count) | (v64 << (64 - count));
+}
+
 #endif
