@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: smm.cc 14256 2021-05-25 06:27:49Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2006-2018 Stanislav Shwartsman
@@ -26,6 +26,10 @@
 #include "smm.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#if BX_SUPPORT_SVM
+#include "svm.h"
+#endif
+
 #if BX_CPU_LEVEL >= 3
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSM(bxInstruction_c *i)
@@ -69,7 +73,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSM(bxInstruction_c *i)
     access_read_physical(base, 4, &saved_state[n]);
     BX_NOTIFY_PHY_MEMORY_ACCESS(base, 4, BX_MEMTYPE_WB, BX_READ, BX_SMRAM_ACCESS, (Bit8u*)(&saved_state[n]));
   }
-  BX_CPU_THIS_PTR in_smm = 0;
+  BX_CPU_THIS_PTR in_smm = false;
 
   // restore the CPU state from SMRAM
   if (! smram_restore_state(saved_state)) {
@@ -108,8 +112,8 @@ void BX_CPU_C::enter_system_management_mode(void)
   BX_CPU_THIS_PTR cr4.set_VMXE(0);
   BX_CPU_THIS_PTR in_smm_vmx = BX_CPU_THIS_PTR in_vmx;
   BX_CPU_THIS_PTR in_smm_vmx_guest = BX_CPU_THIS_PTR in_vmx_guest;
-  BX_CPU_THIS_PTR in_vmx = 0;
-  BX_CPU_THIS_PTR in_vmx_guest = 0;
+  BX_CPU_THIS_PTR in_vmx = false;
+  BX_CPU_THIS_PTR in_vmx_guest = false;
 
   BX_INFO(("enter_system_management_mode: temporary disable VMX while in SMM mode"));
 
@@ -118,7 +122,7 @@ void BX_CPU_C::enter_system_management_mode(void)
   //   * set processor state to standard SMM values
 #endif
 
-  BX_CPU_THIS_PTR in_smm = 1;
+  BX_CPU_THIS_PTR in_smm = true;
 
   mask_event(BX_EVENT_SMI | BX_EVENT_NMI | BX_EVENT_VMX_VIRTUAL_NMI);
 
@@ -132,8 +136,7 @@ void BX_CPU_C::enter_system_management_mode(void)
   // could be optimized with reading of only non-reserved bytes
   for(n=0;n<SMM_SAVE_STATE_MAP_SIZE;n++) {
     base -= 4;
-    access_write_physical(base, 4, &saved_state[n]);
-    BX_NOTIFY_PHY_MEMORY_ACCESS(base, 4, BX_MEMTYPE_WB, BX_WRITE, BX_SMRAM_ACCESS, (Bit8u*)(&saved_state[n]));
+    write_physical_dword(base, saved_state[n], BX_MEMTYPE_WB, BX_SMRAM_ACCESS);
   }
 
   BX_CPU_THIS_PTR setEFlags(0x2); // Bit1 is always set
@@ -273,6 +276,8 @@ void BX_CPU_C::init_SMRAM(void)
   smram_map[SMRAM_FIELD_CR3] = SMRAM_TRANSLATE(0x7f50);
   smram_map[SMRAM_FIELD_CR4_HI32] = SMRAM_TRANSLATE(0x7f4c);    // always zero
   smram_map[SMRAM_FIELD_CR4] = SMRAM_TRANSLATE(0x7f48);
+  smram_map[SMRAM_FIELD_SSP_HI32] = SMRAM_TRANSLATE(0x7f44);
+  smram_map[SMRAM_FIELD_SSP] = SMRAM_TRANSLATE(0x7f40);
   smram_map[SMRAM_FIELD_EFER_HI32] = SMRAM_TRANSLATE(0x7ed4);   // always zero
   smram_map[SMRAM_FIELD_EFER] = SMRAM_TRANSLATE(0x7ed0);
   smram_map[SMRAM_FIELD_IO_INSTRUCTION_RESTART] = SMRAM_TRANSLATE(0x7ec8);
@@ -422,6 +427,11 @@ void BX_CPU_C::smram_save_state(Bit32u *saved_state)
   SMRAM_FIELD(saved_state, SMRAM_FIELD_EIP) = EIP;
   SMRAM_FIELD(saved_state, SMRAM_FIELD_EFLAGS) = read_eflags();
 
+#if BX_SUPPORT_CET
+  SMRAM_FIELD(saved_state, SMRAM_FIELD_SSP_HI32) = GET32H(SSP);
+  SMRAM_FIELD(saved_state, SMRAM_FIELD_SSP) = GET32L(SSP);
+#endif
+
   // --- Debug and Control Registers --- //
   SMRAM_FIELD(saved_state, SMRAM_FIELD_DR6) = BX_CPU_THIS_PTR dr6.get32();
   SMRAM_FIELD(saved_state, SMRAM_FIELD_DR7) = BX_CPU_THIS_PTR dr7.get32();
@@ -489,6 +499,9 @@ bool BX_CPU_C::smram_restore_state(const Bit32u *saved_state)
   }
 
   smm_state.rip = SMRAM_FIELD64(saved_state, SMRAM_FIELD_RIP_HI32, SMRAM_FIELD_EIP);
+#if BX_SUPPORT_CET
+  smm_state.ssp = SMRAM_FIELD64(saved_state, SMRAM_FIELD_SSP_HI32, SMRAM_FIELD_SSP);
+#endif
 
   smm_state.dr6 = SMRAM_FIELD(saved_state, SMRAM_FIELD_DR6);
   smm_state.dr7 = SMRAM_FIELD(saved_state, SMRAM_FIELD_DR7);
@@ -643,7 +656,7 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
 
   if (smm_state->cr4.get_VMXE()) {
     BX_PANIC(("SMM restore: CR4.VMXE is set in restore image !"));
-    return 0;
+    return false;
   }
 
   // restore state normally from SMRAM;
@@ -666,7 +679,7 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
   //   FI;
 
   if (BX_CPU_THIS_PTR in_smm_vmx) {
-    BX_CPU_THIS_PTR in_vmx = 1;
+    BX_CPU_THIS_PTR in_vmx = true;
     BX_CPU_THIS_PTR in_vmx_guest = BX_CPU_THIS_PTR in_smm_vmx_guest;
     BX_INFO(("SMM Restore: enable VMX %s mode", BX_CPU_THIS_PTR in_vmx_guest ? "guest" : "host"));
 
@@ -682,7 +695,7 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
 #if BX_CPU_LEVEL >= 5
   if (smm_state->efer.get32() & ~((Bit64u) BX_CPU_THIS_PTR efer_suppmask)) {
     BX_PANIC(("SMM restore: Attempt to set EFER reserved bits: 0x%08x !", smm_state->efer.get32()));
-    return 0;
+    return false;
   }
 
   BX_CPU_THIS_PTR efer.set32(smm_state->efer.val32);
@@ -691,13 +704,13 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
   // check CR0 conditions for entering to shutdown state
   if (!check_CR0(smm_state->cr0.get32())) {
     BX_PANIC(("SMM restore: CR0 consistency check failed !"));
-    return 0;
+    return false;
   }
 
 #if BX_CPU_LEVEL >= 5
   if (!check_CR4(smm_state->cr4.get32())) {
     BX_PANIC(("SMM restore: CR4 consistency check failed !"));
-    return 0;
+    return false;
   }
 #endif
 
@@ -711,25 +724,25 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
   if (BX_CPU_THIS_PTR efer.get_LMA()) {
     if (smm_state->eflags & EFlagsVMMask) {
       BX_PANIC(("SMM restore: If EFER.LMA = 1 => RFLAGS.VM=0 !"));
-      return 0;
+      return false;
     }
 
     if (!BX_CPU_THIS_PTR cr4.get_PAE() || !BX_CPU_THIS_PTR cr0.get_PG() || !BX_CPU_THIS_PTR cr0.get_PE() || !BX_CPU_THIS_PTR efer.get_LME()) {
       BX_PANIC(("SMM restore: If EFER.LMA = 1 <=> CR4.PAE, CR0.PG, CR0.PE, EFER.LME=1 !"));
-      return 0;
+      return false;
     }
   }
   else {
     if (BX_CPU_THIS_PTR cr4.get_PCIDE()) {
       BX_PANIC(("SMM restore: CR4.PCIDE must be clear when not in long mode !"));
-      return 0;
+      return false;
     }
   }
 
   if (BX_CPU_THIS_PTR cr4.get_PAE() && BX_CPU_THIS_PTR cr0.get_PG() && BX_CPU_THIS_PTR cr0.get_PE() && BX_CPU_THIS_PTR efer.get_LME()) {
     if (! BX_CPU_THIS_PTR efer.get_LMA()) {
       BX_PANIC(("SMM restore: If EFER.LMA = 1 <=> CR4.PAE, CR0.PG, CR0.PE, EFER.LME=1 !"));
-      return 0;
+      return false;
     }
   }
 #endif
@@ -738,7 +751,7 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
   if (BX_CPU_THIS_PTR cr0.get_PG() && BX_CPU_THIS_PTR cr4.get_PAE() && !long_mode()) {
     if (! CheckPDPTR(smm_state->cr3)) {
       BX_ERROR(("SMM restore: PDPTR check failed !"));
-      return 0;
+      return false;
     }
   }
 #endif
@@ -754,6 +767,9 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
 #endif
 
   RIP = BX_CPU_THIS_PTR prev_rip = smm_state->rip;
+#if BX_SUPPORT_CET
+  SSP = smm_state->ssp;
+#endif
 
   BX_CPU_THIS_PTR dr6.val32 = smm_state->dr6;
   BX_CPU_THIS_PTR dr7.val32 = smm_state->dr7;
@@ -771,7 +787,7 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
     {
        if (! BX_CPU_THIS_PTR sregs[segreg].cache.segment) {
          BX_PANIC(("SMM restore: restored valid non segment %d !", segreg));
-         return 0;
+         return false;
        }
     }
   }
@@ -785,7 +801,7 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
   {
      if (BX_CPU_THIS_PTR ldtr.cache.type != BX_SYS_SEGMENT_LDT) {
         BX_PANIC(("SMM restore: LDTR is not LDT descriptor type !"));
-        return 0;
+        return false;
      }
   }
 
@@ -802,7 +818,7 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
          BX_CPU_THIS_PTR tr.cache.type != BX_SYS_SEGMENT_BUSY_386_TSS)
      {
         BX_PANIC(("SMM restore: TR is not TSS descriptor type !"));
-        return 0;
+        return false;
      }
   }
 
@@ -811,7 +827,7 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
 #if BX_CPU_LEVEL < 6
      if (BX_CPU_THIS_PTR smbase & 0x7fff) {
         BX_PANIC(("SMM restore: SMBASE must be aligned to 32K !"));
-        return 0;
+        return false;
      }
 #endif
   }
@@ -824,7 +840,7 @@ bool BX_CPU_C::resume_from_system_management_mode(BX_SMM_State *smm_state)
 
   BX_INSTR_TLB_CNTRL(BX_CPU_ID, BX_INSTR_CONTEXT_SWITCH, 0);
 
-  return 1;
+  return true;
 }
 
 #endif /* BX_CPU_LEVEL >= 3 */
