@@ -461,6 +461,12 @@ bool BX_CPU_C::vmcs_field_supported(Bit32u encoding)
       return is_cpu_extension_supported(BX_ISA_PKS);
 #endif
 
+#if BX_SUPPORT_VMX >= 2    
+    case VMCS_64BIT_GUEST_IA32_SPEC_CTRL:
+    case VMCS_64BIT_GUEST_IA32_SPEC_CTRL_HI:
+      return BX_SUPPORT_VMX_EXTENSION(BX_VMX_SPEC_CTRL_VIRTUALIZATION);
+#endif
+
 #if BX_SUPPORT_VMX >= 2
     /* VMCS 64-bit host state fields */
     /* binary 0010_11xx_xxxx_xxx0 */
@@ -483,6 +489,12 @@ bool BX_CPU_C::vmcs_field_supported(Bit32u encoding)
       return is_cpu_extension_supported(BX_ISA_PKS);
 #endif
 
+#if BX_SUPPORT_VMX >= 2
+    case VMCS_64BIT_HOST_IA32_SPEC_CTRL:
+    case VMCS_64BIT_HOST_IA32_SPEC_CTRL_HI:
+      return BX_SUPPORT_VMX_EXTENSION(BX_VMX_SPEC_CTRL_VIRTUALIZATION);
+#endif
+
     /* VMCS natural width control fields */
     /* binary 0110_00xx_xxxx_xxx0 */
     case VMCS_CONTROL_CR0_GUEST_HOST_MASK:
@@ -877,6 +889,8 @@ void BX_CPU_C::init_tertiary_proc_based_vmexec_ctrls(void)
   //   [06] Enable MSRLIST instructions
   //   [07] Virtualize IA32_SPEC_CTRL
   //    ...
+  //   [13] Emulate AVX10.VL256 mode
+  //    ...
 
   cap->vmx_vmexec_ctrl3_supported_bits = 0;
 
@@ -886,6 +900,9 @@ void BX_CPU_C::init_tertiary_proc_based_vmexec_ctrls(void)
   if (BX_SUPPORT_VMX_EXTENSION(BX_VMX_SPEC_CTRL_VIRTUALIZATION)) {
     cap->vmx_vmexec_ctrl3_supported_bits |= VMX_VM_EXEC_CTRL3_VIRTUALIZE_IA32_SPEC_CTRL;
   }
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX10_1) && BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX10_VL512)) {
+    cap->vmx_vmexec_ctrl3_supported_bits |= VMX_VM_EXEC_CTRL3_EMULATE_AVX10_VL256;
+  }
 }
 
 void BX_CPU_C::init_secondary_vmexit_ctrls(void)
@@ -894,8 +911,11 @@ void BX_CPU_C::init_secondary_vmexit_ctrls(void)
 
   // secondary vmexit controls
   // -----------------------------------------------------------
-  //    [02] Prematurely busy shadow stack control
+  //    [02] load host MSR_IA32_SPEC_CTRL control
+  //    [03] Prematurely busy shadow stack control
 
+  if (BX_SUPPORT_VMX_EXTENSION(BX_VMX_SPEC_CTRL_VIRTUALIZATION))
+    cap->vmx_vmexit_ctrl2_supported_bits |= VMX_VMEXIT_CTRL2_LOAD_HOST_IA32_SPEC_CTRL;
 #if BX_SUPPORT_CET
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_CET))
     cap->vmx_vmexit_ctrl2_supported_bits |= VMX_VMEXIT_CTRL2_SHADOW_STACK_BUSY_CTRL;
@@ -987,6 +1007,7 @@ void BX_CPU_C::init_vmentry_ctrls(void)
   //      [17] Load guest CET state
   //      [19] Load guest UINV state
   //      [22] Load guest MSR_IA32_PKRS value
+  //      [24] Load guest MSR_IA32_SPEC_CTRL value
 
   cap->vmx_vmentry_ctrl_supported_bits = VMX_VMENTRY_CTRL1_LOAD_DBG_CTRLS |
                                          VMX_VMENTRY_CTRL1_SMM_ENTER |
@@ -1016,6 +1037,8 @@ void BX_CPU_C::init_vmentry_ctrls(void)
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_PKS))
     cap->vmx_vmentry_ctrl_supported_bits |= VMX_VMENTRY_CTRL1_LOAD_GUEST_PKRS;
 #endif
+  if (BX_SUPPORT_VMX_EXTENSION(BX_VMX_SPEC_CTRL_VIRTUALIZATION))
+    cap->vmx_vmexit_ctrl2_supported_bits |= VMX_VMENTRY_CTRL1_LOAD_GUEST_IA32_SPEC_CTRL;
 }
 
 #endif // BX_SUPPORT_VMX
