@@ -319,6 +319,10 @@ typedef BxPackedYmmRegister BxPackedAvxRegister;
     { BX_CPU_THIS_PTR vmm[index].vmm64u(0) = (reg64); \
       BX_CPU_THIS_PTR vmm[index].vmm64u(1) = 0; BX_CLEAR_AVX_HIGH128(index); }
 
+#define BX_WRITE_XMM_REG_LO_DWORD_CLEAR_HIGH(index, reg32) \
+    { BX_CPU_THIS_PTR vmm[index].vmm64u(0) = Bit64u(reg32); \
+      BX_CPU_THIS_PTR vmm[index].vmm64u(1) = 0; BX_CLEAR_AVX_HIGH128(index); }
+
 #else /* BX_SUPPORT_AVX */
 
 /* write XMM register while clearing upper part of AVX register */
@@ -385,22 +389,21 @@ BX_CPP_INLINE int is_clear(const BxPackedXmmRegister *r)
   return (r->xmm64u(0) | r->xmm64u(1)) == 0;
 }
 
-#if BX_SUPPORT_EVEX
-// implement SAE and EVEX encoded rounding control
-BX_CPP_INLINE void softfloat_status_word_rc_override(float_status_t &status, bxInstruction_c *i)
-{
-  /* must be VL512 otherwise EVEX.LL encodes vector length */
-  if (i->modC0() && i->getEvexb()) {
-    status.float_rounding_mode = i->getRC();
-    status.float_suppress_exception = float_all_exceptions_mask;
-    status.float_exception_masks = float_all_exceptions_mask;
-  }
-}
+struct softfloat_status_t;
+
+#if BX_SUPPORT_EVEX == 0
+#define softfloat_status_word_rc_override(status, i)
 #else
-  #define softfloat_status_word_rc_override(status, i)
+extern void softfloat_status_word_rc_override(softfloat_status_t &status, bxInstruction_c *i);
 #endif
 
-#if BX_SUPPORT_FPU
+#if BX_SUPPORT_FPU || BX_SUPPORT_3DNOW
+/* convert float16 NaN number to QNaN */
+BX_CPP_INLINE float16 convert_to_QNaN(float16 op)
+{
+  return op | 0x7E00;
+}
+
 /* convert float32 NaN number to QNaN */
 BX_CPP_INLINE float32 convert_to_QNaN(float32 op)
 {
