@@ -24,6 +24,7 @@
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
+#include "cpuid.h"
 #include "msr.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
@@ -203,7 +204,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XSAVEC(bxInstruction_c *i)
 
   if ((xstate_bv & BX_XCR0_SSE_MASK) != 0)
   {
-    xsave_sse_state(i, eaddr);
+    xsave_sse_state(i, eaddr+XSAVE_SSE_STATE_OFFSET);
   }
 
   Bit32u offset = XSAVE_YMM_STATE_OFFSET;
@@ -360,12 +361,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
   // Legacy form of XRSTOR loads the MXCSR register from memory whenever the
   // RFBM[1](SSE) or RFBM[2](AVX) is set, regardless of the values of XSTATE_BV[1] and XSTATE_BV[2]
   if (((requested_feature_bitmap & (BX_XCR0_SSE_MASK|BX_XCR0_YMM_MASK)) != 0 && ! compaction) ||
-      ((requested_feature_bitmap & restore_mask & BX_XCR0_SSE_MASK) != 0))
+      ((requested_feature_bitmap & restore_mask & BX_XCR0_SSE_MASK) != 0 && compaction))
   {
     // read cannot cause any boundary cross because XSAVE image is 64-byte aligned
     Bit32u new_mxcsr = read_virtual_dword(i->seg(), eaddr + 24);
-    if(new_mxcsr & ~MXCSR_MASK)
+    if(new_mxcsr & ~MXCSR_MASK) {
+       BX_ERROR(("%s: corrupted MXCSR state restored new_mxcsr=0x%08x", i->getIaOpcodeNameShort(), new_mxcsr));
        exception(BX_GP_EXCEPTION, 0);
+    }
     BX_MXCSR_REGISTER = new_mxcsr;
   }
 
@@ -513,9 +516,9 @@ void BX_CPU_C::xsave_x87_state(bxInstruction_c *i, bx_address offset)
   {
     const floatx80 &fp = BX_READ_FPU_REG(index);
 
-    xmm.xmm64u(0) = fp.fraction;
+    xmm.xmm64u(0) = fp.signif;
     xmm.xmm64u(1) = 0;
-    xmm.xmm16u(4) = fp.exp;
+    xmm.xmm16u(4) = fp.signExp;
 
     write_virtual_xmmword(i->seg(), (offset+index*16+32) & asize_mask, &xmm);
   }
@@ -523,35 +526,35 @@ void BX_CPU_C::xsave_x87_state(bxInstruction_c *i, bx_address offset)
 
 void BX_CPU_C::xrstor_x87_state(bxInstruction_c *i, bx_address offset)
 {
+  i387_t restore_i387;
   BxPackedXmmRegister xmm;
   bx_address asize_mask = i->asize_mask();
 
   // load FPU state from XSAVE area
   read_virtual_xmmword(i->seg(), offset, &xmm);
 
-  BX_CPU_THIS_PTR the_i387.cwd =  xmm.xmm16u(0);
-  BX_CPU_THIS_PTR the_i387.swd =  xmm.xmm16u(1);
-  BX_CPU_THIS_PTR the_i387.tos = (xmm.xmm16u(1) >> 11) & 0x07;
+  restore_i387.cwd =  xmm.xmm16u(0);
+  restore_i387.swd =  xmm.xmm16u(1);
+  restore_i387.tos = (xmm.xmm16u(1) >> 11) & 0x07;
 
   /* always set bit 6 as '1 */
-  BX_CPU_THIS_PTR the_i387.cwd =
-    (BX_CPU_THIS_PTR the_i387.cwd & ~FPU_CW_Reserved_Bits) | 0x0040;
+  restore_i387.cwd = (restore_i387.cwd & ~FPU_CW_Reserved_Bits) | 0x0040;
 
   /* Restore x87 FPU Opcode */
   /* The lower 11 bits contain the FPU opcode, upper 5 bits are reserved */
-  BX_CPU_THIS_PTR the_i387.foo = xmm.xmm16u(3) & 0x7FF;
+  restore_i387.foo = xmm.xmm16u(3) & 0x7FF;
 
   /* Restore x87 FPU IP */
 #if BX_SUPPORT_X86_64
   if (i->os64L()) {
-    BX_CPU_THIS_PTR the_i387.fip = xmm.xmm64u(1);
-    BX_CPU_THIS_PTR the_i387.fcs = 0;
+    restore_i387.fip = xmm.xmm64u(1);
+    restore_i387.fcs = 0;
   }
   else
 #endif
   {
-    BX_CPU_THIS_PTR the_i387.fip = xmm.xmm32u(2);
-    BX_CPU_THIS_PTR the_i387.fcs = xmm.xmm16u(6);
+    restore_i387.fip = xmm.xmm32u(2);
+    restore_i387.fcs = xmm.xmm16u(6);
   }
 
   Bit32u tag_byte = xmm.xmmubyte(4);
@@ -561,30 +564,32 @@ void BX_CPU_C::xrstor_x87_state(bxInstruction_c *i, bx_address offset)
 
 #if BX_SUPPORT_X86_64
   if (i->os64L()) {
-    BX_CPU_THIS_PTR the_i387.fdp = xmm.xmm64u(0);
-    BX_CPU_THIS_PTR the_i387.fds = 0;
+    restore_i387.fdp = xmm.xmm64u(0);
+    restore_i387.fds = 0;
   }
   else
 #endif
   {
-    BX_CPU_THIS_PTR the_i387.fdp = xmm.xmm32u(0);
-    BX_CPU_THIS_PTR the_i387.fds = xmm.xmm16u(2);
+    restore_i387.fdp = xmm.xmm32u(0);
+    restore_i387.fds = xmm.xmm16u(2);
   }
 
   /* load i387 register file */
   for(unsigned index=0; index < 8; index++)
   {
     floatx80 reg;
-    reg.fraction = read_virtual_qword(i->seg(), (offset+index*16+32) & asize_mask);
-    reg.exp      = read_virtual_word (i->seg(), (offset+index*16+40) & asize_mask);
+    reg.signif  = read_virtual_qword(i->seg(), (offset+index*16+32) & asize_mask);
+    reg.signExp = read_virtual_word (i->seg(), (offset+index*16+40) & asize_mask);
 
     // update tag only if it is not empty
-    BX_WRITE_FPU_REGISTER_AND_TAG(reg,
+    restore_i387.FPU_save_regi(reg,
               IS_TAG_EMPTY(index) ? FPU_Tag_Empty : FPU_tagof(reg), index);
   }
 
   /* Restore floating point tag word - see desription for FXRSTOR instruction */
-  BX_CPU_THIS_PTR the_i387.twd = unpack_FPU_TW(tag_byte);
+  restore_i387.twd = unpack_FPU_TW(&restore_i387, tag_byte);
+
+  BX_CPU_THIS_PTR the_i387 = restore_i387;
 
   /* check for unmasked exceptions */
   if (FPU_PARTIAL_STATUS & ~FPU_CONTROL_WORD & FPU_CW_Exceptions_Mask) {
@@ -619,7 +624,7 @@ bool BX_CPU_C::xsave_x87_state_xinuse(void)
 
   for (unsigned index=0;index<8;index++) {
     floatx80 reg = BX_FPU_REG(index);
-    if (reg.exp != 0 || reg.fraction != 0) return true;
+    if (reg.signExp != 0 || reg.signif != 0) return true;
   }
 
   return false;
@@ -779,6 +784,10 @@ bool BX_CPU_C::xsave_opmask_state_xinuse(void)
 // Outside 64-bit mode, ZMM_Hi256 state is in its initial configuration if each of ZMM0_H-ZMM7_H is 0.
 // An execution of XRSTOR or XRSTORS outside 64-bit mode does not update ZMM8_H-ZMM15_H.
 
+// In processor which support only AVX10.VL256 but not AVX10.VL512 ZMM_Hi256 state always tracked as INIT
+// XRSTOR* ignore the contents of the memory corresponding to ZMM_Hi256 state and the corresponding XSTATE_BV bit. 
+// XSAVE zero the memory corresponding ZMM_Hi256, other XSAVE* instructions incorporate INIT optimization.
+
 void BX_CPU_C::xsave_zmm_hi256_state(bxInstruction_c *i, bx_address offset)
 {
   unsigned num_regs = long64_mode() ? 16 : 8;
@@ -786,15 +795,33 @@ void BX_CPU_C::xsave_zmm_hi256_state(bxInstruction_c *i, bx_address offset)
   bx_address asize_mask = i->asize_mask();
 
   // save upper part of ZMM registers to XSAVE area
-  for(unsigned index=0; index < num_regs; index++) {
-    write_virtual_ymmword(i->seg(), (offset+index*32) & asize_mask, &BX_READ_ZMM_REG_HI(index));
+  if (! BX_CPU_THIS_PTR cpuid->support_avx10_512()) {
+    for(unsigned index=0; index < num_regs; index++) {
+      static BxPackedYmmRegister empty_reg;
+      write_virtual_ymmword(i->seg(), (offset+index*32) & asize_mask, &empty_reg);
+    }
+  }
+  else {
+    for(unsigned index=0; index < num_regs; index++) {
+      write_virtual_ymmword(i->seg(), (offset+index*32) & asize_mask, &BX_READ_ZMM_REG_HI(index));
+    }
   }
 }
 
 void BX_CPU_C::xrstor_zmm_hi256_state(bxInstruction_c *i, bx_address offset)
 {
+  if (! BX_CPU_THIS_PTR cpuid->support_avx10_512()) return;
+
   unsigned num_regs = long64_mode() ? 16 : 8;
 
+#if BX_SUPPORT_VMX
+  if (BX_CPU_THIS_PTR in_vmx_guest && BX_CPU_THIS_PTR vmcs.vmexec_ctrls3.EMULATE_AVX10_VL256()) {
+    for(unsigned index=0; index < num_regs; index++)
+      BX_CLEAR_AVX_HIGH256(index);
+    return;
+  }
+#endif
+
   bx_address asize_mask = i->asize_mask();
 
   // load upper part of ZMM registers from XSAVE area
@@ -815,6 +842,8 @@ void BX_CPU_C::xrstor_init_zmm_hi256_state(void)
 
 bool BX_CPU_C::xsave_zmm_hi256_state_xinuse(void)
 {
+  if (! BX_CPU_THIS_PTR cpuid->support_avx10_512()) return false;
+
   unsigned num_regs = long64_mode() ? 16 : 8;
 
   for(unsigned index=0; index < num_regs; index++) {
@@ -833,15 +862,31 @@ bool BX_CPU_C::xsave_zmm_hi256_state_xinuse(void)
 // Outside 64-bit mode, Hi16_ZMM state is always in its initial configuration.
 // An execution of XRSTOR or XRSTORS outside 64-bit mode does not update ZMM16-ZMM31.
 
+// In processor which support only AVX10.VL256 but not AVX10.VL512:
+// XSAVE* save the lower 256 bits of each ZMM registers in Hi16_ZMM state and zero the memory
+// corresponding to upper 256 bits
+// XRSTOR* restore the lower 256 bit of each ZMM register in Hi16_ZMM state and ignore the contents of
+// the memory corresponding to upper 256 bits.
+
 void BX_CPU_C::xsave_hi_zmm_state(bxInstruction_c *i, bx_address offset)
 {
   if (!long64_mode()) return;
 
   bx_address asize_mask = i->asize_mask();
 
-  // save high ZMM state to XSAVE area
-  for(unsigned index=0; index < 16; index++) {
-    write_virtual_zmmword(i->seg(), (offset+index*64) & asize_mask, &BX_READ_AVX_REG(index+16));
+  if (! BX_CPU_THIS_PTR cpuid->support_avx10_512()) {
+    // save lower 256-bit of high ZMM state to XSAVE area and zero upper 256-bit
+    for(unsigned index=0; index < 16; index++) {
+      BxPackedZmmRegister reg = BX_READ_AVX_REG(index+16);
+      reg.vmm256(1).clear();
+      write_virtual_zmmword(i->seg(), (offset+index*64) & asize_mask, &reg);
+    }
+  }
+  else {
+    // save high ZMM state to XSAVE area
+    for(unsigned index=0; index < 16; index++) {
+      write_virtual_zmmword(i->seg(), (offset+index*64) & asize_mask, &BX_READ_AVX_REG(index+16));
+    }
   }
 }
 
@@ -851,9 +896,26 @@ void BX_CPU_C::xrstor_hi_zmm_state(bxInstruction_c *i, bx_address offset)
 
   bx_address asize_mask = i->asize_mask();
 
-  // load high ZMM state from XSAVE area
-  for(unsigned index=0; index < 16; index++) {
-    read_virtual_zmmword(i->seg(), (offset+index*64) & asize_mask, &BX_READ_AVX_REG(index+16));
+  bool support_avx10_512 = BX_CPU_THIS_PTR cpuid->support_avx10_512();
+#if BX_SUPPORT_VMX
+  if (BX_CPU_THIS_PTR in_vmx_guest && BX_CPU_THIS_PTR vmcs.vmexec_ctrls3.EMULATE_AVX10_VL256())
+    support_avx10_512 = false;
+#endif
+
+  if (! support_avx10_512) {
+    // Restore the lower 256-bit of each ZMM register and ignore the contents of upper 256-bit.
+    // Zero the upper 256 bits of each ZMM register in Hi16_ZMM state irrespective of the
+    // corresponding XSTATE_BV value.
+    for(unsigned index=0; index < 16; index++) {
+      read_virtual_ymmword(i->seg(), (offset+index*64) & asize_mask, &BX_READ_YMM_REG(index+16));
+      BX_CLEAR_AVX_HIGH256(index+16);
+    }
+  }
+  else {
+    // load high ZMM state from XSAVE area
+    for(unsigned index=0; index < 16; index++) {
+      read_virtual_zmmword(i->seg(), (offset+index*64) & asize_mask, &BX_READ_AVX_REG(index+16));
+    }
   }
 }
 
