@@ -26,8 +26,13 @@
 // header bar and status bar stuff
 #define BX_HEADER_BAR_Y 32
 
-#define BX_MAX_PIXMAPS 17
-#define BX_MAX_HEADERBAR_ENTRIES 12
+#if BX_USB_DEBUGGER
+  #define BX_MAX_PIXMAPS 19
+  #define BX_MAX_HEADERBAR_ENTRIES 13
+#else
+  #define BX_MAX_PIXMAPS 17
+  #define BX_MAX_HEADERBAR_ENTRIES 12
+#endif
 
 // align pixmaps towards left or right side of header bar
 #define BX_GRAVITY_LEFT 10
@@ -42,7 +47,12 @@
 #define BX_GUI_DLG_RUNTIME      0x08
 #define BX_GUI_DLG_USER         0x10
 #define BX_GUI_DLG_SAVE_RESTORE 0x20
-#define BX_GUI_DLG_ALL          0x3F
+#if BX_USB_DEBUGGER
+  #define BX_GUI_DLG_USB          0x40
+  #define BX_GUI_DLG_ALL          0x7F
+#else
+  #define BX_GUI_DLG_ALL          0x3F
+#endif
 
 // text mode blink feature
 #define BX_TEXT_BLINK_MODE      0x01
@@ -60,15 +70,23 @@
 #define BX_MT_KEY_ALT           0x02
 #define BX_MT_KEY_F10           0x04
 #define BX_MT_KEY_F12           0x08
-#define BX_MT_MBUTTON           0x10
-#define BX_MT_LBUTTON           0x20
-#define BX_MT_RBUTTON           0x40
+#define BX_MT_KEY_G             0x10
+#define BX_MT_MBUTTON           0x20
+#define BX_MT_LBUTTON           0x40
+#define BX_MT_RBUTTON           0x80
 
 #define BX_GUI_MT_CTRL_MB       (BX_MT_KEY_CTRL | BX_MT_MBUTTON)
 #define BX_GUI_MT_CTRL_LRB      (BX_MT_KEY_CTRL | BX_MT_LBUTTON | BX_MT_RBUTTON)
 #define BX_GUI_MT_CTRL_F10      (BX_MT_KEY_CTRL | BX_MT_KEY_F10)
 #define BX_GUI_MT_F12           (BX_MT_KEY_F12)
 #define BX_GUI_MT_CTRL_ALT      (BX_MT_KEY_CTRL | BX_MT_KEY_ALT)
+#define BX_GUI_MT_CTRL_ALT_G    (BX_MT_KEY_CTRL | BX_MT_KEY_ALT | BX_MT_KEY_G)
+
+// display library option flags
+#define BX_GUI_OPT_HIDE_IPS       0x01
+#define BX_GUI_OPT_NOKEYREPEAT    0x02
+#define BX_GUI_OPT_CMDMODE        0x04
+#define BX_GUI_OPT_NO_GUI_CONSOLE 0x08
 
 typedef struct {
   Bit16u  start_address;
@@ -98,9 +116,9 @@ BOCHSAPI_MSVCONLY Bit8u reverse_bitorder(Bit8u);
 BOCHSAPI extern class bx_gui_c *bx_gui;
 
 #if BX_SUPPORT_X86_64
-  #define BOCHS_WINDOW_NAME "Bochs x86-64 emulator, http://bochs.sourceforge.net/"
+  #define BOCHS_WINDOW_NAME "Bochs x86-64 emulator, https://bochs.sourceforge.io/"
 #else
-  #define BOCHS_WINDOW_NAME "Bochs x86 emulator, http://bochs.sourceforge.net/"
+  #define BOCHS_WINDOW_NAME "Bochs x86 emulator, https://bochs.sourceforge.io/"
 #endif
 
 // The bx_gui_c class provides data and behavior that is common to
@@ -200,7 +218,7 @@ public:
   Bit8u set_modifier_keys(Bit8u modifier, bool pressed);
   bool parse_user_shortcut(const char *val);
 #if BX_DEBUGGER && BX_DEBUGGER_GUI
-  void init_debug_dialog(void);
+  void init_debug_dialog(bool global_ini);
   void close_debug_dialog(void);
 #endif
 #if BX_USE_GUI_CONSOLE
@@ -222,6 +240,9 @@ public:
   void set_fullscreen_mode(bool active) {fullscreen_mode = active;}
   // marklog handler without button, called in gui command mode
   static void marklog_handler(void);
+#if BX_USB_DEBUGGER
+  void set_usbdbg_bitmap(bool trigger);
+#endif
 
 protected:
   // And these are defined and used privately in gui.cc
@@ -235,6 +256,9 @@ protected:
   static void paste_handler(void);
   static void snapshot_handler(void);
   static void config_handler(void);
+#if BX_USB_DEBUGGER
+  static void usb_handler(void);
+#endif
   static void userbutton_handler(void);
   static void save_restore_handler(void);
   // process clicks on the "classic" Bochs headerbar
@@ -251,6 +275,7 @@ protected:
 #else
   void console_cleanup(void) {}
 #endif
+  bool parse_common_gui_options(const char *arg, Bit8u flags);
 
   // header bar buttons
   bool floppyA_status;
@@ -268,6 +293,9 @@ protected:
   unsigned mouse_bmap_id, nomouse_bmap_id, mouse_hbar_id;
   unsigned user_bmap_id, user_hbar_id;
   unsigned save_restore_bmap_id, save_restore_hbar_id;
+#if BX_USB_DEBUGGER
+  unsigned usbdbg_bmap_id, usbdbg_dis_bmap_id, usbdbg_trigger_bmap_id, usbdbg_hbar_id;
+#endif
   // the "classic" Bochs headerbar
   unsigned bx_headerbar_entries;
   struct {
@@ -367,6 +395,17 @@ protected:
   } command_mode;
   bool fullscreen_mode;
   Bit32u marker_count;
+  // display library options
+  struct {
+#if BX_SHOW_IPS
+    bool hide_ips;
+#endif
+    bool nokeyrepeat;
+#if BX_DEBUGGER && BX_DEBUGGER_GUI
+    bool enh_dbg_enabled;
+    bool enh_dbg_global_ini;
+#endif
+  } gui_opts;
 };
 
 
