@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: paramtree.cc 14206 2021-03-28 06:31:03Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2010-2021  The Bochs Project
+//  Copyright (C) 2010-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -40,6 +40,8 @@ bx_param_c::bx_param_c(Bit32u id, const char *param_name, const char *param_desc
     parent(NULL),
     description(NULL),
     label(NULL),
+    text_format(NULL),
+    long_text_format(NULL),
     ask_format(NULL),
     group_name(NULL)
 {
@@ -47,8 +49,8 @@ bx_param_c::bx_param_c(Bit32u id, const char *param_name, const char *param_desc
   this->name = new char[strlen(param_name)+1];
   strcpy(this->name, param_name);
   set_description(param_desc);
-  this->text_format = default_text_format;
-  this->long_text_format = default_text_format;
+  set_format(default_text_format);
+  set_long_format(default_text_format);
   this->runtime_param = 0;
   this->enabled = 1;
   this->options = 0;
@@ -62,6 +64,8 @@ bx_param_c::bx_param_c(Bit32u id, const char *param_name, const char *param_labe
     parent(NULL),
     description(NULL),
     label(NULL),
+    text_format(NULL),
+    long_text_format(NULL),
     ask_format(NULL),
     group_name(NULL)
 {
@@ -70,8 +74,8 @@ bx_param_c::bx_param_c(Bit32u id, const char *param_name, const char *param_labe
   strcpy(this->name, param_name);
   set_description(param_desc);
   set_label(param_label);
-  this->text_format = default_text_format;
-  this->long_text_format = default_text_format;
+  set_format(default_text_format);
+  set_long_format(default_text_format);
   this->runtime_param = 0;
   this->enabled = 1;
   this->options = 0;
@@ -85,6 +89,8 @@ bx_param_c::~bx_param_c()
   delete [] name;
   delete [] label;
   delete [] description;
+  delete [] text_format;
+  delete [] long_text_format;
   delete [] ask_format;
   delete [] group_name;
   delete dependent_list;
@@ -112,6 +118,28 @@ void bx_param_c::set_label(const char *text)
   }
 }
 
+void bx_param_c::set_format(const char *format)
+{
+  delete [] text_format;
+  if (format) {
+    text_format = new char[strlen(format)+1];
+    strcpy(text_format, format);
+  } else {
+    text_format = NULL;
+  }
+}
+
+void bx_param_c::set_long_format(const char *format)
+{
+  delete [] long_text_format;
+  if (format) {
+    long_text_format = new char[strlen(format)+1];
+    strcpy(long_text_format, format);
+  } else {
+    long_text_format = NULL;
+  }
+}
+
 void bx_param_c::set_ask_format(const char *format)
 {
   delete [] ask_format;
@@ -231,7 +259,7 @@ Bit64s bx_param_num_c::get64()
 
 void bx_param_num_c::set(Bit64s newval)
 {
-  if (!enabled) return;
+  if (!enabled && SIM->get_ci_started()) return;
 
   if (handler) {
     // the handler can override the new value and/or perform some side effect
@@ -371,7 +399,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p64bit = ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x" FMT_LL "x";
+    set_format("0x" FMT_LL "x");
   }
 }
 
@@ -390,7 +418,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p64bit = (Bit64s*) ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x" FMT_LL "x";
+    set_format("0x" FMT_LL "x");
   }
 }
 
@@ -409,7 +437,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p32bit = ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x%08x";
+    set_format("0x%08x");
   }
 }
 
@@ -428,7 +456,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p32bit = (Bit32s*) ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x%08x";
+    set_format("0x%08x");
   }
 }
 
@@ -447,7 +475,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p16bit = ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x%04x";
+    set_format("0x%04x");
   }
 }
 
@@ -466,7 +494,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p16bit = (Bit16s*) ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x%04x";
+    set_format("0x%04x");
   }
 }
 
@@ -486,7 +514,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p8bit = ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x%02x";
+    set_format("0x%02x");
   }
 }
 
@@ -505,7 +533,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p8bit = (Bit8s*) ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x%02x";
+    set_format("0x%02x");
   }
 }
 
@@ -611,7 +639,7 @@ int bx_param_bool_c::parse_param(const char *ptr)
   if (ptr != NULL) {
     if (!strcmp(ptr, "0") || !stricmp(ptr, "false")) {
       set(0); return 1;
-    } 
+    }
     if (!strcmp(ptr, "1") || !stricmp(ptr, "true")) {
       set(1); return 1;
     }
