@@ -110,11 +110,13 @@ static unsigned statusitem_pos[12] = {
 };
 static bool statusitem_active[12];
 #if BX_SHOW_IPS
-static bool sdl_hide_ips = 0;
 static bool sdl_ips_update = 0;
 static char sdl_ips_text[20];
 static bool sdl_show_info_msg = 0;
 #endif
+#if defined(WIN32) && BX_DEBUGGER && BX_DEBUGGER_GUI
+bool sdl_enh_dbg_global_ini;
+#endif
 
 
 static void sdl_set_status_text(int element, const char *text, bool active, bool color = 0)
@@ -187,7 +189,13 @@ static Bit32u sdl_sym_to_bx_key(SDLKey sym)
     case SDLK_BACKSPACE:            return BX_KEY_BACKSPACE;
     case SDLK_TAB:                  return BX_KEY_TAB;
     case SDLK_RETURN:               return BX_KEY_ENTER;
-    case SDLK_PAUSE:                return BX_KEY_PAUSE;
+    case SDLK_PAUSE:
+      if (bx_gui->get_modifier_keys() & BX_MOD_KEY_CTRL) {
+        return BX_KEY_CTRL_BREAK;
+      } else {
+        return BX_KEY_PAUSE;
+      }
+      break;
     case SDLK_ESCAPE:               return BX_KEY_ESC;
     case SDLK_SPACE:                return BX_KEY_SPACE;
     case SDLK_QUOTE:                return BX_KEY_SINGLE_QUOTE;
@@ -303,7 +311,7 @@ static Bit32u sdl_sym_to_bx_key(SDLKey sym)
 
 /* Miscellaneous function keys */
     case SDLK_PRINT:                return BX_KEY_PRINT;
-    case SDLK_BREAK:                return BX_KEY_PAUSE;
+    case SDLK_BREAK:                return BX_KEY_CTRL_BREAK;
     case SDLK_MENU:                 return BX_KEY_MENU;
 
     default:
@@ -333,14 +341,14 @@ void switch_to_fullscreen(void)
 {
   SDL_FreeSurface(sdl_screen);
   sdl_screen = NULL;
-#ifdef ANDROID
+#ifdef __ANDROID__
   sdl_fullscreen = SDL_SetVideoMode(res_x,res_y,32, SDL_SWSURFACE|SDL_FULLSCREEN);
 #else
   sdl_fullscreen = SDL_SetVideoMode(res_x,res_y,32, SDL_HWSURFACE|SDL_FULLSCREEN);
 #endif
   DEV_vga_refresh(1);
   SDL_ShowCursor(0);
-#ifndef ANDROID
+#ifndef __ANDROID__
   if (sdl_grab == 0) {
     SDL_WM_GrabInput(SDL_GRAB_ON);
     sdl_grab = 1;
@@ -375,7 +383,7 @@ DWORD WINAPI DebugGuiThread(LPVOID)
 {
   MSG msg;
 
-  bx_gui->init_debug_dialog();
+  bx_gui->init_debug_dialog(sdl_enh_dbg_global_ini);
   while (GetMessage(&msg, NULL, 0, 0)) {
     TranslateMessage(&msg);
     DispatchMessage(&msg);
@@ -416,7 +424,7 @@ bx_sdl_gui_c::bx_sdl_gui_c()
   atexit(SDL_Quit);
 #endif
 
-#ifdef ANDROID
+#ifdef __ANDROID__
   modes = SDL_ListModes(NULL, SDL_FULLSCREEN|SDL_SWSURFACE);
 #else
   modes = SDL_ListModes(NULL, SDL_FULLSCREEN|SDL_HWSURFACE);
@@ -467,7 +475,7 @@ void bx_sdl_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 
   // load keymap for sdl
   if (SIM->get_param_bool(BXPN_KBD_USEMAPPING)->get()) {
-    bx_keymap.loadKeymap(convertStringToSDLKey);
+    bx_keymap.loadKeymap("sdl", convertStringToSDLKey);
   }
 
   if (!gui_ci) {
@@ -475,46 +483,43 @@ void bx_sdl_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
   }
 
   // parse sdl specific options
+  Bit8u flags = BX_GUI_OPT_NOKEYREPEAT | BX_GUI_OPT_HIDE_IPS  | BX_GUI_OPT_CMDMODE
+                | BX_GUI_OPT_NO_GUI_CONSOLE;
   if (argc > 1) {
     for (i = 1; i < argc; i++) {
-      if (!strcmp(argv[i], "fullscreen")) {
-        sdl_fullscreen_toggle = 1;
-        switch_to_fullscreen();
-      } else if (!strcmp(argv[i], "nokeyrepeat")) {
-        BX_INFO(("disabled host keyboard repeat"));
-        SDL_EnableKeyRepeat(0, 0);
-      } else if (!strcmp(argv[i], "gui_debug")) {
-#if BX_DEBUGGER && BX_DEBUGGER_GUI
-        SIM->set_debug_gui(1);
-#ifdef WIN32
-        if (gui_ci) {
-          // on Windows the debugger gui must run in a separate thread
-          DWORD threadID;
-          CreateThread(NULL, 0, DebugGuiThread, NULL, 0, &threadID);
+      if (!parse_common_gui_options(argv[i], flags)) {
+        if (!strcmp(argv[i], "fullscreen")) {
+          sdl_fullscreen_toggle = 1;
+          switch_to_fullscreen();
         } else {
-          BX_PANIC(("Config interface 'win32config' is required for gui debugger"));
+          BX_PANIC(("Unknown sdl option '%s'", argv[i]));
         }
-#else
-        init_debug_dialog();
-#endif
-#else
-        SIM->message_box("ERROR", "Bochs debugger not available - ignoring 'gui_debug' option");
-#endif
-#if BX_SHOW_IPS
-      } else if (!strcmp(argv[i], "hideIPS")) {
-        BX_INFO(("hide IPS display in status bar"));
-        sdl_hide_ips = 1;
-#endif
-      } else if (!strcmp(argv[i], "cmdmode")) {
-        command_mode.present = 1;
-      } else if (!strcmp(argv[i], "no_gui_console")) {
-        console.present = 0;
-      } else {
-        BX_PANIC(("Unknown sdl option '%s'", argv[i]));
       }
     }
   }
 
+  if (gui_opts.nokeyrepeat) {
+    SDL_EnableKeyRepeat(0, 0);
+  }
+
+#if BX_DEBUGGER && BX_DEBUGGER_GUI
+  if (bx_dbg.debugger_active && gui_opts.enh_dbg_enabled) {
+    SIM->set_debug_gui(1);
+#ifdef WIN32
+    if (gui_ci) {
+      sdl_enh_dbg_global_ini = gui_opts.enh_dbg_global_ini;
+      // on Windows the debugger gui must run in a separate thread
+      DWORD threadID;
+      CreateThread(NULL, 0, DebugGuiThread, NULL, 0, &threadID);
+    } else {
+      BX_PANIC(("Config interface 'win32config' is required for gui debugger"));
+    }
+#else
+    init_debug_dialog(gui_opts.enh_dbg_global_ini);
+#endif
+  }
+#endif
+
   new_gfx_api = 1;
   new_text_api = 1;
 #if defined(WIN32) && BX_SHOW_IPS
@@ -539,7 +544,7 @@ void bx_sdl_gui_c::draw_char(Bit8u ch, Bit8u fc, Bit8u bc, Bit16u xc, Bit16u yc,
     buf = (Uint32 *)sdl_screen->pixels + (headerbar_height + yc) * pitch + xc;
   } else {
     pitch = sdl_fullscreen->pitch/4;
-    buf = (Uint32 *)sdl_fullscreen->pixels + yc * pitch + xc;
+    buf = (Uint32 *)sdl_fullscreen->pixels + yc * pitch + xc + sdl_fullscreen->offset/4;
   }
   fgcolor = sdl_palette[fc];
   bgcolor = sdl_palette[bc];
@@ -762,10 +767,20 @@ void bx_sdl_gui_c::handle_events(void)
           SDLKey keysym = sdl_event.key.keysym.sym;
           Bit8u ascii = (Bit8u)sdl_event.key.keysym.unicode;
           if (((keysym >= SDLK_SPACE) && (keysym < SDLK_DELETE)) ||
-              (keysym == SDLK_RETURN) || (keysym == SDLK_BACKSPACE)) {
+              (keysym == SDLK_RETURN) || (keysym == SDLK_KP_ENTER) ||
+              (keysym == SDLK_BACKSPACE)) {
             if (ascii < 0x80) {
               console_key_enq(ascii);
             }
+#ifdef __ANDROID__
+            if (keysym == SDLK_RETURN) {
+              console_key_enq(13);
+            }
+#else
+            if (keysym == SDLK_KP_ENTER) {
+              console_key_enq(13);
+            }
+#endif
           }
           break;
         }
@@ -779,6 +794,8 @@ void bx_sdl_gui_c::handle_events(void)
             mouse_toggle = mouse_toggle_check(BX_MT_KEY_F10, 1);
           } else if (sdl_event.key.keysym.sym == SDLK_F12) {
             mouse_toggle = mouse_toggle_check(BX_MT_KEY_F12, 1);
+          } else if (sdl_event.key.keysym.sym == SDLK_g) {
+            mouse_toggle = mouse_toggle_check(BX_MT_KEY_G, 1);
           }
           if (mouse_toggle) {
             toggle_mouse_enable();
@@ -899,12 +916,12 @@ void bx_sdl_gui_c::handle_events(void)
           mouse_toggle_check(BX_MT_KEY_F10, 0);
         } else if (sdl_event.key.keysym.sym == SDLK_F12) {
           mouse_toggle_check(BX_MT_KEY_F12, 0);
+        } else if (sdl_event.key.keysym.sym == SDLK_g) {
+          mouse_toggle_check(BX_MT_KEY_G, 0);
         }
 
-        // filter out release of Windows/Fullscreen toggle and unsupported keys
-        if ((sdl_event.key.keysym.sym != SDLK_SCROLLOCK)
-           && (sdl_event.key.keysym.sym < SDLK_LAST))
-        {
+        // filter out release of unsupported keys
+        if (sdl_event.key.keysym.sym < SDLK_LAST) {
           // convert sym->bochs code
           if (!SIM->get_param_bool(BXPN_KBD_USEMAPPING)->get()) {
             key_event = sdl_sym_to_bx_key(sdl_event.key.keysym.sym);
@@ -1007,7 +1024,7 @@ void bx_sdl_gui_c::dimension_update(unsigned x, unsigned y,
   guest_yres = y;
 
   if ((x == res_x) && (y == res_y)) return;
-#ifndef ANDROID
+#ifndef __ANDROID__
   // This is not needed on Android
   if (((int)x > sdl_maxres.w) || ((int)y > sdl_maxres.h)) {
     BX_PANIC(("dimension_update(): resolution of out of display bounds"));
@@ -1041,7 +1058,7 @@ void bx_sdl_gui_c::dimension_update(unsigned x, unsigned y,
         BX_HEADERBAR_BG_GREEN,
         BX_HEADERBAR_BG_BLUE);
   } else {
-#ifdef ANDROID
+#ifdef __ANDROID__
     sdl_fullscreen = SDL_SetVideoMode(x, y, 32, SDL_SWSURFACE|SDL_FULLSCREEN);
 #else
     sdl_fullscreen = SDL_SetVideoMode(x, y, 32, SDL_HWSURFACE|SDL_FULLSCREEN);
@@ -1432,7 +1449,7 @@ void bx_sdl_gui_c::set_mouse_mode_absxy(bool mode)
 #if BX_SHOW_IPS
 void bx_sdl_gui_c::show_ips(Bit32u ips_count)
 {
-  if (!sdl_hide_ips && !sdl_ips_update) {
+  if (!gui_opts.hide_ips && !sdl_ips_update) {
     ips_count /= 1000;
     sprintf(sdl_ips_text, "IPS: %u.%3.3uM", ips_count / 1000, ips_count % 1000);
     sdl_ips_update = 1;
