@@ -2,7 +2,7 @@
 // $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2003-2023  The Bochs Project
+//  Copyright (C) 2003-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,6 @@
 
 #include "win32dialog.h"
 #include "bochs.h"
-#include "bx_debug/debug.h"
 #include "param_names.h"
 #include "gui.h"
 #include "win32res.h"
@@ -122,7 +121,9 @@ static BOOL CALLBACK LogAskProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lPara
         SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Kill simulation");
         SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Abort (dump core)");
 #if BX_DEBUGGER
-        SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Continue and return to debugger");
+        if (bx_dbg.debugger_active) {
+          SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_ADDSTRING, 0, (LPARAM)"Continue and return to debugger");
+        }
 #endif
         SendMessage(GetDlgItem(hDlg, IDASKLIST), LB_SETCURSEL, 2, 0);
       } else {
@@ -463,9 +464,6 @@ edit_opts_t start_options[] = {
   {"Logfile", "log"},
   {"Log Options", "#logopts"},
   {"CPU", "cpu"},
-#if BX_CPU_LEVEL >= 4
-  {"CPUID", "cpuid"},
-#endif
   {"Memory", "memory"},
   {"Clock & CMOS", "clock_cmos"},
   {"PCI", "pci"},
@@ -782,13 +780,8 @@ static int win32_ci_callback(void *userdata, ci_command_t command)
       if (!bx_gui->has_gui_console()) {
         if (MainMenuDialog(GetBochsWindow(), 1) < 0) {
           bx_user_quit = 1;
-#if !BX_DEBUGGER
-          bx_atexit();
-          SIM->quit_sim(1);
-#else
-          bx_dbg_exit(1);
-#endif
-          return -1;
+        } else {
+          return 1;
         }
       }
       break;
