@@ -40,6 +40,8 @@
 #include "vga.h"
 #include "virt_timer.h"
 
+#include "bx_debug/debug.h"
+
 #define LOG_THIS theVga->
 
 bx_vga_c *theVga = NULL;
@@ -91,16 +93,16 @@ bool bx_vga_c::init_vga_extension(void)
       DEV_register_ioread_handler(this, vbe_read_handler, addr, "vga video", 7);
       DEV_register_iowrite_handler(this, vbe_write_handler, addr, "vga video", 7);
     }
+    BX_VGA_THIS s.memsize = atoi(SIM->get_param_enum(BXPN_VBE_MEMSIZE)->get_selected()) << 20;
     if (!BX_VGA_THIS pci_enabled) {
       BX_VGA_THIS vbe.base_address = VBE_DISPI_LFB_PHYSICAL_ADDRESS;
       DEV_register_memory_handlers(theVga, mem_read_handler, mem_write_handler,
                                    BX_VGA_THIS vbe.base_address,
-                                   BX_VGA_THIS vbe.base_address + VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES - 1);
+                                   BX_VGA_THIS vbe.base_address + BX_VGA_THIS s.memsize - 1);
     }
     if (BX_VGA_THIS s.memory == NULL)
-      BX_VGA_THIS s.memory = new Bit8u[VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES];
-    memset(BX_VGA_THIS s.memory, 0, VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES);
-    BX_VGA_THIS s.memsize = VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES;
+      BX_VGA_THIS s.memory = new Bit8u[BX_VGA_THIS s.memsize];
+    memset(BX_VGA_THIS s.memory, 0, BX_VGA_THIS s.memsize);
     BX_VGA_THIS vbe.cur_dispi=VBE_DISPI_ID0;
     BX_VGA_THIS vbe.xres=640;
     BX_VGA_THIS vbe.yres=480;
@@ -124,7 +126,7 @@ bool bx_vga_c::init_vga_extension(void)
     BX_VGA_THIS vbe_present = 1;
     ret = 1;
 
-    BX_INFO(("VBE Bochs Display Extension Enabled"));
+    BX_INFO(("VBE Bochs Display Extension Enabled (%d MB)", BX_VGA_THIS s.memsize >> 20));
   }
 #if BX_SUPPORT_PCI
   Bit8u devfunc = 0x00;
@@ -140,7 +142,7 @@ bool bx_vga_c::init_vga_extension(void)
 
     if (BX_VGA_THIS vbe_present) {
       BX_VGA_THIS pci_conf[0x10] = 0x08;
-      BX_VGA_THIS init_bar_mem(0, VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES,
+      BX_VGA_THIS init_bar_mem(0, BX_VGA_THIS s.memsize,
                                mem_read_handler, mem_write_handler);
     }
     BX_VGA_THIS pci_rom_address = 0;
@@ -578,9 +580,6 @@ void bx_vga_c::update(void)
     } else {
       unsigned r, c, x, y;
       unsigned xc, yc, xti, yti;
-#ifndef VGA_MEM_FIX
-      Bit8u *plane[4];
-#endif
       Bit32u row_addr;
 
       if (BX_VGA_THIS vbe.yres < 1024) {
@@ -597,12 +596,6 @@ void bx_vga_c::update(void)
         BX_VGA_THIS s.last_bpp = 8;
       }
 
-#ifndef VGA_MEM_FIX
-      plane[0] = &BX_VGA_THIS s.memory[0<<VBE_DISPI_4BPP_PLANE_SHIFT];
-      plane[1] = &BX_VGA_THIS s.memory[1<<VBE_DISPI_4BPP_PLANE_SHIFT];
-      plane[2] = &BX_VGA_THIS s.memory[2<<VBE_DISPI_4BPP_PLANE_SHIFT];
-      plane[3] = &BX_VGA_THIS s.memory[3<<VBE_DISPI_4BPP_PLANE_SHIFT];
-#endif
       for (yc=0, yti=0; yc<iHeight; yc+=Y_TILESIZE, yti++) {
         for (xc=0, xti=0; xc<iWidth; xc+=X_TILESIZE, xti++) {
           if (GET_TILE_UPDATED (xti, yti)) {
@@ -613,11 +606,7 @@ void bx_vga_c::update(void)
               for (c=0; c<X_TILESIZE; c++) {
                 x = xc + c;
                 BX_VGA_THIS s.tile[r*X_TILESIZE + c] =
-#ifdef VGA_MEM_FIX
                   BX_VGA_THIS get_vga_pixel(x, y, row_addr, 0xffff, 0, BX_VGA_THIS s.memory);
-#else
-                  BX_VGA_THIS get_vga_pixel(x, y, row_addr, 0xffff, 0, plane);
-#endif
               }
             }
             SET_TILE_UPDATED(BX_VGA_THIS, xti, yti, 0);
@@ -775,7 +764,7 @@ bx_vga_c::vbe_mem_read(bx_phy_address addr)
   }
 
   // check for out of memory read
-  if (offset > VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES)
+  if (offset > BX_VGA_THIS s.memsize)
     return 0;
 
   return (BX_VGA_THIS s.memory[offset]);
@@ -800,7 +789,7 @@ bx_vga_c::vbe_mem_write(bx_phy_address addr, Bit8u value)
   }
 
   // check for out of memory write
-  if (offset < VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES) {
+  if (offset < BX_VGA_THIS s.memsize) {
     BX_VGA_THIS s.memory[offset] = value;
   } else {
     // make sure we don't flood the logfile
@@ -907,7 +896,7 @@ Bit32u bx_vga_c::vbe_read(Bit32u address, unsigned io_len)
         return BX_VGA_THIS vbe.virtual_yres;
 
       case VBE_DISPI_INDEX_VIDEO_MEMORY_64K:
-        return (VBE_DISPI_TOTAL_VIDEO_MEMORY_KB >> 6);
+        return (BX_VGA_THIS s.memsize >> 16);
 
       case VBE_DISPI_INDEX_DDC:
         if (BX_VGA_THIS vbe.ddc_enabled) {
@@ -1054,7 +1043,7 @@ Bit32u bx_vga_c::vbe_write(Bit32u address, Bit32u value, unsigned io_len)
 
         case VBE_DISPI_INDEX_BANK: // set bank
         {
-          Bit16u num_banks = (Bit16u)(VBE_DISPI_TOTAL_VIDEO_MEMORY_KB / BX_VGA_THIS vbe.bank_granularity_kb);
+          Bit16u num_banks = (Bit16u)((BX_VGA_THIS s.memsize >> 10) / BX_VGA_THIS vbe.bank_granularity_kb);
           Bit16u rw_mode = VBE_DISPI_BANK_RW; // compatibility mode
           if (BX_VGA_THIS vbe.bpp == VBE_DISPI_BPP_4) num_banks >>= 2;
           if ((value & VBE_DISPI_BANK_RW) != 0) {
@@ -1139,28 +1128,21 @@ Bit32u bx_vga_c::vbe_write(Bit32u address, Bit32u value, unsigned io_len)
             BX_INFO(("VBE enabling x %d, y %d, bpp %d, %u bytes visible", BX_VGA_THIS vbe.xres, BX_VGA_THIS vbe.yres, BX_VGA_THIS vbe.bpp, BX_VGA_THIS vbe.visible_screen_size));
 
             if ((value & VBE_DISPI_NOCLEARMEM) == 0) {
-              memset(BX_VGA_THIS s.memory, 0, VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES);
+              memset(BX_VGA_THIS s.memory, 0, BX_VGA_THIS s.memsize);
             }
             if (depth > 4) {
               bx_gui->dimension_update(BX_VGA_THIS vbe.xres, BX_VGA_THIS vbe.yres, 0, 0, depth);
               BX_VGA_THIS s.last_bpp = depth;
               BX_VGA_THIS s.last_fh = 0;
             } else {
-#ifndef VGA_MEM_FIX
-              BX_VGA_THIS s.plane_shift = VBE_DISPI_4BPP_PLANE_SHIFT;
-#endif
               BX_VGA_THIS s.ext_offset = (BX_VGA_THIS vbe.bank[0] << 16);
             }
-            BX_VGA_THIS s.memsize_mask = BX_VGA_THIS s.memsize - 1;
+            BX_VGA_THIS s.vgamem_mask = BX_VGA_THIS s.memsize - 1;
           } else if (((value & VBE_DISPI_ENABLED) == 0) && BX_VGA_THIS vbe.enabled) {
             BX_INFO(("VBE disabling"));
-#ifdef VGA_MEM_FIX
             BX_VGA_THIS s.text_buffer_update = true;
-#else
-            BX_VGA_THIS s.plane_shift = 16;
-#endif
             BX_VGA_THIS s.ext_offset = 0;
-            BX_VGA_THIS s.memsize_mask = 0x3ffff;
+            BX_VGA_THIS s.vgamem_mask = 0x3ffff;
           }
           BX_VGA_THIS vbe.enabled = ((value & VBE_DISPI_ENABLED) != 0);
           BX_VGA_THIS vbe.get_capabilities = ((value & VBE_DISPI_GETCAPS) != 0);
@@ -1230,14 +1212,14 @@ Bit32u bx_vga_c::vbe_write(Bit32u address, Bit32u value, unsigned io_len)
 
           Bit32u new_screen_start = value * BX_VGA_THIS vbe.line_offset;
           if (BX_VGA_THIS vbe.bpp != VBE_DISPI_BPP_4) {
-            if ((new_screen_start + BX_VGA_THIS vbe.visible_screen_size) > VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES)
+            if ((new_screen_start + BX_VGA_THIS vbe.visible_screen_size) > BX_VGA_THIS s.memsize)
             {
               BX_PANIC(("VBE offset y %d out of bounds", value));
               break;
             }
             new_screen_start += (BX_VGA_THIS vbe.offset_x * BX_VGA_THIS vbe.bpp_multiplier);
           } else {
-            if ((new_screen_start + BX_VGA_THIS vbe.visible_screen_size) > (VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES / 4))
+            if ((new_screen_start + BX_VGA_THIS vbe.visible_screen_size) > (BX_VGA_THIS s.memsize >> 2))
             {
               BX_PANIC(("VBE offset y %d out of bounds", value));
               break;
@@ -1269,9 +1251,9 @@ Bit32u bx_vga_c::vbe_write(Bit32u address, Bit32u value, unsigned io_len)
           Bit16u new_width=value;
           Bit16u new_height;
           if (BX_VGA_THIS vbe.bpp != VBE_DISPI_BPP_4) {
-            new_height=(VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES / BX_VGA_THIS vbe.bpp_multiplier) / new_width;
+            new_height = (BX_VGA_THIS s.memsize / BX_VGA_THIS vbe.bpp_multiplier) / new_width;
           } else {
-            new_height=(VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES * 2) / new_width;
+            new_height = (BX_VGA_THIS s.memsize << 1) / new_width;
           }
           if (new_height >=BX_VGA_THIS vbe.yres)
           {
@@ -1283,9 +1265,9 @@ Bit32u bx_vga_c::vbe_write(Bit32u address, Bit32u value, unsigned io_len)
             // no decent virtual height: adjust width & height
             new_height=BX_VGA_THIS vbe.yres;
             if (BX_VGA_THIS vbe.bpp != VBE_DISPI_BPP_4) {
-              new_width=(VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES / BX_VGA_THIS vbe.bpp_multiplier) / new_height;
+              new_width = (BX_VGA_THIS s.memsize / BX_VGA_THIS vbe.bpp_multiplier) / new_height;
             } else {
-              new_width=(VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES * 2) / new_height;
+              new_width = (BX_VGA_THIS s.memsize << 1) / new_height;
             }
 
             BX_INFO(("VBE recalc virtual width %d height %d",new_width, new_height));
