@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: scancodes.cc 14095 2021-01-30 18:47:25Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002-2009  The Bochs Project
@@ -27,10 +27,10 @@
 #define BX_PLUGGABLE
 
 #include "bochs.h"
-#include "gui/gui.h"
+#include "gui/gui.h"        // for BX_KEY_NBKEYS
 #include "scancodes.h"
 
-unsigned char translation8042[256] = {
+const unsigned char translation8042[256] = {
   0xff,0x43,0x41,0x3f,0x3d,0x3b,0x3c,0x58,0x64,0x44,0x42,0x40,0x3e,0x0f,0x29,0x59,
   0x65,0x38,0x2a,0x70,0x1d,0x10,0x02,0x5a,0x66,0x71,0x2c,0x1f,0x1e,0x11,0x03,0x5b,
   0x67,0x2e,0x2d,0x20,0x12,0x05,0x04,0x5c,0x68,0x39,0x2f,0x21,0x14,0x13,0x06,0x5d,
