@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: serial_raw.h 14112 2021-01-31 10:50:53Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2004-2021  The Bochs Project
@@ -18,7 +18,7 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
 
 #ifdef __linux__
 #include <linux/serial.h>
@@ -29,11 +29,13 @@
 //#define WIN32_RECEIVE_RAW
 #endif
 
-#define P_NONE  0
-#define P_ODD   1
-#define P_EVEN  2
-#define P_HIGH  3
-#define P_LOW   4
+enum {
+  P_NONE = 0,
+  P_ODD  = 1,
+  P_EVEN = 2,
+  P_HIGH = 3,
+  P_LOW  = 4
+};
 
 #define RAW_EVENT_BREAK    -1
 #define RAW_EVENT_CTS_ON   -2
