@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sb16.h 14155 2021-02-19 13:13:42Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2021  The Bochs Project
+//  Copyright (C) 2001-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -158,7 +158,7 @@ private:
   int currentirq;
   int currentdma8;
   int currentdma16;
-  int fmopl_callback_id;
+  int fmopl_callback_id[2];
   int rt_conf_id;
   Bit16u fm_volume;
 
@@ -274,7 +274,7 @@ private:
   BX_SB16_SMF Bit32u dsp_irq16ack();                 // ack 16 bit IRQ      2xf
   BX_SB16_SMF void   dsp_dma(Bit8u command, Bit8u mode, Bit16u length, Bit8u comp);
 						     // initiate a DMA transfer
-  BX_SB16_SMF void   dsp_dmadone();		     // stop a DMA transfer
+  BX_SB16_SMF void   dsp_dmadone(bool irq);	     // stop a DMA transfer
   BX_SB16_SMF void   dsp_enabledma();		     // enable the transfer
   BX_SB16_SMF void   dsp_disabledma();		     // temporarily disable DMA
   BX_SB16_SMF void   dsp_disable_nondma();	     // disable DSP direct mode
