@@ -2,8 +2,8 @@
 // $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2009-2023  Benjamin D Lunt (fys [at] fysnet [dot] net)
-//                2009-2023  The Bochs Project
+//  Copyright (C) 2009-2025  Benjamin D Lunt (fys [at] fysnet [dot] net)
+//                2009-2025  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -740,19 +740,21 @@ bool bx_ohci_core_c::mem_write(bx_phy_address addr, unsigned len, void *data)
       hub.op_regs.HcRhDescriptorA.ocpm     = (value & (1<<11)) ? 1 : 0;
       hub.op_regs.HcRhDescriptorA.nps      = (value & (1<< 9)) ? 1 : 0;
       hub.op_regs.HcRhDescriptorA.psm      = (value & (1<< 8)) ? 1 : 0;
-      if (hub.op_regs.HcRhDescriptorA.psm == 0) {
-
-        BX_INFO(("Ben: hub.op_regs.HcRhDescriptorA.psm == 0"));
-        // all ports have power, etc.
-        // hub.usb_port[p].HcRhPortStatus.pps = 1
-        //  Call a routine to set each ports dword (LS, Connected, etc.)
-
-      } else {
-
-        BX_INFO(("Ben: hub.op_regs.HcRhDescriptorA.psm == 1"));
-        // only ports with bit set in rhstatus have power, etc.
-        //  Call a routine to set each ports dword (LS, Connected, etc.)
-
+      if (hub.op_regs.HcRhDescriptorA.nps == 0) {  // psm is only valid if nps == 0
+        if (hub.op_regs.HcRhDescriptorA.psm == 0) {
+          
+          BX_INFO(("Ben: hub.op_regs.HcRhDescriptorA.psm == 0"));
+          // all ports have power, etc.
+          // hub.usb_port[p].HcRhPortStatus.pps = 1
+          //  Call a routine to set each ports dword (LS, Connected, etc.)
+          
+        } else {
+          
+          BX_INFO(("Ben: hub.op_regs.HcRhDescriptorA.psm == 1"));
+          // only ports with bit set in rhstatus have power, etc.
+          //  Call a routine to set each ports dword (LS, Connected, etc.)
+          
+        }
       }
       break;
 
@@ -771,24 +773,26 @@ bool bx_ohci_core_c::mem_write(bx_phy_address addr, unsigned len, void *data)
       if (value & (1<<15)) hub.op_regs.HcRhStatus.drwe = 1;
 
       if (value & (1<<17)) hub.op_regs.HcRhStatus.ocic = 1;
-      if (value & (1<<16)) {
-        if (hub.op_regs.HcRhDescriptorA.psm == 0) {
-          for (p=0; p<USB_OHCI_PORTS; p++)
-            hub.usb_port[p].HcRhPortStatus.pps = 1;
-        } else {
-          for (p=0; p<USB_OHCI_PORTS; p++)
-            if ((hub.op_regs.HcRhDescriptorB.ppcm & (1<<p)) == 0)
+      if (hub.op_regs.HcRhDescriptorA.nps == 0) {  // psm is only valid if nps == 0
+        if (value & (1<<16)) {
+          if (hub.op_regs.HcRhDescriptorA.psm == 0) {
+            for (p=0; p<USB_OHCI_PORTS; p++)
               hub.usb_port[p].HcRhPortStatus.pps = 1;
+          } else {
+            for (p=0; p<USB_OHCI_PORTS; p++)
+              if ((hub.op_regs.HcRhDescriptorB.ppcm & (1<<p)) == 0)
+                hub.usb_port[p].HcRhPortStatus.pps = 1;
+          }
         }
-      }
-      if (value & (1<<0)) {
-        if (hub.op_regs.HcRhDescriptorA.psm == 0) {
-          for (p=0; p<USB_OHCI_PORTS; p++)
-            hub.usb_port[p].HcRhPortStatus.pps = 0;
-        } else {
-          for (p=0; p<USB_OHCI_PORTS; p++)
-            if (!(hub.op_regs.HcRhDescriptorB.ppcm & (1<<p)))
+        if (value & (1<<0)) {
+          if (hub.op_regs.HcRhDescriptorA.psm == 0) {
+            for (p=0; p<USB_OHCI_PORTS; p++)
               hub.usb_port[p].HcRhPortStatus.pps = 0;
+          } else {
+            for (p=0; p<USB_OHCI_PORTS; p++)
+              if (!(hub.op_regs.HcRhDescriptorB.ppcm & (1<<p)))
+                hub.usb_port[p].HcRhPortStatus.pps = 0;
+          }
         }
       }
       break;
@@ -796,14 +800,23 @@ bool bx_ohci_core_c::mem_write(bx_phy_address addr, unsigned len, void *data)
 
     case 0x60: // HcRhPortStatus[3]
 #if (USB_OHCI_PORTS < 4)
+  #if BX_USB_DEBUGGER
+      SIM->usb_debug_trigger(USB_DEBUG_OHCI, USB_DEBUG_NONEXIST, 0, 0, 0);
+  #endif
       break;
 #endif
     case 0x5C: // HcRhPortStatus[2]
 #if (USB_OHCI_PORTS < 3)
+  #if BX_USB_DEBUGGER
+      SIM->usb_debug_trigger(USB_DEBUG_OHCI, USB_DEBUG_NONEXIST, 0, 0, 0);
+  #endif
       break;
 #endif
     case 0x58: // HcRhPortStatus[1]
 #if (USB_OHCI_PORTS < 2)
+  #if BX_USB_DEBUGGER
+      SIM->usb_debug_trigger(USB_DEBUG_OHCI, USB_DEBUG_NONEXIST, 0, 0, 0);
+  #endif
       break;
 #endif
     case 0x54: { // HcRhPortStatus[0]
@@ -816,6 +829,9 @@ bool bx_ohci_core_c::mem_write(bx_phy_address addr, unsigned len, void *data)
         if (hub.usb_port[p].HcRhPortStatus.ccs == 0)
           hub.usb_port[p].HcRhPortStatus.csc = 1;
         else {
+#if BX_USB_DEBUGGER
+          SIM->usb_debug_trigger(USB_DEBUG_OHCI, USB_DEBUG_ENABLE, 0, 0, 0);
+#endif
           hub.usb_port[p].HcRhPortStatus.pes = 1;
         }
       }
@@ -832,6 +848,9 @@ bool bx_ohci_core_c::mem_write(bx_phy_address addr, unsigned len, void *data)
         if (hub.usb_port[p].HcRhPortStatus.ccs == 0)
           hub.usb_port[p].HcRhPortStatus.csc = 1;
         else {
+#if BX_USB_DEBUGGER
+          SIM->usb_debug_trigger(USB_DEBUG_OHCI, USB_DEBUG_RESET, 0, 0, 0);
+#endif
           reset_port(p);
           hub.usb_port[p].HcRhPortStatus.pps = 1;
           hub.usb_port[p].HcRhPortStatus.pes = 1;
@@ -899,6 +918,9 @@ void bx_ohci_core_c::ohci_timer(void)
   Bit16u zero = 0;
 
   if (hub.op_regs.HcControl.hcfs == OHCI_USB_OPERATIONAL) {
+#if BX_USB_DEBUGGER
+    SIM->usb_debug_trigger(USB_DEBUG_OHCI, USB_DEBUG_FRAME, 0, 0, 0);
+#endif
     // set remaining to the interval amount.
     hub.op_regs.HcFmRemainingToggle = hub.op_regs.HcFmInterval.fit;
     hub.sof_time = bx_pc_system.time_usec();
@@ -1138,6 +1160,10 @@ int bx_ohci_core_c::process_td(struct OHCI_TD *td, struct OHCI_ED *ed, int toggl
     return 0;
   }
 
+#if BX_USB_DEBUGGER
+  SIM->usb_debug_trigger(USB_DEBUG_OHCI, USB_DEBUG_COMMAND, 0, 0, 0);
+#endif
+
   // The td->cc field should be 111x if it hasn't been processed yet.
   if (TD_GET_CC(td) < NotAccessed) {
     BX_ERROR(("Found TD with CC value not 111x"));
