@@ -1,11 +1,11 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: usb_floppy.cc 14225 2021-04-17 09:30:58Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  UFI/CBI floppy disk storage device support
 //
-//  Copyright (c) 2015       Benjamin David Lunt
-//  Copyright (C) 2015-2021  The Bochs Project
+//  Copyright (c) 2015-2023  Benjamin David Lunt
+//  Copyright (C) 2015-2023  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -80,7 +80,7 @@ protected:
 /* A well known, somewhat older, but still widely used Operating System
  *  must have the Vendor ID as the TEAC external drive emulated below.  If
  *  the VendorID is not the TEAC id, this operating system doesn't know what
- *  to do with the drive.  Go figure.  Therefore, use the optionsX="model=teac"
+ *  to do with the drive.  Go figure.  Therefore, use the optionsX="model:teac"
  *  options in the bochsrc.txt file to set this model.  If you do not, a
  *  default model will be used.
  */
@@ -88,21 +88,27 @@ protected:
 // USB requests
 #define GetMaxLun         0xfe
 
+// If you change any of the Max Packet Size, or other fields within these
+//  descriptors, you must also change the d.endpoint_info[] array
+//  to match your changes.
+
 // Full-speed only
+// * USB Mass Storage Class Specification, p4, section 3, states that
+// *  that a Mass Storage Class device using CB/CBI must be full-speed only.
 static Bit8u bx_floppy_dev_descriptor[] = {
   0x12,       /*  u8 bLength; */
   0x01,       /*  u8 bDescriptorType; Device */
-  0x00, 0x02, /*  u16 bcdUSB; v2.0 */
+  0x01, 0x01, /*  u16 bcdUSB; v1.1 */
 
   0x00,       /*  u8  bDeviceClass; */
   0x00,       /*  u8  bDeviceSubClass; */
-  0x00,       /*  u8  bDeviceProtocol; [ low/full speeds only ] */
-  0x08,       /*  u8  bMaxPacketSize0; 8 Bytes */
+  0x00,       /*  u8  bDeviceProtocol; */
+  0x40,       /*  u8  bMaxPacketSize; 64 Bytes */
 
   /* Vendor and product id are arbitrary.  */
   0x00, 0x00, /*  u16 idVendor; */
   0x00, 0x00, /*  u16 idProduct; */
-  0x00, 0x00, /*  u16 bcdDevice */
+  0x00, 0x00, /*  u16 bcdDevice; */
 
   0x01,       /*  u8  iManufacturer; */
   0x02,       /*  u8  iProduct; */
@@ -116,7 +122,11 @@ static const Bit8u bx_floppy_config_descriptor[] = {
   /* one configuration */
   0x09,       /*  u8  bLength; */
   0x02,       /*  u8  bDescriptorType; Configuration */
+#if USB_FLOPPY_USE_INTERRUPT
   0x27, 0x00, /*  u16 wTotalLength; */
+#else
+  0x20, 0x00, /*  u16 wTotalLength; */
+#endif
   0x01,       /*  u8  bNumInterfaces; (1) */
   0x01,       /*  u8  bConfigurationValue; */
   0x00,       /*  u8  iConfiguration; */
@@ -180,7 +190,7 @@ static const Bit8u bx_floppy_dev_inquiry_teac[] = {
   0x01,       /*  response data format */
   0x1F,       /*  additional length */
   0x00, 0x00, 0x00,  /* reserved */
-  0x54, 0x45, 0x41, 0x43, 0x20, 0x20, 0x20, 0x20,  /* vendor information  */ 
+  0x54, 0x45, 0x41, 0x43, 0x20, 0x20, 0x20, 0x20,  /* vendor information  */
   0x46, 0x44, 0x2D, 0x30, 0x35, 0x50, 0x55, 0x57,  /* product information */
   0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,  /*    (cont.)          */
   0x33, 0x30, 0x30, 0x30                           /* revision level      */
@@ -193,7 +203,7 @@ static const Bit8u bx_floppy_dev_inquiry_bochs[] = {
   0x01,       /*  response data format */
   0x1F,       /*  additional length */
   0x00, 0x00, 0x00,  /* reserved */
-  0x42, 0x4F, 0x43, 0x48, 0x53, 0x20, 0x20, 0x20,  /* vendor information  */ 
+  0x42, 0x4F, 0x43, 0x48, 0x53, 0x20, 0x20, 0x20,  /* vendor information  */
   0x55, 0x53, 0x42, 0x20, 0x43, 0x42, 0x49, 0x20,  /* product information */
   0x46, 0x4C, 0x4F, 0x50, 0x50, 0x59, 0x20, 0x20,  /*    (cont.)          */
   0x30, 0x30, 0x31, 0x30                           /* revision level      */
@@ -246,11 +256,11 @@ static Bit8u bx_floppy_dev_mode_sense_cur[] = {
   // header  (8 bytes)
   0x00, 0x46,   // length of remaining data (72 - 2)
   // medium type code (See page 25 of ufi v1.0 specification)
-  // 0x1E,      // (720k) 
+  // 0x1E,      // (720k)
   // 0x93,      // (1.25meg)
   0x94,         // (1.44meg)
   // WP (bit 7), reserved
-  0x00, // ben
+  0x00,
   // reserved
   0x00, 0x00, 0x00, 0x00,
 
@@ -320,13 +330,24 @@ usb_floppy_device_c::usb_floppy_device_c()
   bx_param_bool_c *readonly;
   bx_param_enum_c *status, *mode;
 
+  // MSC Compliance states that a CB(I) device must be full-speed only
   d.speed = d.minspeed = d.maxspeed = USB_SPEED_FULL;
-  memset((void*)&s, 0, sizeof(s));
+  memset((void *) &s, 0, sizeof(s));
   strcpy(d.devname, "BOCHS UFI/CBI FLOPPY");
   d.dev_descriptor = bx_floppy_dev_descriptor;
   d.config_descriptor = bx_floppy_config_descriptor;
   d.device_desc_size = sizeof(bx_floppy_dev_descriptor);
   d.config_desc_size = sizeof(bx_floppy_config_descriptor);
+  d.endpoint_info[USB_CONTROL_EP].max_packet_size = 64; // Control ep0
+  d.endpoint_info[USB_CONTROL_EP].max_burst_size = 0;
+  d.endpoint_info[1].max_packet_size = 64;  // In ep1
+  d.endpoint_info[1].max_burst_size = 0;
+  d.endpoint_info[2].max_packet_size = 64;  // Out ep2
+  d.endpoint_info[2].max_burst_size = 0;
+#if USB_FLOPPY_USE_INTERRUPT
+  d.endpoint_info[3].max_packet_size = 2;  // In ep3
+  d.endpoint_info[3].max_burst_size = 0;
+#endif
   s.inserted = 0;
   s.dev_buffer = new Bit8u[USB_FLOPPY_MAX_SECTORS * 512];
   s.statusbar_id = bx_gui->register_statusitem("USB-FD", 1);
@@ -429,6 +450,12 @@ bool usb_floppy_device_c::set_option(const char *option)
 
 bool usb_floppy_device_c::init()
 {
+  /*  If you wish to set DEBUG=report in the code, instead of
+   *  in the configuration, simply uncomment this line.  I use
+   *  it when I am working on this emulation.
+   */
+//  LOG_THIS setonoff(LOGLEV_DEBUG, ACT_REPORT);
+
   // set the model information
   //  s.model == 1 if use teac model, else use bochs model
   if (s.model) {
@@ -436,13 +463,13 @@ bool usb_floppy_device_c::init()
     bx_floppy_dev_descriptor[9] = 0x06;
     d.vendor_desc = "TEAC    ";
     d.product_desc = "TEAC FD-05PUW   ";
-    d.serial_num = "3000";
+    d.serial_num = "3000        ";
   } else {
     bx_floppy_dev_descriptor[8] = 0x00;
     bx_floppy_dev_descriptor[9] = 0x00;
     d.vendor_desc = "BOCHS   ";
     d.product_desc = d.devname;
-    d.serial_num = "00.10";
+    d.serial_num = "00.10       ";
   }
   if (set_inserted(1)) {
     sprintf(s.info_txt, "USB floppy: path='%s', mode='%s'", s.fname, s.image_mode);
@@ -450,15 +477,18 @@ bool usb_floppy_device_c::init()
     sprintf(s.info_txt, "USB floppy: media not present");
   }
   d.connected = 1;
+  d.alt_iface_max = 0;
 
+#if UFI_DO_INQUIRY_HACK
   s.did_inquiry_fail = 0;
   s.fail_count = 0;
+#endif
   s.status_changed = 0;
 
   return 1;
 }
 
-const char* usb_floppy_device_c::get_info()
+const char *usb_floppy_device_c::get_info()
 {
   // set the write protected bit given by parameter in bochsrc.txt file
   bx_floppy_dev_mode_sense_cur[3] &= ~0x80;
@@ -483,8 +513,10 @@ void usb_floppy_device_c::register_state_specific(bx_list_c *parent)
   BXRS_DEC_PARAM_FIELD(list, cur_track, s.cur_track);
   BXRS_DEC_PARAM_FIELD(list, sense, s.sense);
   BXRS_DEC_PARAM_FIELD(list, asc, s.asc);
+#if UFI_DO_INQUIRY_HACK
   BXRS_DEC_PARAM_FIELD(list, fail_count, s.fail_count);
   BXRS_PARAM_BOOL(list, did_inquiry_fail, s.did_inquiry_fail);
+#endif
   BXRS_PARAM_BOOL(list, seek_pending, s.seek_pending);
   BXRS_PARAM_SPECIAL32(list, usb_buf, param_save_handler, param_restore_handler);
   new bx_shadow_data_c(list, "dev_buffer", s.dev_buffer, USB_FLOPPY_MAX_SECTORS * 512);
@@ -508,7 +540,7 @@ int usb_floppy_device_c::handle_control(int request, int value, int index, int l
   ret = 0;
   switch (request) {
     case DeviceOutRequest | USB_REQ_CLEAR_FEATURE:
-      BX_INFO(("USB_REQ_CLEAR_FEATURE: Not handled: %i %i %i %i", request, value, index, length ));
+      BX_INFO(("USB_REQ_CLEAR_FEATURE: Not handled: %d %d %d %d", request, value, index, length ));
       // It's okay that we don't handle this.  Most likely the guest is just
       //  clearing the toggle bit.  Since we don't worry about the toggle bit (yet?),
       //  we can just ignore and continue.
@@ -522,11 +554,34 @@ int usb_floppy_device_c::handle_control(int request, int value, int index, int l
         case USB_DEVICE_U2_ENABLE:
           break;
         default:
-          BX_DEBUG(("USB_REQ_SET_FEATURE: Not handled: %i %i %i %i", request, value, index, length ));
+          BX_DEBUG(("USB_REQ_SET_FEATURE: Not handled: %d %d %d %d", request, value, index, length ));
           goto fail;
       }
       ret = 0;
       break;
+    case EndpointRequest | USB_REQ_GET_STATUS:
+      BX_DEBUG(("USB_REQ_GET_STATUS: Endpoint."));
+      // if the endpoint is currently halted, return bit 0 = 1
+      if (value == USB_ENDPOINT_HALT) {
+        int indx = (index & 0x7F);
+#if USB_FLOPPY_USE_INTERRUPT
+        int limit = 3;
+#else
+        int limit = 2;
+#endif
+        if ((indx > 0) && (indx <= limit)) {
+          data[0] = 0x00 | (get_halted(indx) ? 1 : 0);
+          data[1] = 0x00;
+          ret = 2;
+        } else {
+          BX_ERROR(("EndpointRequest | USB_REQ_GET_STATUS: index > ep count: %d", index));
+          goto fail;
+        }
+      } else {
+        BX_ERROR(("EndpointRequest | USB_REQ_SET_FEATURE: Unknown Get Status Request found: %d", value));
+        goto fail;
+      }
+      break;
     case DeviceRequest | USB_REQ_GET_DESCRIPTOR:
       switch (value >> 8) {
         case USB_DT_STRING:
@@ -535,6 +590,7 @@ int usb_floppy_device_c::handle_control(int request, int value, int index, int l
             case 0xEE:
               // Microsoft OS Descriptor check
               // We don't support this check, so fail
+              BX_INFO(("USB floppy handle_control: Microsoft OS specific 0xEE string descriptor"));
               goto fail;
             default:
               BX_ERROR(("USB floppy handle_control: unknown string descriptor 0x%02x", value & 0xff));
@@ -553,16 +609,6 @@ int usb_floppy_device_c::handle_control(int request, int value, int index, int l
           goto fail;
       }
       break;
-    case EndpointOutRequest | USB_REQ_CLEAR_FEATURE:
-      BX_DEBUG(("USB_REQ_CLEAR_FEATURE:"));
-      // It's okay that we don't handle this.  Most likely the guest is just
-      //  clearing the toggle bit.  Since we don't worry about the toggle bit (yet?),
-      //  we can just ignore and continue.
-      //if (value == 0 && index != 0x81) { /* clear ep halt */
-      //  goto fail;
-      //}
-      ret = 0;
-      break;
     case DeviceOutRequest | USB_REQ_SET_SEL:
       // Set U1 and U2 System Exit Latency
       BX_DEBUG(("SET_SEL (U1 and U2):"));
@@ -577,7 +623,7 @@ int usb_floppy_device_c::handle_control(int request, int value, int index, int l
       break;
     // this is the request where we should receive 12 bytes
     //  of command data.
-    case InterfaceOutClassRequest:
+    case InterfaceOutClassRequest | 0x00:
       if (!handle_command(data))
         goto fail;
       break;
@@ -618,15 +664,18 @@ bool usb_floppy_device_c::handle_command(Bit8u *command)
     return 0;
   }
 
-  // to be consistant with real hardware, we need to fail with
+#if UFI_DO_INQUIRY_HACK
+  // to be consistent with real hardware, we need to fail with
   //  a STALL twice for any command after the first Inquiry except
   //  for the inquiry and request_sense commands.
   //  (I don't know why, and will document further when I know more)
   if ((s.fail_count > 0) &&
       (s.cur_command != UFI_INQUIRY) && (s.cur_command != UFI_REQUEST_SENSE)) {
+    BX_INFO(("Consistent stall of %d of 2.", 2 - s.fail_count + 1));
     s.fail_count--;
     return 0;
   }
+#endif
 
   if (s.cur_command != UFI_REQUEST_SENSE) {
     s.sense = 0;
@@ -646,10 +695,12 @@ bool usb_floppy_device_c::handle_command(Bit8u *command)
       s.data_len = command[4];
       if (s.data_len > s.usb_len)
         s.data_len = s.usb_len;
+#if UFI_DO_INQUIRY_HACK
       if (s.did_inquiry_fail == 0) {
         s.fail_count = 2;
         s.did_inquiry_fail = 1;
       }
+#endif
       break;
 
     case UFI_READ_FORMAT_CAPACITIES:
@@ -690,7 +741,7 @@ bool usb_floppy_device_c::handle_command(Bit8u *command)
                (command[8] <<  8) |
                (command[9] <<  0));
     case UFI_READ_10:
-      BX_DEBUG(("UFI_READ_%i COMMAND (lba = %i, count = %i)", (s.cur_command == UFI_READ_12) ? 12 : 10, lba, count));
+      BX_DEBUG(("UFI_READ_%d COMMAND (lba = %d, count = %d)", (s.cur_command == UFI_READ_12) ? 12 : 10, lba, count));
       if (!s.inserted) {
         s.sense = 2;
         s.asc = 0x3a;
@@ -718,7 +769,7 @@ bool usb_floppy_device_c::handle_command(Bit8u *command)
                (command[8] <<  8) |
                (command[9] <<  0));
     case UFI_WRITE_10:
-      BX_DEBUG(("UFI_WRITE_%i COMMAND (lba = %i, count = %i)", (s.cur_command == UFI_WRITE_12) ? 12 : 10, lba, count));
+      BX_DEBUG(("UFI_WRITE_%d COMMAND (lba = %d, count = %d)", (s.cur_command == UFI_WRITE_12) ? 12 : 10, lba, count));
       if (!s.inserted) {
         s.sense = 2;
         s.asc = 0x3a;
@@ -760,7 +811,7 @@ bool usb_floppy_device_c::handle_command(Bit8u *command)
       break;
 
     case UFI_PREVENT_ALLOW_REMOVAL:
-      BX_DEBUG(("UFI_PREVENT_ALLOW_REMOVAL COMMAND (prevent = %i)", (command[4] & 1) > 0));
+      BX_DEBUG(("UFI_PREVENT_ALLOW_REMOVAL COMMAND (prevent = %d)", (command[4] & 1) > 0));
       if (command[4] & 1) {
         s.sense = 5;
         s.asc = 0x24;
@@ -771,7 +822,7 @@ bool usb_floppy_device_c::handle_command(Bit8u *command)
     case UFI_MODE_SENSE:
       pc = command[2] >> 6;
       pagecode = command[2] & 0x3F;
-      BX_DEBUG(("UFI_MODE_SENSE COMMAND.  PC = %i, PageCode = %02X", pc, pagecode));
+      BX_DEBUG(("UFI_MODE_SENSE COMMAND.  PC = %d, PageCode = %02X", pc, pagecode));
       switch (pc) {
         case 0:  // current values
           switch (pagecode) {
@@ -819,7 +870,7 @@ bool usb_floppy_device_c::handle_command(Bit8u *command)
       break;
 
     case UFI_START_STOP_UNIT:
-      BX_DEBUG(("UFI_START_STOP_UNIT COMMAND (start = %i)", command[4] & 1));
+      BX_DEBUG(("UFI_START_STOP_UNIT COMMAND (start = %d)", command[4] & 1));
       // The UFI specs say that access to the media is allowed
       //  even if the start/stop command is used to stop the device.
       // However, we'll allow the command to return valid return.
@@ -831,7 +882,7 @@ bool usb_floppy_device_c::handle_command(Bit8u *command)
       break;
 
     case UFI_FORMAT_UNIT:
-      BX_DEBUG(("UFI_FORMAT_UNIT COMMAND (track = %i)", command[2]));
+      BX_DEBUG(("UFI_FORMAT_UNIT COMMAND (track = %d)", command[2]));
       if (!s.inserted) {
         s.sense = 2;
         s.asc = 0x3a;
@@ -844,8 +895,32 @@ bool usb_floppy_device_c::handle_command(Bit8u *command)
       }
       break;
 
-    case UFI_REZERO:
     case UFI_SEND_DIAGNOSTIC:
+      // The PF, Defofl, and UnitOfl bits are irrelevant to the CBI
+      // (Funny note is that the 'DefOfl' bit is misspelled in the specs.
+      //  It should be 'DevOfl' for DevOffLine and UnitOffLine respectively)
+      // 
+      // if the command is 1Dh 04h 00h 00h 00h 00h ...., then this is a Command Block Reset
+      if ((command[1] == 0x04) && !memcmp(&command[2], "\0\0\0\0\0\0\0\0\0", 9)) {
+        BX_DEBUG(("Found UFI_SEND_DIAGNOSTIC: CBI Command Block Reset"));
+        s.status_changed = 0;
+        s.sector =
+          s.sector_count = 0;
+        s.cur_track = 0;
+        s.cur_command = 0;
+        s.sense =
+          s.asc = 0;
+        s.did_inquiry_fail = 0;
+        s.fail_count = 0;
+        
+      // else it is to perform a special diagnostic test
+      } else {
+        BX_DEBUG(("Found UFI_SEND_DIAGNOSTIC: Perform Special Diagnostic Test. Returning good..."));
+        // as long as the LUN == 0, already checked at first of the function, we pass 'good test'
+      }
+      break;
+      
+    case UFI_REZERO:
     case UFI_SEEK_10:
     case UFI_WRITE_VERIFY:
     case UFI_VERIFY:
@@ -866,6 +941,11 @@ int usb_floppy_device_c::handle_data(USBPacket *p)
   Bit8u *data = p->data, *tmpbuf;
   int len = p->len, len1;
   Bit32u count, max_sectors;
+  
+  // check that the length is <= the max packet size of the device
+  if (p->len > get_mps(p->devep)) {
+    BX_DEBUG(("EP%d transfer length (%d) is greater than Max Packet Size (%d).", p->devep, p->len, get_mps(p->devep)));
+  }
 
   switch (p->pid) {
     case USB_TOKEN_OUT:
@@ -913,7 +993,7 @@ int usb_floppy_device_c::handle_data(USBPacket *p)
 
           if (len > (int) s.data_len)
             goto fail;
-          BX_DEBUG(("FORMAT UNIT: single track = %i, side = %i", (data[1] >> 4) & 1, data[1] & 1));
+          BX_DEBUG(("FORMAT UNIT: single track = %d, side = %d", (data[1] >> 4) & 1, data[1] & 1));
           if ((data[1] >> 4) & 1) {
             if (data[1] & 1) {
               s.sector += 18;
@@ -1131,7 +1211,7 @@ int usb_floppy_device_c::floppy_read_sector()
   ssize_t ret;
   USBPacket *p = s.packet;
 
-  BX_DEBUG(("floppy_read_sector(): sector = %i", s.sector));
+  BX_DEBUG(("floppy_read_sector(): sector = %d", s.sector));
   if (((USB_FLOPPY_MAX_SECTORS * 512) - s.usb_len) >= 512) {
     ret = s.hdimage->read((bx_ptr_t) s.usb_buf, 512);
     if (ret > 0) {
@@ -1166,7 +1246,7 @@ int usb_floppy_device_c::floppy_read_sector()
 
 int usb_floppy_device_c::floppy_write_sector()
 {
-  BX_DEBUG(("floppy_write_sector(): sector = %i", s.sector));
+  BX_DEBUG(("floppy_write_sector(): sector = %d", s.sector));
   if (s.hdimage->write((bx_ptr_t) s.usb_buf, 512) < 0) {
     BX_ERROR(("write error"));
     return -1;
