@@ -1,11 +1,11 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: usb_hub.h 14158 2021-02-20 19:58:39Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 // USB hub emulation support (ported from QEMU)
 //
 // Copyright (C) 2005       Fabrice Bellard
-// Copyright (C) 2009-2021  The Bochs Project
+// Copyright (C) 2009-2023  The Bochs Project
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -30,7 +30,7 @@
 #define BX_IODEV_USB_HUB_H
 
 
-#define USB_HUB_MAX_PORTS 8
+#define USB_HUB_MAX_PORTS 8  // must be at least 2 and less than 16
 #define USB_HUB_DEF_PORTS 4
 
 class usb_hub_device_c : public usb_device_c {
@@ -51,7 +51,7 @@ public:
   virtual void after_restore_state();
   virtual void runtime_config();
   void restore_handler(bx_list_c *conf);
-  void event_handler(int event, USBPacket *packet, int port);
+  int event_handler(int event, void *ptr, int port);
 
 private:
   struct {
@@ -67,6 +67,7 @@ private:
       Bit16u PortStatus;
       Bit16u PortChange;
     } usb_port[USB_HUB_MAX_PORTS];
+    Bit16u PortStatusC;
     Bit16u device_change;
   } hub;
 
@@ -77,6 +78,7 @@ private:
 
   static Bit64s hub_param_handler(bx_param_c *param, bool set, Bit64s val);
   static bool hub_param_enable_handler(bx_param_c *param, bool en);
+  static Bit64s hub_param_oc_handler(bx_param_c *param, bool set, Bit64s val);
 };
 
 #endif
