@@ -27,6 +27,8 @@
 #include "cpu/cpu.h"
 #include <assert.h>
 
+#include "bx_debug/debug.h"
+
 #if BX_WITH_CARBON
 #include <Carbon/Carbon.h>
 #endif
@@ -665,9 +667,12 @@ void logfunctions::fatal(int level, const char *prefix, const char *fmt, va_list
     // store prefix and message in 'exit_msg' before unloading device plugins
     sprintf(exit_msg, "%s %s", prefix, tmpbuf);
   }
-#if !BX_DEBUGGER
-  bx_atexit();
+#if BX_DEBUGGER
+  if (!bx_dbg.debugger_active)
 #endif
+  {
+    bx_atexit();
+  }
 #if BX_WITH_CARBON
   if (!isatty(STDIN_FILENO) && !SIM->get_init_done()) {
     snprintf(exit_msg, sizeof(exit_msg), "Bochs startup error\n%s", tmpbuf);
@@ -682,11 +687,14 @@ void logfunctions::fatal(int level, const char *prefix, const char *fmt, va_list
     fprintf(stderr, "%s", exit_msg);
     fprintf(stderr, "\n%s\n", divider);
   }
-#if !BX_DEBUGGER
-  BX_EXIT(exit_status);
-#else
-  bx_dbg_exit(exit_status);
+#if BX_DEBUGGER
+  if (bx_dbg.debugger_active) {
+    bx_dbg_exit(exit_status);
+  } else
 #endif
+  {
+    BX_EXIT(exit_status);
+  }
   // not safe to use BX_* log functions in here.
   fprintf(stderr, "fatal() should never return, but it just did\n");
 }
