@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: misc_mem.cc 14290 2021-06-24 17:03:09Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2020  The Bochs Project
+//  Copyright (C) 2001-2024  The Bochs Project
 //
 //  I/O memory handlers API Copyright (C) 2003 by Frank Cornelis
 //
@@ -28,14 +28,8 @@
 #include "iodev/iodev.h"
 #define LOG_THIS BX_MEM(0)->
 
-// alignment of memory vector, must be a power of 2
-#define BX_MEM_VECTOR_ALIGN 4096
 #define BX_MEM_HANDLERS   ((BX_CONST64(1) << BX_PHY_ADDRESS_WIDTH) >> 20) /* one per megabyte */
 
-#if BX_LARGE_RAMFILE
-Bit8u* const BX_MEM_C::swapped_out = ((Bit8u*)NULL - sizeof(Bit8u));
-#endif
-
 #define FLASH_READ_ARRAY  0xff
 #define FLASH_INT_ID      0x90
 #define FLASH_READ_STATUS 0x70
@@ -45,221 +39,59 @@ Bit8u* const BX_MEM_C::swapped_out = ((Bit8u*)NULL - sizeof(Bit8u));
 #define FLASH_PROG_SETUP  0x40
 #define FLASH_ERASE       0xd0
 
-BX_MEM_C::BX_MEM_C()
+BX_MEM_C::BX_MEM_C() : BX_MEMORY_STUB_C()
 {
-  put("memory", "MEM0");
-
-  vector = NULL;
-  actual_vector = NULL;
-  blocks = NULL;
-  len    = 0;
-  used_blocks = 0;
-
   memory_handlers = NULL;
-
-#if BX_LARGE_RAMFILE
-  next_swapout_idx = 0;
-  overflow_file = NULL;
-#endif
-}
-
-Bit8u* BX_MEM_C::alloc_vector_aligned(Bit64u bytes, Bit64u alignment)
-{
-  Bit64u test_mask = alignment - 1;
-  BX_MEM_THIS actual_vector = new Bit8u [(Bit32u)(bytes + test_mask)];
-  if (BX_MEM_THIS actual_vector == 0) {
-    BX_PANIC(("alloc_vector_aligned: unable to allocate host RAM !"));
-    return 0;
-  }
-  // round address forward to nearest multiple of alignment.  Alignment
-  // MUST BE a power of two for this to work.
-  Bit64u masked = ((Bit64u)(BX_MEM_THIS actual_vector + test_mask)) & ~test_mask;
-  Bit8u *vector = (Bit8u *) masked;
-  // sanity check: no lost bits during pointer conversion
-  assert(sizeof(masked) >= sizeof(vector));
-  // sanity check: after realignment, everything fits in allocated space
-  assert(vector+bytes <= BX_MEM_THIS actual_vector+bytes+test_mask);
-  return vector;
 }
 
 BX_MEM_C::~BX_MEM_C()
 {
-#if BX_LARGE_RAMFILE
-  if (overflow_file)
-    fclose(BX_MEM_THIS overflow_file);
-#endif
-
   cleanup_memory();
 }
 
-void BX_MEM_C::init_memory(Bit64u guest, Bit64u host)
+void BX_MEM_C::init_memory(Bit64u guest, Bit64u host, Bit32u block_size)
 {
-  unsigned i, idx;
+  unsigned idx, i;
 
-  BX_DEBUG(("Init $Id: misc_mem.cc 14290 2021-06-24 17:03:09Z vruppert $"));
+  BX_MEMORY_STUB_C::init_memory(guest, host, block_size);
 
-  // accept only memory size which is multiply of 1M
-  BX_ASSERT((host & 0xfffff) == 0);
-  BX_ASSERT((guest & 0xfffff) == 0);
-
-  if (BX_MEM_THIS actual_vector != NULL) {
-    BX_INFO(("freeing existing memory vector"));
-    delete [] BX_MEM_THIS actual_vector;
-    BX_MEM_THIS actual_vector = NULL;
-    BX_MEM_THIS vector = NULL;
-    BX_MEM_THIS blocks = NULL;
-  }
-  BX_MEM_THIS vector = alloc_vector_aligned(host + BIOSROMSZ + EXROMSIZE + 4096, BX_MEM_VECTOR_ALIGN);
-  BX_INFO(("allocated memory at %p. after alignment, vector=%p",
-        BX_MEM_THIS actual_vector, BX_MEM_THIS vector));
-
-  BX_MEM_THIS len = guest;
-  BX_MEM_THIS allocated = host;
-  BX_MEM_THIS rom = &BX_MEM_THIS vector[host];
-  BX_MEM_THIS bogus = &BX_MEM_THIS vector[host + BIOSROMSZ + EXROMSIZE];
-  memset(BX_MEM_THIS rom, 0xff, BIOSROMSZ + EXROMSIZE + 4096);
-
-  // block must be large enough to fit num_blocks in 32-bit
-  BX_ASSERT((BX_MEM_THIS len / BX_MEM_BLOCK_LEN) <= 0xffffffff);
-
-  Bit32u num_blocks = (Bit32u)(BX_MEM_THIS len / BX_MEM_BLOCK_LEN);
-  BX_INFO(("%.2fMB", (float)(BX_MEM_THIS len / (1024.0*1024.0))));
-  BX_INFO(("mem block size = 0x%08x, blocks=%u", BX_MEM_BLOCK_LEN, num_blocks));
-  BX_MEM_THIS blocks = new Bit8u* [num_blocks];
-  if (0) {
-    // all guest memory is allocated, just map it
-    for (idx = 0; idx < num_blocks; idx++) {
-      BX_MEM_THIS blocks[idx] = BX_MEM_THIS vector + (idx * BX_MEM_BLOCK_LEN);
-    }
-    BX_MEM_THIS used_blocks = num_blocks;
-  }
-  else {
-    // host cannot allocate all requested guest memory
-    for (idx = 0; idx < num_blocks; idx++) {
-      BX_MEM_THIS blocks[idx] = NULL;
-    }
-    BX_MEM_THIS used_blocks = 0;
-  }
+  BX_MEM_THIS smram_available = false;
+  BX_MEM_THIS smram_enable = false;
+  BX_MEM_THIS smram_restricted = false;
 
   BX_MEM_THIS memory_handlers = new struct memory_handler_struct *[BX_MEM_HANDLERS];
   for (idx = 0; idx < BX_MEM_HANDLERS; idx++)
     BX_MEM_THIS memory_handlers[idx] = NULL;
 
   BX_MEM_THIS pci_enabled = SIM->get_param_bool(BXPN_PCI_ENABLED)->get();
-  BX_MEM_THIS bios_write_enabled = 0;
+  BX_MEM_THIS bios_write_enabled = false;
   BX_MEM_THIS bios_rom_addr = 0xffff0000;
   BX_MEM_THIS flash_type = 0;
   BX_MEM_THIS flash_status = 0x80;
   BX_MEM_THIS flash_wsm_state = FLASH_READ_ARRAY;
-
-  BX_MEM_THIS smram_available = 0;
-  BX_MEM_THIS smram_enable = 0;
-  BX_MEM_THIS smram_restricted = 0;
+  BX_MEM_THIS flash_modified = false;
 
   for (i = 0; i < 65; i++)
-    BX_MEM_THIS rom_present[i] = 0;
+    BX_MEM_THIS rom_present[i] = false;
   for (i = 0; i <= BX_MEM_AREA_F0000; i++) {
-    BX_MEM_THIS memory_type[i][0] = 0;
-    BX_MEM_THIS memory_type[i][1] = 0;
+    BX_MEM_THIS memory_type[i][0] = false;
+    BX_MEM_THIS memory_type[i][1] = false;
   }
 
   BX_MEM_THIS register_state();
 }
 
-#if BX_LARGE_RAMFILE
-void BX_MEM_C::read_block(Bit32u block)
-{
-  const Bit64u block_address = ((Bit64u)block)*BX_MEM_BLOCK_LEN;
-
-  if (fseeko64(BX_MEM_THIS overflow_file, block_address, SEEK_SET))
-    BX_PANIC(("FATAL ERROR: Could not seek to 0x" FMT_LL "x in memory overflow file!", block_address));
-
-  // We could legitimately get an EOF condition if we are reading the last bit of memory.ram
-  if ((fread(BX_MEM_THIS blocks[block], BX_MEM_BLOCK_LEN, 1, BX_MEM_THIS overflow_file) != 1) && 
-      (!feof(BX_MEM_THIS overflow_file))) 
-    BX_PANIC(("FATAL ERROR: Could not read from 0x" FMT_LL "x in memory overflow file!", block_address)); 
-}
-#endif
-
-void BX_MEM_C::allocate_block(Bit32u block)
-{
-  const Bit32u max_blocks = (Bit32u)(BX_MEM_THIS allocated / BX_MEM_BLOCK_LEN);
-
-#if BX_LARGE_RAMFILE
-  /* 
-   * Match block to vector address
-   * First, see if there is any spare host memory blocks we can still freely allocate
-   */
-  if (BX_MEM_THIS used_blocks >= max_blocks) {
-    Bit32u original_replacement_block = BX_MEM_THIS next_swapout_idx;
-    // Find a block to replace
-    bool used_for_tlb;
-    Bit8u *buffer;
-    do {
-      do {
-        // Wrap if necessary
-        if (++(BX_MEM_THIS next_swapout_idx)==((BX_MEM_THIS len)/BX_MEM_BLOCK_LEN))
-          BX_MEM_THIS next_swapout_idx = 0;
-        if (BX_MEM_THIS next_swapout_idx == original_replacement_block)
-          BX_PANIC(("FATAL ERROR: Insufficient working RAM, all blocks are currently used for TLB entries!"));
-        buffer = BX_MEM_THIS blocks[BX_MEM_THIS next_swapout_idx];
-      } while ((!buffer) || (buffer == BX_MEM_C::swapped_out));
-
-      used_for_tlb = false;
-      // tlb buffer check loop
-      const Bit8u* buffer_end = buffer+BX_MEM_BLOCK_LEN;
-      // Don't replace it if any CPU is using it as a TLB entry
-      for (int i=0; i<BX_SMP_PROCESSORS && !used_for_tlb;i++)
-        used_for_tlb = BX_CPU(i)->check_addr_in_tlb_buffers(buffer, buffer_end);
-    } while (used_for_tlb);
-    // Flush the block to be replaced
-    bx_phy_address address = ((bx_phy_address)BX_MEM_THIS next_swapout_idx)*BX_MEM_BLOCK_LEN;
-    // Create overflow file if it does not currently exist.
-    if (!BX_MEM_THIS overflow_file) {
-      BX_MEM_THIS overflow_file = tmpfile64();
-      if (!BX_MEM_THIS overflow_file)
-        BX_PANIC(("Unable to allocate memory overflow file"));
-    }
-    // Write swapped out block
-    if (fseeko64(BX_MEM_THIS overflow_file, address, SEEK_SET))
-      BX_PANIC(("FATAL ERROR: Could not seek to 0x" FMT_PHY_ADDRX " in overflow file!", address)); 
-    if (1 != fwrite (BX_MEM_THIS blocks[BX_MEM_THIS next_swapout_idx], BX_MEM_BLOCK_LEN, 1, BX_MEM_THIS overflow_file))
-      BX_PANIC(("FATAL ERROR: Could not write at 0x" FMT_PHY_ADDRX " in overflow file!", address));
-    // Mark swapped out block
-    BX_MEM_THIS blocks[BX_MEM_THIS next_swapout_idx] = BX_MEM_C::swapped_out;
-    BX_MEM_THIS blocks[block] = buffer;
-    read_block(block);
-    BX_DEBUG(("allocate_block: block=0x%x, replaced 0x%x", block, BX_MEM_THIS next_swapout_idx));
-  }
-  else {
-    BX_MEM_THIS blocks[block] = BX_MEM_THIS vector + (BX_MEM_THIS used_blocks++ * BX_MEM_BLOCK_LEN);
-    BX_DEBUG(("allocate_block: block=0x%x used 0x%x of 0x%x",
-          block, BX_MEM_THIS used_blocks, max_blocks));
-  }
-#else
-  // Legacy default allocator
-  if (BX_MEM_THIS used_blocks >= max_blocks) {
-    BX_PANIC(("FATAL ERROR: all available memory is already allocated !"));
-  }
-  else {
-    BX_MEM_THIS blocks[block] = BX_MEM_THIS vector + (BX_MEM_THIS used_blocks * BX_MEM_BLOCK_LEN);
-    BX_MEM_THIS used_blocks++;
-  }
-  BX_DEBUG(("allocate_block: used_blocks=0x%x of 0x%x", BX_MEM_THIS used_blocks, max_blocks));
-#endif
-}
-
 #if BX_LARGE_RAMFILE
 // The blocks in RAM must also be flushed to the save file.
 void ramfile_save_handler(void *devptr, FILE *fp)
 {
-  for (Bit32u idx = 0; idx < (BX_MEM(0)->len / BX_MEM_BLOCK_LEN); idx++) {
+  for (Bit32u idx = 0; idx < (BX_MEM(0)->len / BX_MEM_THIS block_size); idx++) {
     if ((BX_MEM(0)->blocks[idx]) && (BX_MEM(0)->blocks[idx] != BX_MEM(0)->swapped_out))
     {
-      bx_phy_address address = ((bx_phy_address)idx)*BX_MEM_BLOCK_LEN;
+      bx_phy_address address = bx_phy_address(idx) * BX_MEM_THIS block_size;
       if (fseeko64(fp, address, SEEK_SET))
-        BX_PANIC(("FATAL ERROR: Could not seek to 0x" FMT_PHY_ADDRX " in overflow file!", address)); 
-      if (1 != fwrite (BX_MEM(0)->blocks[idx], BX_MEM_BLOCK_LEN, 1, fp))
+        BX_PANIC(("FATAL ERROR: Could not seek to 0x" FMT_PHY_ADDRX " in overflow file!", address));
+      if (1 != fwrite (BX_MEM(0)->blocks[idx], BX_MEM_THIS block_size, 1, fp))
         BX_PANIC(("FATAL ERROR: Could not write at 0x" FMT_PHY_ADDRX " in overflow file!", address));
     }
   }
@@ -269,7 +101,10 @@ void ramfile_save_handler(void *devptr, FILE *fp)
 // Note: This must be called before the memory file save handler is called.
 Bit64s memory_param_save_handler(void *devptr, bx_param_c *param)
 {
+  char imgname[BX_PATHNAME_LEN];
+  char path[BX_PATHNAME_LEN+1];
   const char *pname = param->get_name();
+
   if (! strncmp(pname, "blk", 3)) {
     Bit32u blk_index = atoi(pname + 3);
     if (! BX_MEM(0)->blocks[blk_index])
@@ -281,15 +116,32 @@ Bit64s memory_param_save_handler(void *devptr, bx_param_c *param)
 #endif
     // Return the block offset into the array
     Bit32u val = (Bit32u) (BX_MEM(0)->blocks[blk_index] - BX_MEM(0)->vector);
-    if ((val & (BX_MEM_BLOCK_LEN-1)) == 0)
-       return val / BX_MEM_BLOCK_LEN;
+    if ((val & (BX_MEM_THIS block_size-1)) == 0)
+       return val / BX_MEM_THIS block_size;
+  } else if (!strcmp(pname, "flash_data")) {
+    bool ret = false;
+    if (BX_MEM_THIS flash_modified) {
+      param->get_param_path(imgname, BX_PATHNAME_LEN);
+      if (!strncmp(imgname, "bochs.", 6)) {
+        strcpy(imgname, imgname+6);
+      }
+      if (SIM->get_param_string(BXPN_RESTORE_PATH)->isempty()) {
+        return 0;
+      }
+      sprintf(path, "%s/%s", SIM->get_param_string(BXPN_RESTORE_PATH)->getptr(), imgname);
+      ret = BX_MEM_THIS save_flash_data(path);
+    }
+    return ret;
   }
   return -1;
 }
 
 void memory_param_restore_handler(void *devptr, bx_param_c *param, Bit64s val)
 {
+  char imgname[BX_PATHNAME_LEN];
+  char path[BX_PATHNAME_LEN+1];
   const char *pname = param->get_name();
+
   if (! strncmp(pname, "blk", 3)) {
     Bit32u blk_index = atoi(pname + 3);
 #if BX_LARGE_RAMFILE
@@ -302,10 +154,22 @@ void memory_param_restore_handler(void *devptr, bx_param_c *param, Bit64s val)
         BX_MEM(0)->blocks[blk_index] = NULL;
         return;
       }
-      BX_MEM(0)->blocks[blk_index] = BX_MEM(0)->vector + val * BX_MEM_BLOCK_LEN;
+      BX_MEM(0)->blocks[blk_index] = BX_MEM(0)->vector + val * BX_MEM_THIS block_size;
 #if BX_LARGE_RAMFILE
       BX_MEM(0)->read_block(blk_index);
 #endif
+  } else if (!strcmp(pname, "flash_data")) {
+    if (BX_MEM_THIS flash_modified && val) {
+      param->get_param_path(imgname, BX_PATHNAME_LEN);
+      if (!strncmp(imgname, "bochs.", 6)) {
+        strcpy(imgname, imgname+6);
+      }
+      if (SIM->get_param_string(BXPN_RESTORE_PATH)->isempty()) {
+        return;
+      }
+      sprintf(path, "%s/%s", SIM->get_param_string(BXPN_RESTORE_PATH)->getptr(), imgname);
+      BX_MEM_THIS load_flash_data(path);
+    }
   }
 }
 
@@ -314,7 +178,7 @@ void BX_MEM_C::register_state()
   char param_name[15];
 
   bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "memory", "Memory State");
-  Bit32u num_blocks = (Bit32u)(BX_MEM_THIS len / BX_MEM_BLOCK_LEN);
+  Bit32u num_blocks = (Bit32u)(BX_MEM_THIS len / BX_MEM_THIS block_size);
 #if BX_LARGE_RAMFILE
   bx_shadow_filedata_c *ramfile = new bx_shadow_filedata_c(list, "ram", &(BX_MEM_THIS overflow_file));
   ramfile->set_sr_handlers(this, ramfile_save_handler, (filedata_restore_handler)NULL);
@@ -340,35 +204,87 @@ void BX_MEM_C::register_state()
   }
   BXRS_HEX_PARAM_FIELD(list, flash_status, BX_MEM_THIS flash_status);
   BXRS_DEC_PARAM_FIELD(list, flash_wsm_state, BX_MEM_THIS flash_wsm_state);
+  BXRS_PARAM_BOOL(list, flash_modified, BX_MEM_THIS flash_modified);
+  bx_param_bool_c *flash_data = new bx_param_bool_c(list, "flash_data", "", "", false);
+  flash_data->set_sr_handlers(this, memory_param_save_handler, memory_param_restore_handler);
 }
 
 void BX_MEM_C::cleanup_memory()
 {
-  unsigned idx;
-
-  if (BX_MEM_THIS vector != NULL) {
-    delete [] BX_MEM_THIS actual_vector;
-    BX_MEM_THIS actual_vector = NULL;
-    BX_MEM_THIS vector = NULL;
-    BX_MEM_THIS rom = NULL;
-    BX_MEM_THIS bogus = NULL;
-    delete [] BX_MEM_THIS blocks;
-    BX_MEM_THIS blocks = 0;
-    BX_MEM_THIS used_blocks = 0;
-    if (BX_MEM_THIS memory_handlers != NULL) {
-      for (idx = 0; idx < BX_MEM_HANDLERS; idx++) {
-        struct memory_handler_struct *memory_handler = BX_MEM_THIS memory_handlers[idx];
-        struct memory_handler_struct *prev = NULL;
-        while (memory_handler) {
-          prev = memory_handler;
-          memory_handler = memory_handler->next;
-          delete prev;
-        }
-      }
-      delete [] BX_MEM_THIS memory_handlers;
-      BX_MEM_THIS memory_handlers = NULL;
+  if (BX_MEM_THIS flash_modified) {
+    bx_param_string_c *flash_data = SIM->get_param_string(BXPN_ROM_FLASH_DATA);
+    if (!flash_data->isempty()) {
+      BX_MEM_THIS save_flash_data(flash_data->getptr());
     }
+    BX_MEM_THIS flash_modified = false;
   }
+
+  BX_MEMORY_STUB_C::cleanup_memory();
+
+  if (BX_MEM_THIS memory_handlers != NULL) {
+    for (unsigned idx = 0; idx < BX_MEM_HANDLERS; idx++) {
+      struct memory_handler_struct *memory_handler = BX_MEM_THIS memory_handlers[idx];
+      struct memory_handler_struct *prev = NULL;
+      while (memory_handler) {
+        prev = memory_handler;
+        memory_handler = memory_handler->next;
+        delete prev;
+      }
+    }
+    delete [] BX_MEM_THIS memory_handlers;
+    BX_MEM_THIS memory_handlers = NULL;
+  }
+}
+
+bool BX_MEM_C::load_flash_data(const char *path)
+{
+  int size = 0, offset = 0;
+
+  int fd = open(path, O_RDONLY
+#ifdef O_BINARY
+                | O_BINARY
+#endif
+                );
+  if (fd >= 0) {
+    if (BX_MEM_THIS flash_type == 2) {
+      offset = 0x8000; // 28F002BC-T
+      size = 0x4000;
+    } else if (BX_MEM_THIS flash_type == 1) {
+      offset = 0x4000; // 28F001BX-T
+      size = 0x2000;
+    }
+    if (size > 0) {
+      read(fd, &BX_MEM_THIS rom[BIOSROMSZ - offset], size);
+    }
+    close(fd);
+  }
+  return (fd >= 0);
+}
+
+bool BX_MEM_C::save_flash_data(const char *path)
+{
+  int offset = 0, size = 0;
+
+  int fd = open(path, O_WRONLY | O_CREAT | O_TRUNC
+#ifdef O_BINARY
+                | O_BINARY
+#endif
+                , S_IWUSR | S_IRUSR | S_IWGRP | S_IRGRP
+                );
+  if (fd >= 0) {
+    if (BX_MEM_THIS flash_type == 2) {
+      offset = 0x8000; // 28F002BC-T
+      size = 0x4000;
+    } else if (BX_MEM_THIS flash_type == 1) {
+      offset = 0x4000; // 28F001BX-T
+      size = 0x2000;
+    }
+    if (size > 0) {
+      write(fd, &BX_MEM_THIS rom[BIOSROMSZ - offset], size);
+    }
+    close(fd);
+  }
+  return (fd >= 0);
 }
 
 //
@@ -382,7 +298,7 @@ void BX_MEM_C::load_ROM(const char *path, bx_phy_address romaddress, Bit8u type)
   struct stat stat_buf;
   int fd, ret, i, start_idx, end_idx;
   unsigned long size, max_size, offset;
-  bool is_bochs_bios = 0;
+  bool is_bochs_bios = false;
 
   if (*path == '\0') {
     if (type == 2) {
@@ -448,7 +364,7 @@ void BX_MEM_C::load_ROM(const char *path, bx_phy_address romaddress, Bit8u type)
     }
     offset = romaddress & BIOS_MASK;
     if ((romaddress & 0xf0000) < 0xf0000) {
-      BX_MEM_THIS rom_present[64] = 1;
+      BX_MEM_THIS rom_present[64] = true;
     }
     BX_MEM_THIS bios_rom_addr = (Bit32u)romaddress;
     is_bochs_bios = ((strstr(path, "BIOS-bochs-latest") != NULL) ||
@@ -490,7 +406,7 @@ void BX_MEM_C::load_ROM(const char *path, bx_phy_address romaddress, Bit8u type)
         BX_PANIC(("ROM: address space 0x%x already in use", (i * 2048) + 0xc0000));
         return;
       } else {
-        BX_MEM_THIS rom_present[i] = 1;
+        BX_MEM_THIS rom_present[i] = true;
       }
     }
   }
@@ -527,6 +443,27 @@ void BX_MEM_C::load_ROM(const char *path, bx_phy_address romaddress, Bit8u type)
                         (unsigned) romaddress,
                         (unsigned) stat_buf.st_size,
                          path));
+  if ((type == 0) && !is_bochs_bios && (BX_MEM_THIS flash_type > 0)) {
+    bx_param_string_c *flash_data = SIM->get_param_string(BXPN_ROM_FLASH_DATA);
+    if (!flash_data->isempty()) {
+      if (!BX_MEM_THIS load_flash_data(flash_data->getptr())) {
+        Bit32u foffset = 0, fsize = 0;
+        if (BX_MEM_THIS flash_type == 2) {
+          foffset = 0x8000; // 28F002BC-T
+          fsize = 0x4000;
+        } else if (BX_MEM_THIS flash_type == 1) {
+          foffset = 0x4000; // 28F001BX-T
+          fsize = 0x2000;
+        }
+        if (fsize > 0) {
+          for (i = 0; i < (int)fsize; i++) {
+            BX_MEM_THIS rom[BIOSROMSZ - foffset + i] = 0xff;
+          }
+          BX_MEM_THIS flash_modified = true;
+        }
+      }
+    }
+  }
 }
 
 void BX_MEM_C::load_RAM(const char *path, bx_phy_address ramaddress)
@@ -578,18 +515,18 @@ bool BX_MEM_C::dbg_fetch_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, B
 {
   bx_phy_address a20addr = A20ADDR(addr);
   struct memory_handler_struct *memory_handler = NULL;
-  bool ret = 1, use_memory_handler = 0, use_smram = 0;
+  bool ret = true, use_memory_handler = false, use_smram = false;
 
   bool is_bios = (a20addr >= (bx_phy_address)BX_MEM_THIS bios_rom_addr);
 #if BX_PHY_ADDRESS_LONG
-  if (a20addr > BX_CONST64(0xffffffff)) is_bios = 0;
+  if (a20addr > BX_CONST64(0xffffffff)) is_bios = false;
 #endif
 
   if ((a20addr >= 0x000a0000 && a20addr < 0x000c0000) && BX_MEM_THIS smram_available)
   {
     // SMRAM memory space
     if (BX_MEM_THIS smram_enable || (cpu->smm_mode() && !BX_MEM_THIS smram_restricted))
-      use_smram = 1;
+      use_smram = true;
   }
 
   memory_handler = BX_MEM_THIS memory_handlers[a20addr >> 20];
@@ -597,7 +534,7 @@ bool BX_MEM_C::dbg_fetch_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, B
     if (memory_handler->begin <= a20addr && memory_handler->end >= a20addr)
     {
       if (!use_smram) {
-        use_memory_handler = 1;
+        use_memory_handler = true;
         break;
       }
     }
@@ -612,7 +549,7 @@ bool BX_MEM_C::dbg_fetch_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, B
     else if (BX_MEM_THIS pci_enabled && (a20addr >= 0x000c0000 && a20addr < 0x00100000)) {
       unsigned area = (unsigned)(a20addr >> 14) & 0x0f;
       if (area > BX_MEM_AREA_F0000) area = BX_MEM_AREA_F0000;
-      if (BX_MEM_THIS memory_type[area][0] == 0) {
+      if (BX_MEM_THIS memory_type[area][0] == false) {
         // Read from ROM
         if ((a20addr & 0xfffe0000) == 0x000e0000) {
           // last 128K of BIOS ROM mapped to 0xE0000-0xFFFFF
@@ -646,7 +583,7 @@ bool BX_MEM_C::dbg_fetch_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, B
 #if BX_PHY_ADDRESS_LONG
     else if (a20addr > BX_CONST64(0xffffffff)) {
       *buf = 0xff;
-      ret = 0; // error, beyond limits of memory
+      ret = false; // error, beyond limits of memory
     }
 #endif
     else if (is_bios)
@@ -660,7 +597,7 @@ bool BX_MEM_C::dbg_fetch_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, B
     else
     {
       *buf = 0xff;
-      ret = 0; // error, beyond limits of memory
+      ret = false; // error, beyond limits of memory
     }
     buf++;
     a20addr++;
@@ -673,22 +610,22 @@ bool BX_MEM_C::dbg_set_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, Bit
 {
   bx_phy_address a20addr = A20ADDR(addr);
   struct memory_handler_struct *memory_handler = NULL;
-  bool use_memory_handler = 0, use_smram = 0;
+  bool use_memory_handler = false, use_smram = false;
 
   if ((a20addr + len - 1) > BX_MEM_THIS len) {
-    return(0); // error, beyond limits of memory
+    return false; // error, beyond limits of memory
   }
 
   bool is_bios = (a20addr >= (bx_phy_address)BX_MEM_THIS bios_rom_addr);
 #if BX_PHY_ADDRESS_LONG
-  if (a20addr > BX_CONST64(0xffffffff)) is_bios = 0;
+  if (a20addr > BX_CONST64(0xffffffff)) is_bios = false;
 #endif
 
   if ((a20addr >= 0x000a0000 && a20addr < 0x000c0000) && BX_MEM_THIS smram_available)
   {
     // SMRAM memory space
     if (BX_MEM_THIS smram_enable || (cpu->smm_mode() && !BX_MEM_THIS smram_restricted))
-      use_smram = 1;
+      use_smram = true;
   }
 
   memory_handler = BX_MEM_THIS memory_handlers[a20addr >> 20];
@@ -696,7 +633,7 @@ bool BX_MEM_C::dbg_set_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, Bit
     if (memory_handler->begin <= a20addr && memory_handler->end >= a20addr)
     {
       if (!use_smram) {
-        use_memory_handler = 1;
+        use_memory_handler = true;
         break;
       }
     }
@@ -711,7 +648,7 @@ bool BX_MEM_C::dbg_set_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, Bit
     else if (BX_MEM_THIS pci_enabled && (a20addr >= 0x000c0000 && a20addr < 0x00100000)) {
       unsigned area = (unsigned)(a20addr >> 14) & 0x0f;
       if (area > BX_MEM_AREA_F0000) area = BX_MEM_AREA_F0000;
-      if (BX_MEM_THIS memory_type[area][1] == 1) {
+      if (BX_MEM_THIS memory_type[area][1] == true) {
         // Write to ShadowRAM
         *(BX_MEM_THIS get_vector(a20addr)) = *buf;
       } else {
@@ -726,62 +663,17 @@ bool BX_MEM_C::dbg_set_mem(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, Bit
     buf++;
     a20addr++;
   }
-  return(1);
-}
-
-bool BX_MEM_C::dbg_crc32(bx_phy_address addr1, bx_phy_address addr2, Bit32u *crc)
-{
-  *crc = 0;
-  if (addr1 > addr2)
-    return(0);
-
-  if (addr2 >= BX_MEM_THIS len)
-    return(0); // error, specified address past last phy mem addr
-
-  unsigned len = 1 + addr2 - addr1;
-
-  // do not cross 4K boundary
-  while(1) { 
-    unsigned remainsInPage = 0x1000 - (addr1 & 0xfff);
-    unsigned access_length = (len < remainsInPage) ? len : remainsInPage;
-    *crc = crc32(BX_MEM_THIS get_vector(addr1), access_length);
-    addr1 += access_length;
-    len -= access_length;
-  }
-
-  return(1);
+  return true;
 }
 #endif
 
-//
-// Return a host address corresponding to the guest physical memory
-// address (with A20 already applied), given that the calling
-// code will perform an 'op' operation.  This address will be
-// used for direct access to guest memory.
-// Values of 'op' are { BX_READ, BX_WRITE, BX_EXECUTE, BX_RW }.
-//
-// The other assumption is that the calling code _only_ accesses memory
-// directly within the page that encompasses the address requested.
-//
-
-//
-// Memory map inside the 1st megabyte:
-//
-// 0x00000 - 0x7ffff    DOS area (512K)
-// 0x80000 - 0x9ffff    Optional fixed memory hole (128K)
-// 0xa0000 - 0xbffff    Standard PCI/ISA Video Mem / SMMRAM (128K)
-// 0xc0000 - 0xdffff    Expansion Card BIOS and Buffer Area (128K)
-// 0xe0000 - 0xeffff    Lower BIOS Area (64K)
-// 0xf0000 - 0xfffff    Upper BIOS Area (64K)
-//
-
 Bit8u *BX_MEM_C::getHostMemAddr(BX_CPU_C *cpu, bx_phy_address addr, unsigned rw)
 {
   bx_phy_address a20addr = A20ADDR(addr);
 
   bool is_bios = (a20addr >= (bx_phy_address)BX_MEM_THIS bios_rom_addr);
 #if BX_PHY_ADDRESS_LONG
-  if (a20addr > BX_CONST64(0xffffffff)) is_bios = 0;
+  if (a20addr > BX_CONST64(0xffffffff)) is_bios = false;
 #endif
 
   bool write = rw & 1;
@@ -822,7 +714,7 @@ Bit8u *BX_MEM_C::getHostMemAddr(BX_CPU_C *cpu, bx_phy_address addr, unsigned rw)
     else if (BX_MEM_THIS pci_enabled && (a20addr >= 0x000c0000 && a20addr < 0x00100000)) {
       unsigned area = (unsigned)(a20addr >> 14) & 0x0f;
       if (area > BX_MEM_AREA_F0000) area = BX_MEM_AREA_F0000;
-      if (BX_MEM_THIS memory_type[area][0] == 0) {
+      if (BX_MEM_THIS memory_type[area][0] == false) {
         // Read from ROM
         if ((a20addr & 0xfffe0000) == 0x000e0000) {
           // last 128K of BIOS ROM mapped to 0xE0000-0xFFFFF
@@ -902,9 +794,9 @@ BX_MEM_C::registerMemoryHandlers(void *param, memory_handler_t read_handler,
                 bx_phy_address begin_addr, bx_phy_address end_addr)
 {
   if (end_addr < begin_addr)
-    return 0;
+    return false;
   if (!read_handler) // allow NULL write and fetch handler
-    return 0;
+    return false;
   BX_INFO(("Register memory access handlers: 0x" FMT_PHY_ADDRX " - 0x" FMT_PHY_ADDRX, begin_addr, end_addr));
   for (Bit32u page_idx = (Bit32u)(begin_addr >> 20); page_idx <= (Bit32u)(end_addr >> 20); page_idx++) {
     Bit16u bitmap = 0xffff;
@@ -917,7 +809,7 @@ BX_MEM_C::registerMemoryHandlers(void *param, memory_handler_t read_handler,
     if (BX_MEM_THIS memory_handlers[page_idx] != NULL) {
       if ((bitmap & BX_MEM_THIS memory_handlers[page_idx]->bitmap) != 0) {
         BX_ERROR(("Register failed: overlapping memory handlers!"));
-        return 0;
+        return false;
       } else {
         bitmap |= BX_MEM_THIS memory_handlers[page_idx]->bitmap;
       }
@@ -933,13 +825,12 @@ BX_MEM_C::registerMemoryHandlers(void *param, memory_handler_t read_handler,
     memory_handler->end = end_addr;
     memory_handler->bitmap = bitmap;
   }
-  return 1;
+  return true;
 }
 
-  bool
-BX_MEM_C::unregisterMemoryHandlers(void *param, bx_phy_address begin_addr, bx_phy_address end_addr)
+bool BX_MEM_C::unregisterMemoryHandlers(void *param, bx_phy_address begin_addr, bx_phy_address end_addr)
 {
-  bool ret = 1;
+  bool ret = true;
   BX_INFO(("Memory access handlers unregistered: 0x" FMT_PHY_ADDRX " - 0x" FMT_PHY_ADDRX, begin_addr, end_addr));
   for (Bit32u page_idx = (Bit32u)(begin_addr >> 20); page_idx <= (Bit32u)(end_addr >> 20); page_idx++) {
     Bit16u bitmap = 0xffff;
@@ -961,7 +852,7 @@ BX_MEM_C::unregisterMemoryHandlers(void *param, bx_phy_address begin_addr, bx_ph
       memory_handler = memory_handler->next;
     }
     if (!memory_handler) {
-      ret = 0;  // we should have found it
+      ret = false;  // we should have found it
       continue; // anyway, try the other pages
     }
     if (prev)
@@ -975,16 +866,16 @@ BX_MEM_C::unregisterMemoryHandlers(void *param, bx_phy_address begin_addr, bx_ph
 
 void BX_MEM_C::enable_smram(bool enable, bool restricted)
 {
-  BX_MEM_THIS smram_available = 1;
+  BX_MEM_THIS smram_available = true;
   BX_MEM_THIS smram_enable = enable;
   BX_MEM_THIS smram_restricted = restricted;
 }
 
 void BX_MEM_C::disable_smram(void)
 {
-  BX_MEM_THIS smram_available  = 0;
-  BX_MEM_THIS smram_enable     = 0;
-  BX_MEM_THIS smram_restricted = 0;
+  BX_MEM_THIS smram_available  = false;
+  BX_MEM_THIS smram_enable     = false;
+  BX_MEM_THIS smram_restricted = false;
 }
 
 // check if SMRAM is aavailable for CPU data accesses
@@ -1056,6 +947,7 @@ void BX_MEM_C::flash_write(Bit32u addr, Bit8u data)
     BX_DEBUG(("flash write to ROM (address = 0x%08x, data = 0x%02x)", flash_addr, data));
     BX_MEM_THIS rom[addr] &= data;
     BX_MEM_THIS flash_wsm_state = FLASH_READ_STATUS;
+    BX_MEM_THIS flash_modified = true;
   } else {
     BX_DEBUG(("flash write command (address = 0x%08x, code = 0x%02x)", flash_addr, data));
     switch (data) {
@@ -1080,15 +972,17 @@ void BX_MEM_C::flash_write(Bit32u addr, Bit8u data)
           BX_MEM_THIS flash_status &= ~0xc0;
           BX_MEM_THIS flash_wsm_state = FLASH_ERASE;
           if ((BX_MEM_THIS flash_type == 1) &&
-              ((flash_addr = 0x1c000) || (flash_addr = 0x1d000))) {
+              ((flash_addr == 0x1c000) || (flash_addr == 0x1d000))) {
             for (i = 0; i < 0x1000; i++) {
               BX_MEM_THIS rom[addr + i] = 0xff;
             }
+            BX_MEM_THIS flash_modified = true;
           } else if ((BX_MEM_THIS flash_type == 2) &&
-                     ((flash_addr = 0x38000) || (flash_addr = 0x3a000))) {
+                     ((flash_addr == 0x38000) || (flash_addr == 0x3a000))) {
             for (i = 0; i < 0x2000; i++) {
               BX_MEM_THIS rom[addr + i] = 0xff;
             }
+            BX_MEM_THIS flash_modified = true;
           }
         } else if (BX_MEM_THIS flash_wsm_state == FLASH_ERASE_SUSP) {
           BX_MEM_THIS flash_status &= ~0x40;
@@ -1102,28 +996,3 @@ void BX_MEM_C::flash_write(Bit32u addr, Bit8u data)
     }
   }
 }
-
-#if BX_SUPPORT_MONITOR_MWAIT
-
-//
-// MONITOR/MWAIT - x86arch way to optimize idle loops in CPU
-//
-
-bool BX_MEM_C::is_monitor(bx_phy_address begin_addr, unsigned len)
-{
-  for (int i=0; i<BX_SMP_PROCESSORS;i++) {
-    if (BX_CPU(i)->is_monitor(begin_addr, len))
-      return 1;
-  }
-
-  return 0; // // this is NOT monitored page
-}
-
-void BX_MEM_C::check_monitor(bx_phy_address begin_addr, unsigned len)
-{
-  for (int i=0; i<BX_SMP_PROCESSORS;i++) {
-    BX_CPU(i)->check_monitor(begin_addr, len);
-  }
-}
-
-#endif
