@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pc_system.h 14096 2021-01-30 18:50:09Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2017  The Bochs Project
@@ -122,8 +122,8 @@ public:
 
   int register_timer_ticks(void* this_ptr, bx_timer_handler_t, Bit64u ticks,
                            bool continuous, bool active, const char *id);
-  void activate_timer_ticks(unsigned index, Bit64u instructions,
-                            bool continuous);
+  void activate_timer_ticks(unsigned index, Bit64u instructions, bool continuous);
+
   Bit64u time_usec();
   Bit64u time_nsec();
   Bit64u time_usec_sequential();
